@@ -1,5 +1,11 @@
 /* config.h:  a general config file */
 
+/* Define IN_UCD_SNMP_SOURCE if compiling inside the ucd-snmp source tree */
+#define IN_UCD_SNMP_SOURCE 1
+
+/* Our assigned enterprise number */
+#define ENTERPRISE_NUMBER 2021
+
 /* don't change these values! */
 #define SNMPV1      0xAAAA       /* readable by anyone */
 #define SNMPV2ANY   0xA000       /* V2 Any type (includes NoAuth) */
@@ -26,12 +32,22 @@
    and are not using the UC-Davis SNMP library. */
 #define UCD_SNMP_LIBRARY 1
 
-/* comment the next line if you do not want SNMPv2 party-based auth. */
-#define USE_V2PARTY_PROTOCOL 1
+/* define if you want to compile support for both authentication and
+   privacy support. */
+#undef SCAPI_AUTHPRIV
+
+/* define if you are using the MD5 code ...*/
+#undef USE_INTERNAL_MD5
+
+/* ... or define this if you're using openssl support */
+#undef USE_OPENSSL
 
 /* add in recent CMU library extensions (not complete) */
 #undef CMU_COMPATIBLE
 
+/* add in recent resource lock functions (not complete) */
+#undef _REENTRANT
+
 /* should "--" comments in mibs be a comment till the end of the line
    or also until another "--", the latter being the technically
    correct. */
@@ -43,6 +59,26 @@
 #define SNMP_ALWAYS_DEBUG 0        /* Always print debugging information and
                                       ignore the -D flag passed to the cmds */
 
+/* PERSISTENT_DIRECTORY: If defined, the library is capabile of saving
+   persisant information to this directory in the form of configuration
+   lines: PERSISTENT_DIRECTORY/NAME.persistent.conf */
+#define PERSISTENT_DIRECTORY "/var/snmp"
+
+/* PERSISTENT_MASK: the umask permissions to set up the persistent files with */
+#define PERSISTENT_MASK 077
+
+/* AGENT_DIRECTORY_MODE: the mode the agents should use to create
+   directories with. Since the data stored here is probably sensitive, it
+   probably should be read-only by root/administrator. */
+#define AGENT_DIRECTORY_MODE 0700
+
+/* MAX_PERSISTENT_BACKUPS:
+ *   The maximum number of persistent backups the library will try to
+ *   read from the persistent cache directory.  If an application fails to
+ *   close down successfully more than this number of times, data will be lost.
+ */
+#define MAX_PERSISTENT_BACKUPS 10
+
 @TOP@
 
 /* define the system type include file here */
@@ -55,6 +91,7 @@
 
 #undef SNMPLIBPATH
 #undef SNMPSHAREPATH
+#undef SNMPCONFPATH
 
 /* LOGFILE:  If defined it closes stdout/err/in and opens this in out/err's
    place.  (stdin is closed so that sh scripts won't wait for it) */
@@ -70,6 +107,9 @@
 /* Use libwrap to handle allow/deny hosts? */
 #undef USE_LIBWRAP
 
+/* Use dmalloc to do malloc debugging? */
+#undef HAVE_DMALLOC_H
+
 /* location of UNIX kernel */
 #define KERNEL_LOC "/vmunix"
 
@@ -87,13 +127,23 @@
 /* Where is the uname command */
 #define UNAMEPROG "/bin/uname"
 
+/* testing code sections. */
+#undef SNMP_TESTING_CODE 
+
 /* If you don't have root access don't exit upon kmem errors */
 #undef NO_ROOT_ACCESS
 
+/* If you don't want the agent to report on variables it doesn't have data for */
+#undef NO_DUMMY_VALUES
+
 /* Define if statfs takes 2 args and the second argument has
    type struct fs_data. [Ultrix] */
 #undef STAT_STATFS_FS_DATA
 
+/* Define if the TCP timer constants in <netinet/tcp_timer.h>
+   depend on the integer variable `hz'.  [FreeBSD 4.x] */
+#undef TCPTV_NEEDS_HZ
+
 @BOTTOM@
 
 /* define if you are using linux and /proc/net/dev has the compressed
@@ -106,6 +156,15 @@
 /* Use BSD 4.4 routing table entries? */
 #undef RTENTRY_4_4
 
+/* Does struct sigaction have a sa_sigaction field? */
+#undef STRUCT_SIGACTION_HAS_SA_SIGACTION
+
+/* Does struct sockaddr have a sa_len field? */
+#undef STRUCT_SOCKADDR_HAS_SA_LEN
+
+/* Does struct sockaddr have a sa_family2 field? */
+#undef STRUCT_SOCKADDR_HAS_SA_UNION_SA_GENERIC_SA_FAMILY2
+
 /* rtentry structure tests */
 #undef RTENTRY_RT_NEXT
 #undef STRUCT_RTENTRY_HAS_RT_DST
@@ -152,6 +211,9 @@
 /* vfsstat.f_frsize */
 #undef STRUCT_STATVFS_HAS_F_FRSIZE
 
+/* vfsstat.f_files */
+#undef STRUCT_STATVFS_HAS_F_FILES
+
 /* ifnet needs to have _KERNEL defined */
 #undef IFNET_NEEDS_KERNEL
 
@@ -252,7 +314,7 @@
 #ifdef netbsd1
 #define OSTYPE NETBSD1ID
 #endif
-#if defined(freebsd2) || defined(freebsd3)
+#ifdef freebsd2
 #define OSTYPE FREEBSDID
 #endif
 #if defined(irix6) || defined(irix5)
@@ -278,12 +340,12 @@
 /* Exec command to fix PROC problems */
 /* %s will be replaced by the process name in error */
 
-#define PROCFIXCMD "/usr/local/bin/perl /local/scripts/fixproc %s"
+/* #define PROCFIXCMD "/usr/bin/perl /local/scripts/fixproc %s" */
 
 /* Exec command to fix EXEC problems */
 /* %s will be replaced by the exec/script name in error */
 
-#define EXECFIXCMD "/usr/local/bin/perl /local/scripts/fixproc %s"
+/* #define EXECFIXCMD "/usr/bin/perl /local/scripts/fixproc %s" */
 
 /* Should exec output Cashing be used (speeds up things greatly), and
    if so, After how many seconds should the cache re-newed?  Note:
@@ -391,6 +453,7 @@
 #endif
 
 /* define if you have librpm and libdb */
+#undef HAVE_LIBDB
 #undef HAVE_LIBRPM
 
 /* define if you have gethostbyname */
@@ -402,19 +465,15 @@
 /* Not-to-be-compiled macros for use by configure only */
 #define config_require(x)
 #define config_arch_require(x,y)
-#define config_load_mib(x,y,z)
 #define config_parse_dot_conf(w,x,y,z)
 #define config_add_mib(x)
   
-#include <mib_module_config.h>
-
 #ifdef WIN32
 #define ENV_SEPARATOR ";"
 #define ENV_SEPARATOR_CHAR ';'
 #else
 #define ENV_SEPARATOR ":"
 #define ENV_SEPARATOR_CHAR ':'
-#define _CRTIMP
 #endif
 
 #include SYSTEM_INCLUDE_FILE
@@ -423,3 +482,5 @@
 #if defined(HAVE_NLIST) && defined(STRUCT_NLIST_HAS_N_VALUE) && !defined(DONT_USE_NLIST)
 #define CAN_USE_NLIST
 #endif
+
+#undef INET6
