@@ -49,7 +49,30 @@ void  Save_HR_Partition (int, int);
 
 void  Init_HR_Partition (void);
 int   Get_Next_HR_Partition (void);
-int header_hrpartition (struct variable *,oid *, int *, int, int *, WriteMethod **);
+int header_hrpartition (struct variable *,oid *, size_t *, int, size_t *, WriteMethod **);
+
+
+#define	HRPART_INDEX		1
+#define	HRPART_LABEL		2
+#define	HRPART_ID		3
+#define	HRPART_SIZE		4
+#define	HRPART_FSIDX		5
+
+struct variable4 hrpartition_variables[] = {
+    { HRPART_INDEX,     ASN_INTEGER, RONLY, var_hrpartition, 2, {1,1}},
+    { HRPART_LABEL,   ASN_OCTET_STR, RONLY, var_hrpartition, 2, {1,2}},
+    { HRPART_ID,      ASN_OCTET_STR, RONLY, var_hrpartition, 2, {1,3}},
+    { HRPART_SIZE,      ASN_INTEGER, RONLY, var_hrpartition, 2, {1,4}},
+    { HRPART_FSIDX,     ASN_INTEGER, RONLY, var_hrpartition, 2, {1,5}}
+};
+oid hrpartition_variables_oid[] = { 1,3,6,1,2,1,25,3,7 };
+
+
+void init_hr_partition( void )
+{
+    REGISTER_MIB("host/hr_partition", hrpartition_variables, variable4, hrpartition_variables_oid);
+}
+
 
 /*
   header_hrpartition(...
@@ -66,9 +89,9 @@ int header_hrpartition (struct variable *,oid *, int *, int, int *, WriteMethod
 int
 header_hrpartition(struct variable *vp,
 		   oid *name,
-		   int *length,
+		   size_t *length,
 		   int exact,
-		   int *var_len,
+		   size_t *var_len,
 		   WriteMethod **write_method)
 {
 #define HRPART_DISK_NAME_LENGTH		11
@@ -76,12 +99,10 @@ header_hrpartition(struct variable *vp,
     oid newname[MAX_OID_LEN];
     int part_idx, LowDiskIndex=-1, LowPartIndex = -1;
     int result;
-    char c_oid[SPRINT_MAX_LEN];
 
-    if (snmp_get_do_debugging()) {
-      sprint_objid (c_oid, name, *length);
-      DEBUGMSGTL(("host/hr_partition", "var_hrpartition: %s %d\n", c_oid, exact));
-    }
+    DEBUGMSGTL(("host/hr_partition", "var_hrpartition: "));
+    DEBUGMSGOID(("host/hr_partition", name, *length));
+    DEBUGMSG(("host/hr_partition"," %d\n", exact));
 
     memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
 	/* Find "next" partition entry */
@@ -102,7 +123,7 @@ header_hrpartition(struct variable *vp,
 		 *     (i.e. *length is too short to be a full instance)
 		 */
 
-    if (( snmp_oid_compare( vp->name, (int)vp->namelen, name, (int)vp->namelen ) == 0 ) &&
+    if (( snmp_oid_compare( vp->name, vp->namelen, name, vp->namelen ) == 0 ) &&
 	( *length > HRPART_DISK_NAME_LENGTH )) {
         LowDiskIndex = (name[HRPART_DISK_NAME_LENGTH] & ((1<<HRDEV_TYPE_SHIFT)-1));
 
@@ -119,7 +140,7 @@ header_hrpartition(struct variable *vp,
 	    break;
 	newname[HRPART_DISK_NAME_LENGTH] = (HRDEV_DISK << HRDEV_TYPE_SHIFT) + HRD_index;
 	newname[HRPART_ENTRY_NAME_LENGTH] = part_idx;
-        result = snmp_oid_compare(name, *length, newname, (int)vp->namelen + 2);
+        result = snmp_oid_compare(name, *length, newname, vp->namelen + 2);
         if (exact && (result == 0)) {
 	    Save_HR_Partition( HRD_index, part_idx );
 	    LowDiskIndex = HRD_index;
@@ -157,10 +178,9 @@ header_hrpartition(struct variable *vp,
     *write_method = 0;
     *var_len = sizeof(long);	/* default to 'long' results */
 
-    if (snmp_get_do_debugging()) {
-      sprint_objid (c_oid, name, *length);
-      DEBUGMSGTL(("host/hr_partition", "... get partition stats %s\n", c_oid));
-    }
+    DEBUGMSGTL(("host/hr_partition", "... get partition stats "));
+    DEBUGMSGOID(("host/hr_partition", name, *length));
+    DEBUGMSG(("host/hr_partition","\n"));
     return LowPartIndex;
 }
 
@@ -172,12 +192,12 @@ header_hrpartition(struct variable *vp,
 	 *********************/
 
 
-u_char	*
+u_char *
 var_hrpartition(struct variable *vp,
 		oid *name,
-		int *length,
+		size_t *length,
 		int exact,
-		int *var_len,
+		size_t *var_len,
 		WriteMethod **write_method)
 {
     int  part_idx;
@@ -210,7 +230,7 @@ var_hrpartition(struct variable *vp,
 	    long_return = Get_FSIndex( HRP_savedName );
 	    return (u_char *)&long_return;
 	default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_hrpartition\n", vp->magic));
     }
     return NULL;
 }
