@@ -6,6 +6,20 @@
 #include <config.h>
 #include <sys/types.h>
 #include <sys/param.h>
+#if TIME_WITH_SYS_TIME
+# ifdef WIN32
+#  include <sys/timeb.h>
+# else
+#  include <sys/time.h>
+# endif
+# include <time.h>
+#else
+# if HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
 #if HAVE_SYS_VM_H
 #include <sys/vm.h>
 #else
@@ -17,6 +31,9 @@
 #if HAVE_SYS_VMMETER_H
 #include <sys/vmmeter.h>
 #endif
+#if HAVE_VM_VM_PARAM_H
+#include <vm/vm_param.h>
+#endif
 #else
 #if HAVE_SYS_VMPARAM_H
 #include <sys/vmparam.h>
@@ -32,9 +49,22 @@
 #endif
 #endif /* vm/vm.h */
 #endif /* sys/vm.h */
-#ifdef HAVE_SYS_MBUF_H
+#if HAVE_SYS_POOL_H
+#define __POOL_EXPOSE
+#include <sys/pool.h>
+#endif
+#if HAVE_SYS_MBUF_H
 #include <sys/mbuf.h>
 #endif
+#if HAVE_SYS_SYSCTL_H
+#include <sys/sysctl.h>
+#if defined(CTL_HW) && defined(HW_PAGESIZE)
+#define USE_SYSCTL
+#endif
+#if defined(CTL_VM) && defined(VM_METER)
+#define USE_SYSCTL_VM
+#endif
+#endif
 
 #include "host_res.h"
 #include "hr_storage.h"
@@ -47,18 +77,15 @@
 #if HAVE_SYS_MNTTAB_H
 #include <sys/mnttab.h>
 #endif
+#if HAVE_SYS_STATVFS_H
+#include <sys/statvfs.h>
+#endif
 #if HAVE_SYS_VFS_H
 #include <sys/vfs.h>
 #endif
 #if HAVE_SYS_MOUNT_H
 #include <sys/mount.h>
 #endif
-#ifdef HAVE_SYS_MBUF_H
-#include <sys/mbuf.h>
-#endif
-#ifdef HAVE_SYS_PARAM_H
-#include <sys/param.h>
-#endif
 #ifdef HAVE_MACHINE_PARAM_H
 #include <machine/param.h>
 #endif
@@ -101,6 +128,10 @@ extern struct mntent *HRFS_entry;
 #define HRFS_mount	mnt_dir
 #define HRFS_statfs	statfs
 
+#endif
+
+#ifndef linux
+static int physmem, pagesize;
 #endif
 
 	/*********************
@@ -110,18 +141,71 @@ extern struct mntent *HRFS_entry;
 	 *********************/
 int Get_Next_HR_Store (void);
 void  Init_HR_Store (void);
-int header_hrstore (struct variable *,oid *, int *, int, int *, WriteMethod **);
-int header_hrstoreEntry (struct variable *,oid *, int *, int, int *, WriteMethod **);
+int header_hrstore (struct variable *,oid *, size_t *, int, size_t *, WriteMethod **);
+int header_hrstoreEntry (struct variable *,oid *, size_t *, int, size_t *, WriteMethod **);
 
 int linux_mem (int, int);
 
-void init_hr_storeage (void)
+#define	HRSTORE_MEMSIZE		1
+#define	HRSTORE_INDEX		2
+#define	HRSTORE_TYPE		3
+#define	HRSTORE_DESCR		4
+#define	HRSTORE_UNITS		5
+#define	HRSTORE_SIZE		6
+#define	HRSTORE_USED		7
+#define	HRSTORE_FAILS		8
+
+struct variable4 hrstore_variables[] = {
+    { HRSTORE_MEMSIZE,   ASN_INTEGER, RONLY, var_hrstore, 1, {2}},
+    { HRSTORE_INDEX,     ASN_INTEGER, RONLY, var_hrstore, 3, {3,1,1}},
+    { HRSTORE_TYPE,    ASN_OBJECT_ID, RONLY, var_hrstore, 3, {3,1,2}},
+    { HRSTORE_DESCR,   ASN_OCTET_STR, RONLY, var_hrstore, 3, {3,1,3}},
+    { HRSTORE_UNITS,     ASN_INTEGER, RONLY, var_hrstore, 3, {3,1,4}},
+    { HRSTORE_SIZE,      ASN_INTEGER, RONLY, var_hrstore, 3, {3,1,5}},
+    { HRSTORE_USED,      ASN_INTEGER, RONLY, var_hrstore, 3, {3,1,6}},
+    { HRSTORE_FAILS,     ASN_COUNTER, RONLY, var_hrstore, 3, {3,1,7}}
+};
+oid hrstore_variables_oid[] = { 1,3,6,1,2,1,25,2 };
+
+
+void init_hr_storage (void)
 {
-    auto_nlist(PHYSMEM_SYMBOL,0,0);
+#ifdef USE_SYSCTL
+    int mib [2];
+    size_t len;
+#endif
+
+#ifdef USE_SYSCTL
+    mib[0] = CTL_HW;
+    mib[1] = HW_PHYSMEM;
+    len = sizeof(physmem);
+    if (sysctl(mib, 2, &physmem, &len, NULL, 0) == -1) perror("sysctl: physmem");
+    mib[1] = HW_PAGESIZE;
+    len = sizeof(pagesize);
+    if (sysctl(mib, 2, &pagesize, &len, NULL, 0) == -1) perror("sysctl: pagesize");
+    physmem /= pagesize;
+#else	/* USE_SYSCTL */
+    auto_nlist(PHYSMEM_SYMBOL, (char *)&physmem, sizeof (physmem));
+#ifdef PGSHIFT
+    pagesize = 1 << PGSHIFT;
+#elif defined(PAGE_SHIFT)
+    pagesize = 1 << PAGE_SHIFT;
+#elif defined(PAGE_SIZE)
+    pagesize = PAGE_SIZE;
+#else
+  #ifndef linux
+    pagesize = PAGESIZE;
+  #endif
+#endif
+#endif	/* USE_SYSCTL */
 #ifdef TOTAL_MEMORY_SYMBOL
     auto_nlist(TOTAL_MEMORY_SYMBOL,0,0);
 #endif
+#ifdef MBSTAT_SYMBOL
     auto_nlist(MBSTAT_SYMBOL,0,0);
+#endif
+
+    REGISTER_MIB("host/hr_storage", hrstore_variables, variable4, hrstore_variables_oid);
 }
 
 /*
@@ -139,27 +223,25 @@ void init_hr_storeage (void)
 int
 header_hrstore(struct variable *vp,
 	       oid *name,
-	       int *length,
+	       size_t *length,
 	       int exact,
-	       int *var_len,
+	       size_t *var_len,
 	       WriteMethod **write_method)
 {
 #define HRSTORE_NAME_LENGTH	9
     oid newname[MAX_OID_LEN];
     int result;
-    char c_oid[SPRINT_MAX_LEN];
 
-    if (snmp_get_do_debugging()) {
-      sprint_objid (c_oid, name, *length);
-      DEBUGMSGTL(("host/hr_storage", "var_hrstore: %s %d\n", c_oid, exact));
-    }
+    DEBUGMSGTL(("host/hr_storage", "var_hrstore: "));
+    DEBUGMSGOID(("host/hr_storage", name, *length));
+    DEBUGMSG(("host/hr_storage"," %d\n", exact));
 
-    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
+    memcpy( (char *)newname,(char *)vp->name, vp->namelen * sizeof(oid));
     newname[HRSTORE_NAME_LENGTH] = 0;
-    result = snmp_oid_compare(name, *length, newname, (int)vp->namelen + 1);
+    result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
     if ((exact && (result != 0)) || (!exact && (result >= 0)))
         return(MATCH_FAILED);
-    memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
+    memcpy( (char *)name,(char *)newname, (vp->namelen + 1) * sizeof(oid));
     *length = vp->namelen + 1;
 
     *write_method = 0;
@@ -170,21 +252,19 @@ header_hrstore(struct variable *vp,
 int
 header_hrstoreEntry(struct variable *vp,
 		    oid *name,
-		    int *length,
+		    size_t *length,
 		    int exact,
-		    int *var_len,
+		    size_t *var_len,
 		    WriteMethod **write_method)
 {
 #define HRSTORE_ENTRY_NAME_LENGTH	11
     oid newname[MAX_OID_LEN];
     int storage_idx, LowIndex = -1;
     int result;
-    char c_oid[SPRINT_MAX_LEN];
 
-    if (snmp_get_do_debugging()) {
-      sprint_objid (c_oid, name, *length);
-      DEBUGMSGTL(("host/hr_storage", "var_hrstoreEntry: %s %d\n", c_oid, exact));
-    }
+    DEBUGMSGTL(("host/hr_storage", "var_hrstoreEntry: "));
+    DEBUGMSGOID(("host/hr_storage", name, *length));
+    DEBUGMSG(("host/hr_storage"," %d\n", exact));
 
     memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
 	/* Find "next" storage entry */
@@ -196,11 +276,9 @@ header_hrstoreEntry(struct variable *vp,
         if ( storage_idx == -1 )
 	    break;
 	newname[HRSTORE_ENTRY_NAME_LENGTH] = storage_idx;
-        if (snmp_get_do_debugging()) {
-          sprint_objid (c_oid, newname, *length);
-          DEBUGMSGTL(("host/hr_storage", "%s\n", c_oid));
-        }
-        result = snmp_oid_compare(name, *length, newname, (int)vp->namelen + 1);
+    DEBUGMSGOID(("host/hr_storage", newname, *length));
+    DEBUGMSG(("host/hr_storage","\n"));
+        result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
         if (exact && (result == 0)) {
 	    LowIndex = storage_idx;
 	    /* Save storage status information */
@@ -226,10 +304,9 @@ header_hrstoreEntry(struct variable *vp,
     *write_method = 0;
     *var_len = sizeof(long);	/* default to 'long' results */
 
-    if (snmp_get_do_debugging()) {
-      sprint_objid (c_oid, name, *length);
-      DEBUGMSGTL(("host/hr_storage", "... get storage stats %s\n", c_oid));
-    }
+    DEBUGMSGTL(("host/hr_storage", "... get storage stats "));
+    DEBUGMSGOID(("host/hr_storage", name, *length));
+    DEBUGMSG(("host/hr_storage","\n"));
     return LowIndex;
 }
 
@@ -242,7 +319,7 @@ int storage_type_len = sizeof(storage_type_id)/sizeof(storage_type_id[0]);
 	 *
 	 *********************/
 
-static char *hrs_descr[] = {
+static const char *hrs_descr[] = {
 	NULL,
 	"Real Memory",		/* HRS_TYPE_MEM */
 	"Swap Space",		/* HRS_TYPE_SWAP */
@@ -251,35 +328,35 @@ static char *hrs_descr[] = {
 
 
 
-u_char	*
+u_char *
 var_hrstore(struct variable *vp,
 	    oid *name,
-	    int *length,
+	    size_t *length,
 	    int exact,
-	    int *var_len,
+	    size_t *var_len,
 	    WriteMethod **write_method)
 {
     int store_idx=0;
 #ifndef linux
-    int physmem;
 #ifndef solaris2
-#ifdef TOTAL_MEMORY_SYMBOL
+#if defined(TOTAL_MEMORY_SYMBOL) || defined(USE_SYSCTL_VM)
     struct vmtotal memory_totals;
+#endif
+#if HAVE_SYS_POOL_H
+    struct pool mbpool, mclpool;
+    int i;
 #endif
     struct mbstat  mbstat;
-#endif
-#else
+#endif	/* solaris2 */
+#else	/* linux */
     struct stat  kc_buf;
-#endif
+#endif	/* linux */
     static char string[100];
     struct HRFS_statfs stat_buf;
 
     if ( vp->magic == HRSTORE_MEMSIZE ) {
         if (header_hrstore(vp, name, length, exact, var_len, write_method) == MATCH_FAILED )
 	    return NULL;
-#ifndef linux
-	auto_nlist(PHYSMEM_SYMBOL, (char *)&physmem, sizeof (int));
-#endif
     }
     else {
         
@@ -295,12 +372,23 @@ var_hrstore(struct variable *vp,
 	else switch ( store_idx ) {
 		case HRS_TYPE_MEM:
 		case HRS_TYPE_SWAP:
-#ifdef TOTAL_MEMORY_SYMBOL
+#ifdef USE_SYSCTL_VM
+			{   int mib[2];
+			    size_t len = sizeof(memory_totals);
+			    mib[0] = CTL_VM;
+			    mib[1] = VM_METER;
+			    sysctl(mib, 2, &memory_totals, &len, NULL, 0);
+			}
+#elif defined(TOTAL_MEMORY_SYMBOL)
 			auto_nlist(TOTAL_MEMORY_SYMBOL, (char *)&memory_totals, sizeof (struct vmtotal));
 #endif
 			break;
 		case HRS_TYPE_MBUF:
-			auto_nlist(MBSTAT_SYMBOL, (char *)&mbstat, sizeof (struct mbstat));
+#if HAVE_SYS_POOL_H
+			auto_nlist(MBPOOL_SYMBOL, (char *)&mbpool, sizeof(mbpool));
+			auto_nlist(MCLPOOL_SYMBOL, (char *)&mclpool, sizeof(mclpool));
+#endif
+			auto_nlist(MBSTAT_SYMBOL, (char *)&mbstat, sizeof (mbstat));
 			break;
 		default:
 			break;
@@ -313,15 +401,7 @@ var_hrstore(struct variable *vp,
     switch (vp->magic){
 	case HRSTORE_MEMSIZE:
 #ifndef linux
-#ifdef PGSHIFT
-	    long_return = physmem << PGSHIFT;
-#elif defined(PAGE_SHIFT)
-	    long_return = physmem << PAGE_SHIFT;
-#elif defined(PAGE_SIZE)
-	    long_return = physmem * PAGE_SIZE;
-#else
-	    long_return = physmem * PAGESIZE;
-#endif
+	    long_return = physmem * (pagesize / 1024);
 #else
 	    stat("/proc/kcore", &kc_buf);
 	    long_return = kc_buf.st_size/1024;	/* 4K too large ? */
@@ -367,7 +447,9 @@ var_hrstore(struct variable *vp,
 	    else switch ( store_idx ) {
 		case HRS_TYPE_MEM:
 		case HRS_TYPE_SWAP:
-#ifdef NBPG
+#ifdef USE_SYSCTL
+			long_return = pagesize;
+#elif defined(NBPG)
 			long_return = NBPG;
 #else
 			long_return = 1024;	/* Report in Kb */
@@ -381,6 +463,9 @@ var_hrstore(struct variable *vp,
 #endif
 			break;
 		default:
+#if NO_DUMMY_VALUES
+			return NULL;
+#endif
 			long_return = 1024;	/* As likely as any! */
 			break;
 	    }
@@ -390,7 +475,7 @@ var_hrstore(struct variable *vp,
 		long_return = stat_buf.f_blocks;
 	    else switch ( store_idx ) {
 #if !defined(linux) && !defined(solaris2)
-#ifdef TOTAL_MEMORY_SYMBOL
+#if defined(TOTAL_MEMORY_SYMBOL) || defined(USE_SYSCTL_VM)
 		case HRS_TYPE_MEM:
 			long_return = memory_totals.t_rm;
 			break;
@@ -399,13 +484,24 @@ var_hrstore(struct variable *vp,
 			break;
 #else
 		case HRS_TYPE_MEM:
-			long_return = physmem * PAGE_SIZE / 1024;
+			auto_nlist(PHYSMEM_SYMBOL, (char *)&physmem,
+                                   sizeof (int));
+			long_return = physmem * (pagesize / 1024);
 			break;
 		case HRS_TYPE_SWAP:
+#if NO_DUMMY_VALUES
+			return NULL;
+#endif
 			break;
 #endif
 		case HRS_TYPE_MBUF:
+#if HAVE_SYS_POOL_H
+			long_return = 0;
+			for (i = 0; i < sizeof(mbstat.m_mtypes)/sizeof(mbstat.m_mtypes[0]); i++)
+			    long_return += mbstat.m_mtypes[i];
+#else
 			long_return = mbstat.m_mbufs;
+#endif
 			break;
 #else	/* linux */
 #ifdef linux
@@ -416,6 +512,9 @@ var_hrstore(struct variable *vp,
 #endif
 #endif
 		default:
+#if NO_DUMMY_VALUES
+			return NULL;
+#endif
 			long_return = 1024;
 			break;
 	    }
@@ -425,7 +524,7 @@ var_hrstore(struct variable *vp,
 		long_return = (stat_buf.f_blocks - stat_buf.f_bfree);
 	    else switch ( store_idx ) {
 #if !defined(linux) && !defined(solaris2)
-#ifdef TOTAL_MEMORY_SYMBOL
+#if defined(TOTAL_MEMORY_SYMBOL) || defined(USE_SYSCTL_VM)
 		case HRS_TYPE_MEM:
 			long_return = memory_totals.t_arm;
 			break;
@@ -434,7 +533,12 @@ var_hrstore(struct variable *vp,
 			break;
 #endif
 		case HRS_TYPE_MBUF:
+#if HAVE_SYS_POOL_H
+			long_return = (mbpool.pr_nget - mbpool.pr_nput)*mbpool.pr_size
+				+ (mclpool.pr_nget - mclpool.pr_nput)*mclpool.pr_size;
+#else
 			long_return = mbstat.m_clusters - mbstat.m_clfree;	/* unlikely, but... */
+#endif
 			break;
 #else	/* linux */
 #ifdef linux
@@ -445,6 +549,9 @@ var_hrstore(struct variable *vp,
 #endif
 #endif
 		default:
+#if NO_DUMMY_VALUES
+			return NULL;
+#endif
 			long_return = 1024;
 			break;
 	    }
@@ -455,6 +562,9 @@ var_hrstore(struct variable *vp,
 	    else switch ( store_idx ) {
 		case HRS_TYPE_MEM:
 		case HRS_TYPE_SWAP:
+#if NO_DUMMY_VALUES
+			return NULL;
+#endif
 			long_return = 0;
 			break;
 #if !defined(linux) && !defined(solaris2)
@@ -463,12 +573,15 @@ var_hrstore(struct variable *vp,
 			break;
 #endif
 		default:
+#if NO_DUMMY_VALUES
+			return NULL;
+#endif
 			long_return = 0;
 			break;
 	    }
 	    return (u_char *)&long_return;
 	default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_hrstore\n", vp->magic));
     }
     return NULL;
 }
