@@ -16,6 +16,9 @@
 #include "host_res.h"
 #include "hr_swrun.h"
 #include "auto_nlist.h"
+#if solaris2
+#include "kernel_sunos5.h"
+#endif
 
 #include <sys/param.h>
 #include <ctype.h>
@@ -72,8 +75,8 @@
 void  Init_HR_SWRun (void);
 int   Get_Next_HR_SWRun (void);
 void  End_HR_SWRun (void);
-int header_hrswrun (struct variable *,oid *, int *, int, int *, WriteMethod **);
-int header_hrswrunEntry (struct variable *,oid *, int *, int, int *, WriteMethod **);
+int header_hrswrun (struct variable *,oid *, size_t *, int, size_t *, WriteMethod **);
+int header_hrswrunEntry (struct variable *,oid *, size_t *, int, size_t *, WriteMethod **);
 
 #ifndef linux
 static int LowProcIndex;
@@ -92,6 +95,40 @@ struct proc *proc_table;
 #endif
 int current_proc_entry;
 
+
+#define	HRSWRUN_OSINDEX		1
+
+#define	HRSWRUN_INDEX		2
+#define	HRSWRUN_NAME		3
+#define	HRSWRUN_ID		4
+#define	HRSWRUN_PATH		5
+#define	HRSWRUN_PARAMS		6
+#define	HRSWRUN_TYPE		7
+#define	HRSWRUN_STATUS		8
+
+#define	HRSWRUNPERF_CPU		9
+#define	HRSWRUNPERF_MEM		10
+
+struct variable4 hrswrun_variables[] = {
+    { HRSWRUN_OSINDEX,   ASN_INTEGER, RONLY, var_hrswrun, 1, {1}},
+    { HRSWRUN_INDEX,     ASN_INTEGER, RONLY, var_hrswrun, 3, {2,1,1}},
+    { HRSWRUN_NAME,    ASN_OCTET_STR, RONLY, var_hrswrun, 3, {2,1,2}},
+    { HRSWRUN_ID,      ASN_OBJECT_ID, RONLY, var_hrswrun, 3, {2,1,3}},
+    { HRSWRUN_PATH,    ASN_OCTET_STR, RONLY, var_hrswrun, 3, {2,1,4}},
+    { HRSWRUN_PARAMS,  ASN_OCTET_STR, RONLY, var_hrswrun, 3, {2,1,5}},
+    { HRSWRUN_TYPE,      ASN_INTEGER, RONLY, var_hrswrun, 3, {2,1,6}},
+    { HRSWRUN_STATUS,    ASN_INTEGER, RONLY, var_hrswrun, 3, {2,1,7}}
+};
+
+struct variable4 hrswrunperf_variables[] = {
+    { HRSWRUNPERF_CPU,   ASN_INTEGER, RONLY, var_hrswrun, 3, {1,1,1}},
+    { HRSWRUNPERF_MEM,   ASN_INTEGER, RONLY, var_hrswrun, 3, {1,1,2}}
+};
+
+oid hrswrun_variables_oid[]     = { 1,3,6,1,2,1,25,4 };
+oid hrswrunperf_variables_oid[] = { 1,3,6,1,2,1,25,5 };
+
+
 void init_hr_swrun(void)
 {
 #ifdef PROC_SYMBOL
@@ -100,6 +137,11 @@ void init_hr_swrun(void)
 #ifdef NPROC_SYMBOL
   auto_nlist( NPROC_SYMBOL,0,0 );
 #endif
+
+  proc_table = 0;
+
+    REGISTER_MIB("host/hr_swrun", hrswrun_variables, variable4, hrswrun_variables_oid);
+    REGISTER_MIB("host/hr_swrun", hrswrunperf_variables, variable4, hrswrunperf_variables_oid);
 }
 
 /*
@@ -117,27 +159,25 @@ void init_hr_swrun(void)
 int
 header_hrswrun(struct variable *vp,
 	       oid *name,
-	       int *length,
+	       size_t *length,
 	       int exact,
-	       int *var_len,
+	       size_t *var_len,
 	       WriteMethod **write_method)
 {
 #define HRSWRUN_NAME_LENGTH	9
     oid newname[MAX_OID_LEN];
     int result;
-    char c_oid[SPRINT_MAX_LEN];
 
-    if (snmp_get_do_debugging()) {
-      sprint_objid (c_oid, name, *length);
-      DEBUGMSGTL(("host/hr_swrun", "var_hrswrun: %s %d\n", c_oid, exact));
-    }
+    DEBUGMSGTL(("host/hr_swrun", "var_hrswrun: "));
+    DEBUGMSGOID(("host/hr_swrun", name, *length));
+    DEBUGMSG(("host/hr_swrun"," %d\n", exact));
 
-    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
+    memcpy( (char *)newname,(char *)vp->name, vp->namelen * sizeof(oid));
     newname[HRSWRUN_NAME_LENGTH] = 0;
-    result = snmp_oid_compare(name, *length, newname, (int)vp->namelen + 1);
+    result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
     if ((exact && (result != 0)) || (!exact && (result >= 0)))
         return(MATCH_FAILED);
-    memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
+    memcpy( (char *)name,(char *)newname, (vp->namelen + 1) * sizeof(oid));
     *length = vp->namelen + 1;
 
     *write_method = 0;
@@ -148,23 +188,21 @@ header_hrswrun(struct variable *vp,
 int
 header_hrswrunEntry(struct variable *vp,
 		    oid *name,
-		    int *length,
+		    size_t *length,
 		    int exact,
-		    int *var_len,
+		    size_t *var_len,
 		    WriteMethod **write_method)
 {
 #define HRSWRUN_ENTRY_NAME_LENGTH	11
     oid newname[MAX_OID_LEN];
     int pid, LowPid=-1;
     int result;
-    char c_oid[SPRINT_MAX_LEN];
 
-    if (snmp_get_do_debugging()) {
-      sprint_objid (c_oid, name, *length);
-      DEBUGMSGTL(("host/hr_swrun", "var_hrswrunEntry: %s %d\n", c_oid, exact));
-    }
+    DEBUGMSGTL(("host/hr_swrun", "var_hrswrunEntry: "));
+    DEBUGMSGOID(("host/hr_swrun", name, *length));
+    DEBUGMSG(("host/hr_swrun"," %d\n", exact));
 
-    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
+    memcpy( (char *)newname,(char *)vp->name, vp->namelen * sizeof(oid));
 
 		/*
 	 	 *  Find the "next" running process
@@ -179,11 +217,9 @@ header_hrswrunEntry(struct variable *vp,
         if ( pid == -1 )
 	    break;
 	newname[HRSWRUN_ENTRY_NAME_LENGTH] = pid;
-        if (snmp_get_do_debugging()) {
-          sprint_objid (c_oid, newname, *length);
-          DEBUGMSG(("host/hr_swrun", "%s", c_oid));
-        }
-        result = snmp_oid_compare(name, *length, newname, (int)vp->namelen + 1);
+    DEBUGMSGOID(("host/hr_swrun", newname, *length));
+    DEBUGMSG(("host/hr_swrun","\n"));
+        result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
         if (exact && (result == 0)) {
 	    LowPid = pid;
 #ifndef linux
@@ -211,13 +247,14 @@ DEBUGMSG(("host/hr_swrun", "\n"));
     }
 
     newname[HRSWRUN_ENTRY_NAME_LENGTH] = LowPid;
-    memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
+    memcpy( (char *)name,(char *)newname, (vp->namelen + 1) * sizeof(oid));
     *length = vp->namelen + 1;
     *write_method = 0;
     *var_len = sizeof(long);	/* default to 'long' results */
 
-    sprint_objid (c_oid, name, *length);
-    DEBUGMSGTL(("host/hr_swrun", "... get process stats %s\n", c_oid));
+    DEBUGMSGTL(("host/hr_swrun", "... get process stats "));
+    DEBUGMSGOID(("host/hr_swrun", name, *length));
+    DEBUGMSG(("host/hr_swrun","\n"));
     return LowPid;
 }
 
@@ -228,16 +265,16 @@ DEBUGMSG(("host/hr_swrun", "\n"));
 	 *********************/
 
 
-u_char	*
+u_char *
 var_hrswrun(struct variable *vp,
 	    oid *name,
-	    int *length,
+	    size_t *length,
 	    int exact,
-	    int *var_len,
+	    size_t *var_len,
 	    WriteMethod **write_method)
 {
     int pid=0;
-    static char string[100];
+    static char string[256];
 #ifdef HAVE_SYS_PSTAT_H
     struct pst_status proc_buf;
 #elif defined(solaris2)
@@ -302,7 +339,11 @@ var_hrswrun(struct variable *vp,
 
     switch (vp->magic){
 	case HRSWRUN_OSINDEX:
+#if NO_DUMMY_VALUES
+		return NULL;
+#else
 	    long_return = 1;		/* Probably! */
+#endif
 	    return (u_char *)&long_return;
 
 	case HRSWRUN_INDEX:
@@ -335,6 +376,9 @@ var_hrswrun(struct variable *vp,
 	    strcpy( string, cp );
             fclose(fp);
 #else
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    sprintf(string, "process name");
 #endif
 	    *var_len = strlen(string);
@@ -369,6 +413,9 @@ var_hrswrun(struct variable *vp,
 	    strcpy( string, buf );
             fclose(fp);
 #else
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    sprintf(string, "/bin/wombat");
 #endif
 	    *var_len = strlen(string);
@@ -429,6 +476,9 @@ var_hrswrun(struct variable *vp,
 	    strcpy( string, cp );
             fclose(fp);
 #else
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    sprintf(string, "-h -q -v");
 #endif
 	    *var_len = strlen(string);
@@ -593,14 +643,17 @@ var_hrswrun(struct variable *vp,
 		    ++cp;
 	        ++cp;
 	    }
-	    long_return = atoi( cp );		/* rss */
+	    long_return = atoi( cp ) * (getpagesize()/1024);		/* rss */
             fclose(fp);
 #else
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = 16*1024;	/* XXX - 16M! */
 #endif
 	    return (u_char *)&long_return;
 	default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_hrswrun\n", vp->magic));
     }
     return NULL;
 }
@@ -618,7 +671,7 @@ static int nproc;
 void
 Init_HR_SWRun (void)
 {
-    int bytes;
+    size_t bytes;
     static time_t iwhen = 0;
     time_t now;
 
@@ -678,30 +731,32 @@ Init_HR_SWRun (void)
 #else
 
     current_proc_entry = 1;
+#ifndef bsdi2
     nproc = 0;
 
     if (auto_nlist(NPROC_SYMBOL, (char *)&nproc, sizeof(int)) == 0) {
-        perror("Init_HR_SWRun-auto_nlist NPROC");
+        snmp_log_perror("Init_HR_SWRun-auto_nlist NPROC");
         return;
     }
+#endif
     bytes = nproc*sizeof(struct proc);
 
     if (proc_table) free((char *)proc_table);
     if ((proc_table=(struct proc *) malloc(bytes)) == NULL ) {
         nproc = 0;
-        perror("Init_HR_SWRun-malloc");
+        snmp_log_perror("Init_HR_SWRun-malloc");
         return;
     }
 
     {   int proc_table_base;
         if (auto_nlist(PROC_SYMBOL, (char *)&proc_table_base, sizeof(proc_table_base)) == 0) {
             nproc = 0;
-            perror("Init_HR_SWRun-auto_nlist PROC");
+            snmp_log_perror("Init_HR_SWRun-auto_nlist PROC");
             return;
         }
         if (klookup( proc_table_base, (char *)proc_table, bytes) == 0) {
             nproc = 0;
-            perror("Init_HR_SWRun-klookup");
+            snmp_log_perror("Init_HR_SWRun-klookup");
             return;
         }
     }
