@@ -3,25 +3,23 @@
  *
  */
 
-#include "mib_module_config.h"
-
 #include <config.h>
 #if defined(IFNET_NEEDS_KERNEL) && !defined(_KERNEL)
 #define _KERNEL 1
 #define _I_DEFINED_KERNEL
 #endif
 #include <sys/types.h>
+#if HAVE_SYS_PARAM_H
 #include <sys/param.h>
+#endif
 #include <sys/socket.h>
 
-#if STDC_HEADERS
+#if HAVE_STRING_H
 #include <string.h>
-#include <stdlib.h>
-#else
+#endif
 #if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#endif
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
@@ -48,7 +46,9 @@
 #ifdef _I_DEFINED_KERNEL
 #undef _KERNEL
 #endif
+#if HAVE_NETINET_IN_SYSTM_H
 #include <netinet/in_systm.h>
+#endif
 #include <netinet/ip.h>
 
 #include <netinet/ip_icmp.h>
@@ -62,11 +62,16 @@
 #if HAVE_INET_MIB2_H
 #include <inet/mib2.h>
 #endif
+
+#if HAVE_DMALLOC_H
+#include <dmalloc.h>
+#endif
+
 #ifdef solaris2
 #include "kernel_sunos5.h"
 #endif
 
-#include "../../../snmplib/system.h"
+#include "system.h"
 #include "asn1.h"
 #include "snmp_debug.h"
 
@@ -81,6 +86,7 @@
 /* #include "../common_header.h" */
 
 #include "icmp.h"
+#include "sysORTable.h"
 
 	/*********************
 	 *
@@ -136,13 +142,24 @@ struct variable2 icmp_variables[] = {
 
 /* Define the OID pointer to the top of the mib tree that we're
    registering underneath */
-oid icmp_variables_oid[] = { 1,3,6,1,2,1,5 };
+oid icmp_variables_oid[] = { SNMP_OID_MIB2,5 };
+#ifdef USING_MIBII_IP_MODULE
+extern oid ip_module_oid[];
+extern int ip_module_oid_len;
+extern int ip_module_count;
+#endif
 
 void init_icmp(void)
 {
   /* register ourselves with the agent to handle our mib tree */
   REGISTER_MIB("mibII/icmp", icmp_variables, variable2, icmp_variables_oid);
 
+#ifdef USING_MIBII_IP_MODULE
+  if ( ++ip_module_count == 2 )
+	REGISTER_SYSOR_TABLE( ip_module_oid, ip_module_oid_len,
+		"The MIB module for managing IP and ICMP implementations");
+#endif 
+
 #ifdef ICMPSTAT_SYMBOL
     auto_nlist( ICMPSTAT_SYMBOL,0,0 );
 #endif
@@ -162,20 +179,18 @@ void init_icmp(void)
 int
 header_icmp(struct variable *vp,
 	    oid *name,
-	    int *length,
+	    size_t *length,
 	    int exact,
-	    int *var_len,
+	    size_t *var_len,
 	    WriteMethod **write_method)
 {
 #define ICMP_NAME_LENGTH	8
     oid newname[MAX_OID_LEN];
     int result;
-    char c_oid[SPRINT_MAX_LEN];
 
-    if (snmp_get_do_debugging()) {
-      sprint_objid (c_oid, name, *length);
-      DEBUGMSGTL(("mibII/icmp", "var_icmp: %s %d\n", c_oid, exact));
-    }
+    DEBUGMSGTL(("mibII/icmp", "var_icmp: "));
+    DEBUGMSGOID(("mibII/icmp", name, *length));
+    DEBUGMSG(("mibII/icmp"," %d\n", exact));
 
     memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
     newname[ICMP_NAME_LENGTH] = 0;
@@ -198,14 +213,14 @@ header_icmp(struct variable *vp,
 
 #ifndef solaris2
 #ifndef linux
-#ifdef HAVE_SYS_TCPIPSTATS_H
+#if HAVE_SYS_TCPIPSTATS_H
 
 u_char *
 var_icmp(struct variable *vp,
 	 oid *name,
-	 int *length,
+	 size_t *length,
 	 int exact,
-	 int *var_len,
+	 size_t *var_len,
 	 WriteMethod **write_method)
 {
     register int i;
@@ -218,7 +233,7 @@ var_icmp(struct variable *vp,
      *	Get the ICMP statistics from the kernel...
      */
     if (sysmp (MP_SAGET, MPSA_TCPIPSTATS, &tcpipstats, sizeof tcpipstats) == -1) {
-	perror ("sysmp(MP_SAGET)(MPSA_TCPIPSTATS)");
+	snmp_log_perror ("sysmp(MP_SAGET)(MPSA_TCPIPSTATS)");
     }
 #define icmpstat tcpipstats.icmpstat
 
@@ -308,7 +323,7 @@ var_icmp(struct variable *vp,
           long_return = icmpstat.icps_outhist[ICMP_MASKREPLY];
           return (u_char *) &long_return;
 	default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_icmp\n", vp->magic));
     }
 
     return NULL;
@@ -319,9 +334,9 @@ var_icmp(struct variable *vp,
 u_char *
 var_icmp(struct variable *vp,
 	 oid *name,
-	 int *length,
+	 size_t *length,
 	 int exact,
-	 int *var_len,
+	 size_t *var_len,
 	 WriteMethod **write_method)
 {
     register int i;
@@ -432,7 +447,7 @@ var_icmp(struct variable *vp,
           long_return = icmpstat.icps_outhist[ICMP_MASKREPLY];
           return (u_char *) &long_return;
 	default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_icmp\n", vp->magic));
     }
 
     return NULL;
@@ -445,9 +460,9 @@ var_icmp(struct variable *vp,
 u_char *
 var_icmp(struct variable *vp,
 	 oid *name,
-	 int *length,
+	 size_t *length,
 	 int exact,
-	 int *var_len,
+	 size_t *var_len,
 	 WriteMethod **write_method)
 {
     static struct icmp_mib icmpstat;
@@ -486,7 +501,7 @@ var_icmp(struct variable *vp,
     case ICMPOUTADDRMASKREPS: return (u_char *) &icmpstat.IcmpOutAddrMaskReps;
 
     default:
-      ERROR_MSG("");
+      DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_icmp\n", vp->magic));
     }
     return NULL;
 
@@ -498,9 +513,9 @@ var_icmp(struct variable *vp,
 u_char *
 var_icmp(struct variable *vp,
 	 oid *name,
-	 int *length,
+	 size_t *length,
 	 int exact,
-	 int *var_len,
+	 size_t *var_len,
 	 WriteMethod **write_method)
 {
     mib2_icmp_t icmpstat;
@@ -594,7 +609,7 @@ var_icmp(struct variable *vp,
       		long_return = icmpstat.icmpOutAddrMaskReps;
       		break;
 	default:
-		ERROR_MSG("");
+		DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_icmp\n", vp->magic));
                 return(NULL);
     }
     return ((u_char *) &long_return);
