@@ -3,8 +3,6 @@
  *
  */
 
-#include "mib_module_config.h"
-
 #include <config.h>
 
 #if defined(IFNET_NEEDS_KERNEL) && !defined(_KERNEL) && !defined(IFNET_NEEDS_KERNEL_LATE)
@@ -12,14 +10,18 @@
 #define _I_DEFINED_KERNEL
 #endif
 
-#if STDC_HEADERS
-#include <stdlib.h>
+#if HAVE_STRING_H
 #include <string.h>
 #endif
+#if HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#if HAVE_SYS_PARAM_H
 #include <sys/param.h>
+#endif
 #include <sys/types.h>
 #if defined(IFNET_NEEDS_KERNEL) && !defined(_KERNEL) && defined(IFNET_NEEDS_KERNEL_LATE)
 #define _KERNEL 1
@@ -59,10 +61,12 @@
 #if HAVE_SYS_STREAM_H
 #include <sys/stream.h>
 #endif
-#ifdef HAVE_NET_ROUTE_H
+#if HAVE_NET_ROUTE_H
 #include <net/route.h>
 #endif
+#if HAVE_NETINET_IN_SYSTM_H
 #include <netinet/in_systm.h>
+#endif
 #if HAVE_SYS_HASHING_H
 #include <sys/hashing.h>
 #endif
@@ -102,14 +106,18 @@
 # endif
 #endif
 
+#if HAVE_DMALLOC_H
+#include <dmalloc.h>
+#endif
+
+#include "mibincl.h"
+
 #ifdef solaris2
 #include "kernel_sunos5.h"
 #else
 #include "kernel.h"
 #endif
 
-#include "../mibincl.h"
-
 #ifdef hpux
 #include <sys/mib.h>
 #include <netinet/mib_kern.h>
@@ -118,13 +126,15 @@
 #if HAVE_SYS_SYSCTL_H
 #include <sys/sysctl.h>
 
-#if defined(freebsd3)
+#ifdef freebsd3
 #    define USE_SYSCTL_IFLIST
 #else
 # if defined(CTL_NET) && !defined(freebsd2)
 #  ifdef PF_ROUTE
 #   ifdef NET_RT_IFLIST
-#    define USE_SYSCTL_IFLIST
+#    ifndef netbsd1
+#     define USE_SYSCTL_IFLIST
+#    endif
 #   endif
 #  endif
 # endif
@@ -133,9 +143,10 @@
 
 /* #include "../common_header.h" */
 
-#include "../../../snmplib/system.h"
+#include "system.h"
+#include "snmp_logging.h"
 
-#ifdef HAVE_OSRELDATE_H
+#if HAVE_OSRELDATE_H
 #include <osreldate.h>
 #endif
 #ifdef CAN_USE_SYSCTL
@@ -145,6 +156,7 @@
 #include "../struct.h"
 #include "../util_funcs.h"
 #include "auto_nlist.h"
+#include "sysORTable.h"
 
 static int Interface_Scan_Get_Count (void);
 
@@ -175,17 +187,20 @@ struct variable4 interfaces_variables[] = {
 };
 
 /* Define the OID pointer to the top of the mib tree that we're
-   registering underneath */
-oid interfaces_variables_oid[] = { 1,3,6,1,2,1,2 };
+   registering underneath, and the OID of the MIB module */
+oid interfaces_variables_oid[] = { SNMP_OID_MIB2,2 };
+oid interfaces_module_oid[]    = { SNMP_OID_MIB2,31 };
 
 void init_interfaces(void)
 {
   /* register ourselves with the agent to handle our mib tree */
   REGISTER_MIB("mibII/interfaces", interfaces_variables, variable4, \
                interfaces_variables_oid);
+  REGISTER_SYSOR_ENTRY(interfaces_module_oid,
+	"The MIB module to describe generic objects for network interface sub-layers");
   
 #ifndef USE_SYSCTL_IFLIST
-#ifdef HAVE_NET_IF_MIB_H
+#if HAVE_NET_IF_MIB_H
   init_interfaces_setup();
 #endif
 #endif
@@ -247,21 +262,19 @@ static size_t if_list_size = 0;
 static int
 header_ifEntry(struct variable *vp,
 	       oid *name,
-	       int *length,
+	       size_t *length,
 	       int exact,
-	       int *var_len,
+	       size_t *var_len,
 	       WriteMethod **write_method)
 {
 #define IFENTRY_NAME_LENGTH	10
     oid newname[MAX_OID_LEN];
     register int	interface;
     int result, count;
-    char c_oid[SPRINT_MAX_LEN];
 
-    if (snmp_get_do_debugging()) {
-      sprint_objid (c_oid, name, *length);
-      DEBUGMSGTL(("mibII/interfaces", "var_ifEntry: %s %d\n", c_oid, exact));
-    }
+    DEBUGMSGTL(("mibII/interfaces", "var_ifEntry: "));
+    DEBUGMSGOID(("mibII/interfaces", name, *length));
+    DEBUGMSG(("mibII/interfaces"," %d\n", exact));
     
     memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
     /* find "next" interface */
@@ -283,8 +296,9 @@ header_ifEntry(struct variable *vp,
     *write_method = 0;
     *var_len = sizeof(long);	/* default to 'long' results */
 
-    sprint_objid (c_oid, name, *length);
-    DEBUGMSGTL(("mibII/interfaces", "... get I/F stats %s\n", c_oid));
+    DEBUGMSGTL(("mibII/interfaces", "... get I/F stats "));
+    DEBUGMSGOID(("mibII/interfaces", name, *length));
+    DEBUGMSG(("mibII/interfaces","\n"));
 
     return interface;
 }
@@ -305,20 +319,18 @@ header_ifEntry(struct variable *vp,
 static int
 header_interfaces(struct variable *vp,
 		  oid *name,
-		  int *length,
+		  size_t *length,
 		  int exact,
-		  int *var_len,
+		  size_t *var_len,
 		  WriteMethod **write_method)
 {
 #define INTERFACES_NAME_LENGTH	8
   oid newname[MAX_OID_LEN];
   int result;
-  char c_oid[SPRINT_MAX_LEN];
 
-  if (snmp_get_do_debugging()) {
-    sprint_objid (c_oid, name, *length);
-    DEBUGMSGTL(("mibII/interfaces", "var_interfaces: %s %d\n", c_oid, exact));
-  }
+    DEBUGMSGTL(("mibII/interfaces", "var_interfaces: "));
+    DEBUGMSGOID(("mibII/interfaces", name, *length));
+    DEBUGMSG(("mibII/interfaces"," %d\n", exact));
 
   memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
   newname[INTERFACES_NAME_LENGTH] = 0;
@@ -331,14 +343,14 @@ header_interfaces(struct variable *vp,
   *write_method = 0;
   *var_len = sizeof(long);	/* default to 'long' results */
   return MATCH_SUCCEEDED;
-};
+}
 
 u_char *
 var_interfaces(struct variable *vp,
 	       oid *name,
-	       int *length,
+	       size_t *length,
 	       int exact,
-	       int *var_len,
+	       size_t *var_len,
 	       WriteMethod **write_method)
 {
   if (header_interfaces(vp, name, length, exact, var_len, write_method) == MATCH_FAILED )
@@ -350,7 +362,7 @@ var_interfaces(struct variable *vp,
       long_return = Interface_Scan_Get_Count ();
       return (u_char *)&long_return;
     default:
-      ERROR_MSG("");
+      DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_interfaces\n", vp->magic));
     }
   return NULL;
 }
@@ -368,7 +380,7 @@ static int Interface_Scan_By_Index (int, struct if_msghdr *, char *, struct smal
 static int Interface_Get_Ether_By_Index (int, u_char *);
 
 static int
-Interface_Scan_By_Index (int index,
+Interface_Scan_By_Index (int iindex,
 			 struct if_msghdr *if_msg,
 			 char *if_name,
 			 struct small_ifaddr *sifa)
@@ -391,15 +403,15 @@ Interface_Scan_By_Index (int index,
 	  {
 	    const struct sockaddr *a;
 
-	    if (ifp->ifm_index == index)
+	    if (ifp->ifm_index == iindex)
 	      {
 		a = get_address (ifp+1, ifp->ifm_addrs, RTA_IFP);
                 if (a == NULL)
                   return NULL;
 		strncpy (if_name,
-			 ((struct sockaddr_in *) a)->sin_zero,
-			 ((u_char *) a)[5]);
-		if_name[((u_char *) a)[5]] = 0;
+			 ((const struct sockaddr_in *) a)->sin_zero,
+			 ((const u_char *) a)[5]);
+		if_name[((const u_char *) a)[5]] = 0;
 		*if_msg = *ifp;
 		++have_ifinfo;
 	      }
@@ -409,7 +421,7 @@ Interface_Scan_By_Index (int index,
 	  {
 	    struct ifa_msghdr *ifap = (struct ifa_msghdr *) cp;
 
-	    if (ifap->ifam_index == index)
+	    if (ifap->ifam_index == iindex)
 	      {
 		const struct in_addr *ia;
 
@@ -487,7 +499,7 @@ Interface_Scan_Init (void)
   if (sysctl (name, sizeof(name)/sizeof(int),
 	      0, &size, 0, 0) == -1)
     {
-      ERROR_MSG("sysctl(CTL_NET,PF_ROUTE,0,0,NET_RT_IFLIST,0)\n");
+      snmp_log(LOG_ERR,"sysctl size fail\n");
     }
   else
     {
@@ -500,7 +512,8 @@ Interface_Scan_Init (void)
 	    }
 	  if ((if_list = malloc (size)) == 0)
 	    {
-	      ERROR_MSG("out of memory allocating route table\n");
+	      snmp_log(LOG_ERR,"out of memory allocating route table\n");
+	      return;
 	    }
 	  if_list_size = size;
 	}
@@ -511,7 +524,7 @@ Interface_Scan_Init (void)
       if (sysctl (name, sizeof (name) / sizeof (int),
 		  if_list, &size, 0, 0) == -1)
 	{
-	  ERROR_MSG("sysctl(CTL_NET,PF_ROUTE,0,0,NET_RT_IFLIST,0)\n");
+	  snmp_log(LOG_ERR,"sysctl get fail\n");
 	}
       if_list_end = if_list + size;
     }
@@ -520,9 +533,9 @@ Interface_Scan_Init (void)
 u_char *
 var_ifEntry(struct variable *vp,
 	    oid *name,
-	    int *length,
+	    size_t *length,
 	    int exact,
-	    int *var_len,
+	    size_t *var_len,
 	    WriteMethod **write_method)
 {
   int interface;
@@ -596,6 +609,9 @@ var_ifEntry(struct variable *vp,
 #ifdef if_odrops
     long_return = (u_long) if_msg.ifm_data.ifi_odrops;
 #else
+#if NO_DUMMY_VALUES
+    return NULL;
+#endif
     long_return = 0;
 #endif
     return (u_char *) &long_return;
@@ -657,24 +673,22 @@ struct ifnet *ifnetaddr_list;
 static int
 header_interfaces(struct variable *vp,
 		  oid *name,
-		  int *length,
+		  size_t *length,
 		  int exact,
-		  int *var_len,
+		  size_t *var_len,
 		  WriteMethod **write_method)
 {
 #define INTERFACES_NAME_LENGTH	8
     oid newname[MAX_OID_LEN];
     int result;
-    char c_oid[SPRINT_MAX_LEN];
 
-    if (snmp_get_do_debugging()) {
-      sprint_objid (c_oid, name, *length);
-      DEBUGMSGTL(("mibII/interfaces", "var_interfaces: %s %d\n", c_oid, exact));
-    }
+    DEBUGMSGTL(("mibII/interfaces", "var_interfaces: "));
+    DEBUGMSGOID(("mibII/interfaces", name, *length));
+    DEBUGMSG(("mibII/interfaces"," %d\n", exact));
 
     memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
     newname[INTERFACES_NAME_LENGTH] = 0;
-    result = snmp_oid_compare(name, *length, newname, (int)vp->namelen + 1);
+    result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
     if ((exact && (result != 0)) || (!exact && (result >= 0)))
         return MATCH_FAILED;
     memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
@@ -690,28 +704,26 @@ header_interfaces(struct variable *vp,
 static int
 header_ifEntry(struct variable *vp,
 	       oid *name,
-	       int *length,
+	       size_t *length,
 	       int exact,
-	       int *var_len,
+	       size_t *var_len,
 	       WriteMethod **write_method)
 {
 #define IFENTRY_NAME_LENGTH	10
     oid newname[MAX_OID_LEN];
     register int	interface;
     int result, count;
-    char c_oid[SPRINT_MAX_LEN];
 
-    if (snmp_get_do_debugging()) {
-      sprint_objid (c_oid, name, *length);
-      DEBUGMSGTL(("mibII/interfaces", "var_ifEntry: %s %d\n", c_oid, exact));
-    }
+    DEBUGMSGTL(("mibII/interfaces", "var_ifEntry: "));
+    DEBUGMSGOID(("mibII/interfaces", name, *length));
+    DEBUGMSG(("mibII/interfaces"," %d\n", exact));
 
     memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
     /* find "next" interface */
     count = Interface_Scan_Get_Count();
     for(interface = 1; interface <= count; interface++){
 	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
-	result = snmp_oid_compare(name, *length, newname, (int)vp->namelen + 1);
+	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
 	if ((exact && (result == 0)) || (!exact && (result < 0)))
 	    break;
     }
@@ -726,10 +738,9 @@ header_ifEntry(struct variable *vp,
     *write_method = 0;
     *var_len = sizeof(long);	/* default to 'long' results */
 
-    if (snmp_get_do_debugging()) {
-      sprint_objid (c_oid, name, *length);
-      DEBUGMSGTL(("mibII/interfaces", "... get I/F stats %s\n", c_oid));
-    }
+    DEBUGMSGTL(("mibII/interfaces", "... get I/F stats "));
+    DEBUGMSGOID(("mibII/interfaces", name, *length));
+    DEBUGMSG(("mibII/interfaces","\n"));
 
     return interface;
 }
@@ -745,9 +756,9 @@ header_ifEntry(struct variable *vp,
 u_char	*
 var_interfaces(struct variable *vp,
 	       oid *name,
-	       int *length,
+	       size_t *length,
 	       int exact,
-	       int *var_len,
+	       size_t *var_len,
 	       WriteMethod **write_method)
 {
     if (header_interfaces(vp, name, length, exact, var_len, write_method) == MATCH_FAILED )
@@ -758,7 +769,7 @@ var_interfaces(struct variable *vp,
 	    long_return = Interface_Scan_Get_Count();
 	    return (u_char *)&long_return;
 	default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_interfaces\n", vp->magic));
     }
     return NULL;
 }
@@ -770,9 +781,9 @@ var_interfaces(struct variable *vp,
 u_char *
 var_ifEntry(struct variable *vp,
 	    oid *name,
-	    int *length,
+	    size_t *length,
 	    int exact,
-	    int *var_len,
+	    size_t *var_len,
 	    WriteMethod **write_method)
 {
     static struct ifnet ifnet;
@@ -823,6 +834,9 @@ var_ifEntry(struct variable *vp,
 		if(ifnet.if_type == IFT_ISDNPRIMARY) long_return=64000*30;
 	    }
 #else
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = (u_long) 10000000;
 #endif
 	    return (u_char *) &long_return;
@@ -860,6 +874,9 @@ var_ifEntry(struct variable *vp,
                + (now.tv_usec - ifnet.if_lastchange.tv_usec) / 10000);
           }
 #else
+#if NO_DUMMY_VALUES
+	  return NULL;
+#endif
           long_return = 0; /* XXX */
 #endif
           return (u_char *) &long_return;
@@ -867,6 +884,9 @@ var_ifEntry(struct variable *vp,
 #ifdef STRUCT_IFNET_HAS_IF_IBYTES
           long_return = (u_long)  ifnet.if_ibytes;
 #else
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = (u_long)  ifnet.if_ipackets * 308; /* XXX */
 #endif
 	    return (u_char *) &long_return;
@@ -882,6 +902,9 @@ var_ifEntry(struct variable *vp,
 #if STRUCT_IFNET_HAS_IF_IMCASTS
 	    long_return = (u_long)  ifnet.if_imcasts;
 #else
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = (u_long)  0; /* XXX */
 #endif
 	    return (u_char *) &long_return;
@@ -889,6 +912,9 @@ var_ifEntry(struct variable *vp,
 #if STRUCT_IFNET_HAS_IF_IQDROPS
 	    long_return = (u_long)  ifnet.if_iqdrops;
 #else
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = (u_long)  0; /* XXX */
 #endif
 	    return (u_char *) &long_return;
@@ -899,6 +925,9 @@ var_ifEntry(struct variable *vp,
 #if STRUCT_IFNET_HAS_IF_NOPROTO
 	    long_return = (u_long)  ifnet.if_noproto;
 #else
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = (u_long)  0; /* XXX */
 #endif
 	    return (u_char *) &long_return;
@@ -906,6 +935,9 @@ var_ifEntry(struct variable *vp,
 #ifdef STRUCT_IFNET_HAS_IF_OBYTES
           long_return = (u_long)  ifnet.if_obytes;
 #else
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = (u_long)  ifnet.if_opackets * 308; /* XXX */
 #endif
 	    return (u_char *) &long_return;
@@ -921,6 +953,9 @@ var_ifEntry(struct variable *vp,
 #if STRUCT_IFNET_HAS_IF_OMCASTS
 	    long_return = (u_long)  ifnet.if_omcasts;
 #else
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = (u_long)  0; /* XXX */
 #endif
 	    return (u_char *) &long_return;
@@ -937,7 +972,7 @@ var_ifEntry(struct variable *vp,
 	    *var_len = nullOidLen;
 	    return (u_char *) nullOid;
 	default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ifEntry\n", vp->magic));
     }
     return NULL;
 }
@@ -947,14 +982,14 @@ var_ifEntry(struct variable *vp,
 u_char *
 var_ifEntry(struct variable *vp,
 	    oid *name,
-	    int *length,
+	    size_t *length,
 	    int exact,
-	    int *var_len,
+	    size_t *var_len,
 	    WriteMethod **write_method)
 {
     static struct ifnet ifnet;
     register int interface;
-    static struct in_ifaddr in_ifaddr;
+    static struct in_ifaddr in_ifaddrVar;
     static char Name[16];
     register char *cp;
 #if STRUCT_IFNET_HAS_IF_LASTCHANGE_TV_SEC
@@ -970,7 +1005,7 @@ var_ifEntry(struct variable *vp,
     if ( interface == MATCH_FAILED )
 	return NULL;
 
-    Interface_Scan_By_Index(interface, Name, &ifnet, &in_ifaddr);
+    Interface_Scan_By_Index(interface, Name, &ifnet, &in_ifaddrVar);
 
 	/*
 	 * Additional information about the interfaces is available under
@@ -1097,7 +1132,7 @@ var_ifEntry(struct variable *vp,
 	    *var_len = nullOidLen;
 	    return (u_char *) nullOid;
 	default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ifEntry\n", vp->magic));
     }
     return NULL;
 }
@@ -1119,9 +1154,9 @@ IF_cmp(void *addr, void *ep)
 u_char *
 var_ifEntry(struct variable *vp,
 	    oid *name,
-	    int *length,
+	    size_t *length,
 	    int exact,
-	    int *var_len,
+	    size_t *var_len,
 	    WriteMethod **write_method)
 {
     int        interface;
@@ -1204,7 +1239,7 @@ var_ifEntry(struct variable *vp,
       long_return = (u_long)ifstat.ifOutQLen;
       return (u_char *) &long_return;
     default:
-      ERROR_MSG("");
+      DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ifEntry\n", vp->magic));
     }
     return NULL;
 }
@@ -1237,12 +1272,13 @@ Interface_Scan_Init (void)
     struct ifreq ifrq;
     struct ifnet **ifnetaddr_ptr;
     FILE *devin;
-    int a, b, c, d, e, i, fd;
+    unsigned long rec_pkt, rec_oct, rec_err, snd_pkt, snd_oct, snd_err, coll;
+    int i, fd;
     extern conf_if_list *if_list;
     conf_if_list *if_ptr;
-    char *scan_line_2_2="%[^:]: %*d %d %d %*d %*d %*d %*d %*d %*d %d %d %*d %*d %d";
-    char *scan_line_2_0="%[^:]: %d %d %*d %*d %*d %d %d %*d %*d %d";
-    char *scan_line_to_use;
+    const char *scan_line_2_2="%lu %lu %lu %*lu %*lu %*lu %*lu %*lu %lu %lu %lu %*lu %*lu %lu";
+    const char *scan_line_2_0="%lu %lu %*lu %*lu %*lu %lu %lu %*lu %*lu %lu";
+    const char *scan_line_to_use;
     
 #endif  
 
@@ -1256,6 +1292,7 @@ Interface_Scan_Init (void)
 	struct ifnet *old = ifnetaddr_list;
 	ifnetaddr_list = ifnetaddr_list->if_next;
 	free (old->if_name);
+	free (old->if_unit);
 	free (old);
       }
 
@@ -1264,7 +1301,7 @@ Interface_Scan_Init (void)
 
     if ((fd = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
       {
-	ERROR_MSG("cannot open inet/dgram socket - continuing...\n");
+	DEBUGMSGTL(("snmpd", "socket open failure in Interface_Scan_Init\n"));
 	return; /** exit (1); **/
       }
 
@@ -1276,7 +1313,7 @@ Interface_Scan_Init (void)
     if (! (devin = fopen ("/proc/net/dev", "r")))
       {
 	close (fd);
-	ERROR_MSG("cannot open /proc/net/dev - continuing...\n");
+	snmp_log(LOG_ERR,"cannot open /proc/net/dev - continuing...\n");
 	return; /** exit (1); **/
       }
 
@@ -1300,25 +1337,43 @@ Interface_Scan_Init (void)
     while (fgets (line, sizeof(line), devin))
       {
 	struct ifnet *nnew;
+	char *stats, *ifstart = line;
 
-	if (6 != sscanf (line, scan_line_to_use,
-			 ifname_buf, &a, &b, &c, &d, &e))
+	while (*ifstart == ' ') ifstart++;
+	stats = strrchr(ifstart, ':');
+	*stats++ = 0;
+	strcpy(ifname_buf, ifstart);
+	while (*stats == ' ') stats++;
+
+	if ((scan_line_to_use == scan_line_2_2 &&
+	    sscanf (stats, scan_line_to_use, &rec_oct, &rec_pkt, &rec_err, &snd_oct, &snd_pkt, &snd_err, &coll) != 7) ||
+	    (scan_line_to_use == scan_line_2_0 && 
+	    sscanf (stats, scan_line_to_use, &rec_pkt, &rec_err, &snd_pkt, &snd_err, &coll) != 5))
 	  continue;
 	
-	nnew = (struct ifnet *) malloc (sizeof (struct ifnet));	    
-	memset ( nnew, 0, sizeof (struct ifnet));
-	
+	nnew = (struct ifnet *) calloc (1, sizeof (struct ifnet));	    
+	if (nnew == NULL)  break; /* alloc error */
+
 	/* chain in: */
 	*ifnetaddr_ptr = nnew;
 	ifnetaddr_ptr = &nnew->if_next;
 	i++;
 	
 	/* linux previous to 1.3.~13 may miss transmitted loopback pkts: */
-	if (! strcmp (ifname_buf, "lo") && a > 0 && ! c)
-	  c = a;
+	if (! strcmp (ifname_buf, "lo") && rec_pkt > 0 && ! snd_pkt)
+	  snd_pkt = rec_pkt;
 
-	nnew->if_ipackets = a, nnew->if_ierrors = b, nnew->if_opackets = c,
-	nnew->if_oerrors = d, nnew->if_collisions = e;
+	nnew->if_ipackets = rec_pkt;
+	nnew->if_ierrors = rec_err;
+	nnew->if_opackets = snd_pkt;
+	nnew->if_oerrors = snd_err;
+	nnew->if_collisions = coll;
+	if (scan_line_to_use == scan_line_2_2) {
+	  nnew->if_ibytes = rec_oct; nnew->if_obytes = snd_oct;
+	}
+	else {
+	  nnew->if_ibytes = rec_pkt*308; nnew->if_obytes = snd_pkt*308;
+	}
 	
 	/* ifnames are given as ``   eth0'': split in ``eth'' and ``0'': */
 	for (ifname = ifname_buf; *ifname && *ifname == ' '; ifname++) ;
@@ -1327,11 +1382,9 @@ Interface_Scan_Init (void)
 	nnew->if_name = (char *) strdup (ifname);
 	for (ptr = nnew->if_name; *ptr && (*ptr < '0' || *ptr > '9'); 
 	     ptr++) ;
-	nnew->if_unit = (*ptr) ? atoi (ptr) : 0;
+	nnew->if_unit = strdup(*ptr ? ptr : "0");
 	*ptr = 0;
 
-	sprintf (fullname, "%s%d", nnew->if_name, nnew->if_unit);
-
 	strcpy (ifrq.ifr_name, ifname);
 	if (ioctl (fd, SIOCGIFADDR, &ifrq) < 0)
 	  memset ((char *) &nnew->if_addr, 0, sizeof (nnew->if_addr));
@@ -1364,9 +1417,13 @@ Interface_Scan_Init (void)
 	nnew->if_metric = ioctl (fd, SIOCGIFMETRIC, &ifrq) < 0
 	  		? 0 : ifrq.ifr_metric;
 	    
+#ifdef SIOCGIFMTU
 	strcpy (ifrq.ifr_name, ifname);
 	nnew->if_mtu = (ioctl (fd, SIOCGIFMTU, &ifrq) < 0) 
 			  ? 0 : ifrq.ifr_mtu;
+#else
+	nnew->if_mtu = 0;
+#endif
 
 	for (if_ptr = if_list; if_ptr; if_ptr = if_ptr->next)
 	    if (! strcmp (if_ptr->name, fullname))
@@ -1438,7 +1495,11 @@ int Interface_Scan_Next(short *Index,
 
  	    saveName[sizeof (saveName)-1] = '\0';
 	    cp = (char *) strchr(saveName, '\0');
+#ifdef linux
+	    strcat (cp, ifnet.if_unit);
+#else
 	    string_append_int (cp, ifnet.if_unit);
+#endif
 	    if (1 || strcmp(saveName,"lo0") != 0) {  /* XXX */
 
 		if (Index)
@@ -1690,7 +1751,7 @@ Interface_Index_By_Name(char *Name,
 	if ((sd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
 	  return (0);
 	ifconf.ifc_buf = buf;
-	ifconf.ifc_len = sizeof buf;
+	ifconf.ifc_len = sizeof(buf);
 	if (ioctl(sd, SIOCGIFCONF, &ifconf) == -1) {
 	  ret = 0;
 	  goto Return;
@@ -1726,6 +1787,13 @@ Interface_Index_By_Name(char *Name,
 #include <net/if_mib.h>
 #include <net/route.h>
 
+static int header_interfaces(struct variable *, oid *, size_t *, int, size_t *,
+                             WriteMethod **write);
+static int header_ifEntry(struct variable *, oid *, size_t *, int, size_t *,
+                             WriteMethod **write);
+u_char	*var_ifEntry(struct variable *, oid *, size_t *, int, size_t *, 
+                             WriteMethod **write);
+
 static	char *physaddrbuf;
 static	int nphysaddrs;
 struct	sockaddr_dl **physaddrs;
@@ -1818,14 +1886,14 @@ loop:
 }
 
 static int
-get_phys_address(int index, char **ap, int *len)
+get_phys_address(int iindex, char **ap, int *len)
 {
 	int i;
 	int once = 1;
 
 	do {
 		for (i = 0; i < nphysaddrs; i++) {
-			if (physaddrs[i]->sdl_index == index)
+			if (physaddrs[i]->sdl_index == iindex)
 				break;
 		}
 		if (i < nphysaddrs)
@@ -1844,9 +1912,9 @@ get_phys_address(int index, char **ap, int *len)
 static int
 header_interfaces(struct variable *vp,
 		  oid *name,
-		  int *length,
+		  size_t *length,
 		  int exact,
-		  int *var_len,
+		  size_t *var_len,
 		  WriteMethod **write_method)
 {
 #define INTERFACES_NAME_LENGTH	8
@@ -1876,9 +1944,9 @@ static int count_oid[5] = { CTL_NET, PF_LINK, NETLINK_GENERIC,
 static int
 header_ifEntry(struct variable *vp,
 	       oid *name,
-	       int *length,
+	       size_t *length,
 	       int exact,
-	       int *var_len,
+	       size_t *var_len,
 	       WriteMethod **write_method)
 {
 #define IFENTRY_NAME_LENGTH	10
@@ -1888,11 +1956,11 @@ header_ifEntry(struct variable *vp,
     static int count_oid[5] = { CTL_NET, PF_LINK, NETLINK_GENERIC, 
 				IFMIB_SYSTEM, IFMIB_IFCOUNT };
     size_t len;
-#ifdef DODEBUG
-    char c_oid[SPRINT_MAX_LEN];
 
-    sprint_objid (c_oid, name, *length);
-    printf ("var_ifEntry: %s %d\n", c_oid, exact);
+#ifdef DODEBUG
+    DEBUGMSGTL(("mibII/interfaces", "var_ifEntry: "));
+    DEBUGMSGOID(("mibII/interfaces", name, *length));
+    DEBUGMSG(("mibII/interfaces"," %d\n", exact));
 #endif
 
     memcpy(newname, vp->name, (int)vp->namelen * sizeof(oid));
@@ -1909,7 +1977,7 @@ header_ifEntry(struct variable *vp,
     }
     if (interface > count) {
 #ifdef DODEBUG
-	printf ("... index out of range\n");
+ DEBUGMSGTL(("interfaces", "... index out of range\n"));
 #endif
         return MATCH_FAILED;
     }
@@ -1921,19 +1989,20 @@ header_ifEntry(struct variable *vp,
     *var_len = sizeof(long);	/* default to 'long' results */
 
 #ifdef DODEBUG
-    sprint_objid (c_oid, name, *length);
-    printf ("... get I/F stats %s\n", c_oid);
+    DEBUGMSGTL(("interfaces", "... get I/F stats "));
+    DEBUGMSGOID(("interfaces", name, *length));
+    DEBUGMSG(("interfaces","\n"));
 #endif
 
     return interface;
 }
 
-u_char	*
+u_char *
 var_interfaces(struct variable *vp,
 		oid *name,
-		int *length,
+		size_t *length,
 		int exact,
-		int *var_len,
+		size_t *var_len,
 		WriteMethod **write_method)
 {
     size_t len;
@@ -1951,7 +2020,7 @@ var_interfaces(struct variable *vp,
 	    long_return = count;
 	    return (u_char *)&long_return;
     default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_interfaces\n", vp->magic));
     }
     return NULL;
 }
@@ -1959,9 +2028,9 @@ var_interfaces(struct variable *vp,
 u_char *
 var_ifEntry(struct variable *vp,
 	    oid *name,
-	    int *length,
+	    size_t *length,
 	    int exact,
-	    int *var_len,
+	    size_t *var_len,
 	    WriteMethod **write_method)
 {
 	int interface;
@@ -2070,7 +2139,7 @@ var_ifEntry(struct variable *vp,
 		*var_len = nullOidLen;
 		return (u_char *) nullOid;
 	default:
-		ERROR_MSG("");
+		DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ifEntry\n", vp->magic));
 	}
 	return NULL;
 }
