@@ -3,25 +3,23 @@
  *
  */
 
-#include "mib_module_config.h"
-
 #include <config.h>
 #if defined(IFNET_NEEDS_KERNEL) && !defined(_KERNEL)
 #define _KERNEL 1
 #define _I_DEFINED_KERNEL
 #endif
 #include <sys/types.h>
+#if HAVE_SYS_PARAM_H
 #include <sys/param.h>
+#endif
 #include <sys/socket.h>
 
-#if STDC_HEADERS
+#if HAVE_STRING_H
 #include <string.h>
-#include <stdlib.h>
-#else
+#endif
 #if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#endif
 #if HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
@@ -47,7 +45,9 @@
 #ifdef _I_DEFINED_KERNEL
 #undef _KERNEL
 #endif
+#if HAVE_NETINET_IN_SYSTM_H
 #include <netinet/in_systm.h>
+#endif
 #if HAVE_SYS_HASHING_H
 #include <sys/hashing.h>
 #endif
@@ -65,21 +65,26 @@
 #include <sys/stream.h>
 #endif
 #include <net/route.h>
-#ifdef HAVE_SYSLOG_H
+#if HAVE_SYSLOG_H
 #include <syslog.h>
 #endif
 
+#if HAVE_DMALLOC_H
+#include <dmalloc.h>
+#endif
+
+#include "mibincl.h"
+
 #ifdef solaris2
 #include "kernel_sunos5.h"
 #else
 #include "kernel.h"
 #endif
 
-#include "../../../snmplib/system.h"
+#include "system.h"
 #include "asn1.h"
 #include "snmp_debug.h"
 
-#include "mibincl.h"
 #include "auto_nlist.h"
 
 #ifdef MIB_IPCOUNTER_SYMBOL
@@ -91,6 +96,7 @@
 
 #include "ip.h"
 #include "interfaces.h"
+#include "sysORTable.h"
 
 
 	/*********************
@@ -171,13 +177,20 @@ struct variable4 ip_variables[] = {
 };
 
 /* Define the OID pointer to the top of the mib tree that we're
-   registering underneath */
-oid ip_variables_oid[] = { 1,3,6,1,2,1,4 };
+   registering underneath, and the OID of the MIB module */
+oid ip_variables_oid[] = { SNMP_OID_MIB2,4 };
+oid ip_module_oid[]    = { SNMP_OID_MIB2,4 };
+oid ip_module_oid_len  = sizeof(ip_module_oid) / sizeof( oid );
+int ip_module_count    = 0;   /* Need to liaise with icmp.c */
 
 void init_ip(void)
 {
   /* register ourselves with the agent to handle our mib tree */
   REGISTER_MIB("mibII/ip", ip_variables, variable4, ip_variables_oid);
+  if ( ++ip_module_count == 2 )
+      REGISTER_SYSOR_ENTRY( ip_module_oid,
+		"The MIB module for managing IP and ICMP implementations");
+
 
   /* for speed optimization, we call this now to do the lookup */
 #ifdef IPSTAT_SYMBOL
@@ -210,20 +223,18 @@ void init_ip(void)
 static int
 header_ip(struct variable *vp,
 	  oid *name,
-	  int *length,
+	  size_t *length,
 	  int exact,
-	  int *var_len,
+	  size_t *var_len,
 	  WriteMethod **write_method)
 {
 #define IP_NAME_LENGTH	8
     oid newname[MAX_OID_LEN];
     int result;
-    char c_oid[SPRINT_MAX_LEN];
 
-    if (snmp_get_do_debugging()) {
-      sprint_objid (c_oid, name, *length);
-      DEBUGMSGTL(("mibII/ip", "var_ip: %s %d\n", c_oid, exact));
-    }
+    DEBUGMSGTL(("mibII/ip", "var_ip: "));
+    DEBUGMSGOID(("mibII/ip", name, *length));
+    DEBUGMSG(("mibII/ip"," %d\n", exact));
 
     memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
     newname[IP_NAME_LENGTH] = 0;
@@ -255,9 +266,9 @@ header_ip(struct variable *vp,
 u_char *
 var_ip(struct variable *vp,
        oid *name,
-       int *length,
+       size_t *length,
        int exact,
-       int *var_len,
+       size_t *var_len,
        WriteMethod **write_method)
 {
     static struct ipstat ipstat;
@@ -280,18 +291,20 @@ var_ip(struct variable *vp,
                (MIB_ipMAXCTR+1)*sizeof (counter));
 #endif
 
+    long_return = 0;
     switch (vp->magic){
 	case IPFORWARDING:
 #ifndef sparc	  
             auto_nlist(IP_FORWARDING_SYMBOL,(char *) &i, sizeof(i));
-	    fflush(stderr);
 	    if (i) {
 		long_return = 1;		/* GATEWAY */
 	    } else {
 		long_return = 2;	    /* HOST    */
 	    }
 #else /* sparc */
-	    long_return = 0;
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 #endif /* sparc */
 
 	    return (u_char *) &long_return;
@@ -301,7 +314,14 @@ var_ip(struct variable *vp,
 	     */
 	    if (!auto_nlist(TCP_TTL_SYMBOL, (char *) &long_return,
                             sizeof(long_return)))
+#if NO_DUMMY_VALUES
+	      return NULL;
+#else
               long_return = 60;	    /* XXX */
+#endif
+#ifdef hpux
+	    long_return = (u_char)long_return;
+#endif
 	    return (u_char *) &long_return;
 	case IPINRECEIVES:
           long_return = ipstat.ips_total;
@@ -323,14 +343,18 @@ var_ip(struct variable *vp,
 #ifdef MIB_IPCOUNTER_SYMBOL
 	    long_return = MIB_ipcounter[7];
 #else
-	    long_return = 0;
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 #endif
 	    return (u_char *) &long_return;
 	case IPINDISCARDS:
 #ifdef MIB_IPCOUNTER_SYMBOL
 	    long_return = MIB_ipcounter[8];
 #else
-	    long_return = 0;
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 #endif
 	    return (u_char *) &long_return;
 	case IPINDELIVERS:
@@ -345,14 +369,18 @@ var_ip(struct variable *vp,
 #ifdef MIB_IPCOUNTER_SYMBOL
 	    long_return = MIB_ipcounter[10];
 #else
-	    long_return = 0;
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 #endif
 	    return (u_char *) &long_return;
 	case IPOUTDISCARDS:
 #ifdef MIB_IPCOUNTER_SYMBOL
 	    long_return = MIB_ipcounter[11];
 #else
-	    long_return = 0;
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 #endif
 	    return (u_char *) &long_return;
 	case IPOUTNOROUTES:
@@ -370,6 +398,9 @@ var_ip(struct variable *vp,
 #ifdef MIB_IPCOUNTER_SYMBOL
 	    long_return = MIB_ipcounter[15];
 #else
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = ipstat.ips_fragments;		/* XXX */
 		/*
 		 * NB: This is the count of fragments received, rather than
@@ -386,28 +417,36 @@ var_ip(struct variable *vp,
 #ifdef MIB_IPCOUNTER_SYMBOL
 	    long_return = MIB_ipcounter[17];
 #else
-	    long_return = 0;
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 #endif
 	    return (u_char *) &long_return;
 	case IPFRAGFAILS:
 #ifdef MIB_IPCOUNTER_SYMBOL
 	    long_return = MIB_ipcounter[18];
 #else
-	    long_return = 0;
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 #endif
 	    return (u_char *) &long_return;
 	case IPFRAGCREATES:
 #ifdef MIB_IPCOUNTER_SYMBOL
 	    long_return = MIB_ipcounter[19];
 #else
-	    long_return = 0;
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 #endif
 	    return (u_char *) &long_return;
 	case IPROUTEDISCARDS:
-	    long_return = 0;
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    return (u_char *) &long_return;
 	default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ip\n", vp->magic));
     }
     return NULL;
 }
@@ -417,9 +456,9 @@ var_ip(struct variable *vp,
 u_char *
 var_ip(struct variable *vp,
        oid *name,
-       int *length,
+       size_t *length,
        int exact,
-       int *var_len,
+       size_t *var_len,
        WriteMethod **write_method)
 {
     static struct kna tcpipstats;
@@ -432,10 +471,11 @@ var_ip(struct variable *vp,
      *	Get the IP statistics from the kernel...
      */
     if (sysmp (MP_SAGET, MPSA_TCPIPSTATS, &tcpipstats, sizeof tcpipstats) == -1) {
-	perror ("sysmp(MP_SAGET)(MPSA_TCPIPSTATS)");
+	snmp_log_perror ("sysmp(MP_SAGET)(MPSA_TCPIPSTATS)");
     }
 #define ipstat tcpipstats.ipstat
 
+    long_return = 0;
     switch (vp->magic){
 	case IPFORWARDING:
 #if defined(HAVE_SYS_SYSCTL_H) && defined(CTL_NET)
@@ -448,7 +488,7 @@ var_ip(struct variable *vp,
 			&result, &result_size,
 			0, 0) == -1)
 	      {
-		fprintf (stderr, "sysctl(CTL_NET,PF_NET,IPPROTO_IP,IPCTL_FORWARDING)\n");
+	 snmp_log(LOG_ERR, "sysctl(CTL_NET,PF_NET,IPPROTO_IP,IPCTL_FORWARDING)\n");
 	      }
 	    else
 	      {
@@ -462,14 +502,11 @@ var_ip(struct variable *vp,
 #else /* not (HAVE_SYS_SYSCTL_H && CTL_NET) */
 #ifndef sparc	  
             auto_nlist(IP_FORWARDING_SYMBOL,(char *) &i, sizeof(i));
-	    fflush(stderr);
 	    if (i) {
 		long_return = 1;		/* GATEWAY */
 	    } else {
 		long_return = 2;	    /* HOST    */
 	    }
-#else /* sparc */
-	    long_return = 0;
 #endif /* sparc */
 #endif /* not (HAVE_SYS_SYSCTL_H && CTL_NET) */
 
@@ -480,7 +517,11 @@ var_ip(struct variable *vp,
 	     */
 	    if (!auto_nlist(TCP_TTL_SYMBOL,(char *) &long_return,
                             sizeof(long_return)))
+#if NO_DUMMY_VALUES
+	      return NULL;
+#else
               long_return = 60;	    /* XXX */
+#endif
 	    return (u_char *) &long_return;
 	case IPINRECEIVES:
           long_return = ipstat.ips_total;
@@ -531,7 +572,9 @@ var_ip(struct variable *vp,
 	      - (ipstat.ips_fragdropped + ipstat.ips_fragtimeout);
 	    return (u_char *) &long_return;
 	case IPFRAGFAILS:
-	    long_return = 0;
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    return (u_char *) &long_return;
 	case IPFRAGCREATES:
           long_return = ipstat.ips_ofragments;
@@ -540,7 +583,7 @@ var_ip(struct variable *vp,
           long_return = ipstat.ips_noroute;
           return (u_char *) &long_return;
 	default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ip\n", vp->magic));
     }
 }
 
@@ -551,9 +594,9 @@ var_ip(struct variable *vp,
 u_char *
 var_ip(struct variable *vp,
        oid *name,
-       int *length,
+       size_t *length,
        int exact,
-       int *var_len,
+       size_t *var_len,
        WriteMethod **write_method)
 {
     static struct ip_mib ipstat;
@@ -589,7 +632,7 @@ var_ip(struct variable *vp,
 	case IPFRAGFAILS: return (u_char *) &ipstat.IpFragFails;
 	case IPFRAGCREATES: return (u_char *) &ipstat.IpFragCreates;
 	default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ip\n", vp->magic));
     }
     return NULL;
 }
@@ -617,9 +660,9 @@ static int Address_Scan_Next (short *, struct in_ifaddr *);
 u_char *
 var_ipAddrEntry(struct variable *vp,
 		oid *name,
-		int *length,
+		size_t *length,
 		int exact,
-		int *var_len,
+		size_t *var_len,
 		WriteMethod **write_method)
 {
     /*
@@ -731,14 +774,17 @@ var_ipAddrEntry(struct variable *vp,
 #if defined(linux) || defined(sunV3)
 	    long_return = ntohl(((struct sockaddr_in *) &lowin_ifnet.ifu_broadaddr)->sin_addr.s_addr) & 1;
 #else
-          long_return = ntohl(((struct sockaddr_in *) &lowin_ifaddr.ia_broadaddr)->sin_addr.s_addr) & 1;
+	    long_return = ntohl(((struct sockaddr_in *) &lowin_ifaddr.ia_broadaddr)->sin_addr.s_addr) & 1;
 #endif
 	    return(u_char *) &long_return;	   
 	case IPADREASMMAX:
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = -1;
 	    return(u_char *) &long_return;
 	default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ipAddrEntry\n", vp->magic));
     }
     return NULL;
 }
@@ -759,7 +805,7 @@ struct in_ifaddr *Retin_ifaddr;
 {
       struct in_ifaddr in_ifaddr;
       struct ifnet ifnet,*ifnetaddr;  /* NOTA: same name as another one */
-      short index=1;
+      short iindex=1;
 
       while (in_ifaddraddr) {
           /*
@@ -780,13 +826,13 @@ struct in_ifaddr *Retin_ifaddr;
 	  while (ifnetaddr && ifnetaddr != in_ifaddr.ia_ifp) {
 	      klookup((unsigned long)ifnetaddr, (char *)&ifnet, sizeof ifnet);
 	      ifnetaddr = ifnet.if_next;
-	      index++;
+	      iindex++;
 	  }
 
 	  /* XXX - might not find it? */
 
 	  if (Index)
-                  *Index = index;
+                  *Index = iindex;
 
           return(1);  /* DONE */
       }
@@ -800,9 +846,9 @@ struct in_ifaddr *Retin_ifaddr;
 u_char *
 var_ip(struct variable *vp,
 		oid *name,
-		int *length,
+		size_t *length,
 		int exact,
-		int *var_len,
+		size_t *var_len,
 		WriteMethod **write_method)
 {
     mib2_ip_t ipstat;
@@ -877,7 +923,7 @@ var_ip(struct variable *vp,
       	    break;
 	default:
 	    ret = NULL;		/* Failure */
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ip\n", vp->magic));
     }
     return (ret);
 }
@@ -896,9 +942,9 @@ IP_Cmp(void *addr, void *ep)
 u_char *
 var_ipAddrEntry(struct variable *vp,
 		oid *name,
-		int *length,
+		size_t *length,
 		int exact,
-		int *var_len,
+		size_t *var_len,
 		WriteMethod **write_method)
 {
     /*
@@ -915,14 +961,13 @@ var_ipAddrEntry(struct variable *vp,
     mib2_ipAddrEntry_t	    entry, Lowentry;
     int			    Found = 0;
     req_e		    req_type;
-    char		    c_oid[SPRINT_MAX_LEN];
     
     /* fill in object part of name for current (less sizeof instance part) */
 
-    if (snmp_get_do_debugging()) {
-      sprint_objid (c_oid, name, *length);
-      DEBUGMSGTL(("mibII/ip", "var_ipAddrEntry: %s %d\n", c_oid, exact));
-    }
+    DEBUGMSGTL(("mibII/ip", "var_ipAddrEntry: "));
+    DEBUGMSGOID(("mibII/ip", name, *length));
+    DEBUGMSG(("mibII/ip"," %d\n", exact));
+
     memset (&Lowentry, 0, sizeof (Lowentry));
     memcpy( (char *)current,(char *)vp->name, (int)vp->namelen * sizeof(oid));
     if (*length == IP_ADDRNAME_LENGTH) /* Assume that the input name is the lowest */
@@ -978,7 +1023,7 @@ var_ipAddrEntry(struct variable *vp,
 	    long_return = Lowentry.ipAdEntBcastAddr;
 	    return(u_char *) &long_return;	   
 	default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ipAddrEntry\n", vp->magic));
     }
     return NULL;
 }
@@ -1031,13 +1076,12 @@ linux_read_ip_stat (struct ip_mib *ipstat)
 
 #else /* CAN_USE_SYSCTL && IPCTL_STATS */
 
-
 u_char *
 var_ip(struct variable *vp,
        oid *name,
-       int *length,
+       size_t *length,
        int exact,
-       int *var_len,
+       size_t *var_len,
        WriteMethod **write_method)
 {
 	struct ipstat ipstat;
@@ -1103,6 +1147,9 @@ var_ip(struct variable *vp,
 		return (u_char *) &long_return;
 
 	case IPINDISCARDS:
+#if NO_DUMMY_VALUES
+		return NULL;
+#endif
 		long_return = 0;
 		return (u_char *) &long_return;
 
@@ -1151,11 +1198,14 @@ var_ip(struct variable *vp,
 		return (u_char *) &long_return;
 
 	case IPROUTEDISCARDS:
+#if NO_DUMMY_VALUES
+		return NULL;
+#endif
 		long_return = 0;
 		return (u_char *) &long_return;
 
 	default:
-		ERROR_MSG("");
+		DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ip\n", vp->magic));
 	}
 	return NULL;
 }
@@ -1285,9 +1335,9 @@ loop:
 u_char *
 var_ipAddrEntry(struct variable *vp,
 		oid *name,
-		int *length,
+		size_t *length,
 		int exact,
-		int *var_len,
+		size_t *var_len,
 		WriteMethod **write_method)
 {
 	/*
@@ -1363,11 +1413,14 @@ var_ipAddrEntry(struct variable *vp,
 		return (u_char *)&long_return;	   
 
 	case IPADREASMMAX:
+#if NO_DUMMY_VALUES
+		return NULL;
+#endif
 		long_return = -1;
 		return (u_char *)&long_return;
 
 	default:
-		ERROR_MSG("");
+		DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ipAddrEntry\n", vp->magic));
 	}
 	return NULL;
 }
