@@ -6,7 +6,6 @@
 
 #include <config.h>
 #include <unistd.h>
-#include "mib_module_config.h"
 #if TIME_WITH_SYS_TIME
 # ifdef WIN32
 #  include <sys/timeb.h>
@@ -23,7 +22,9 @@
 #endif
 
 #include <sys/types.h>
+#if HAVE_SYS_PARAM_H
 #include <sys/param.h>
+#endif
 #include <sys/socket.h>
 #if HAVE_SYS_PROTOSW_H
 #include <sys/protosw.h>
@@ -46,20 +47,28 @@
 #if HAVE_SYS_STREAM_H
 #include <sys/stream.h>
 #endif
-#ifdef HAVE_NET_ROUTE_H
+#if HAVE_NET_ROUTE_H
 #include <net/route.h>
 #endif
 #if HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
+#if HAVE_NETINET_IN_SYSTM_H
 #include <netinet/in_systm.h>
+#endif
 #include <netinet/ip.h>
+#ifdef INET6
+#include <netinet/ip6.h>
+#endif
 #if HAVE_SYS_QUEUE_H
 #include <sys/queue.h>
 #endif
 #if HAVE_NETINET_IP_VAR_H
 #include <netinet/ip_var.h>
 #endif
+#ifdef INET6
+#include <netinet/ip6_var.h>
+#endif
 #if HAVE_SYS_SOCKETVAR_H
 #include <sys/socketvar.h>
 #endif
@@ -69,20 +78,19 @@
 #if HAVE_INET_MIB2_H
 #include <inet/mib2.h>
 #endif
-#if STDC_HEADERS
+#if HAVE_STRING_H
 #include <string.h>
-#include <stdlib.h>
-#else
+#endif
 #if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#endif
 #ifdef solaris2
 #include "kernel_sunos5.h"
 #else
 #include "kernel.h"
 #endif
-#include "../mibincl.h"
+
+#include "mibincl.h"
 #include "../../../snmplib/system.h"
 #if HAVE_SYS_SYSCTL_H
 #include <sys/sysctl.h>
@@ -115,6 +123,10 @@
 #include <sys/tcpipstats.h>
 #endif
 
+#if HAVE_DMALLOC_H
+#include <dmalloc.h>
+#endif
+
 #include "auto_nlist.h"
 
 #ifdef hpux
@@ -124,6 +136,7 @@
 
 /* #include "../common_header.h" */
 #include "tcp.h"
+#include "sysORTable.h"
 
 	/*********************
 	 *
@@ -134,6 +147,9 @@
 
 #ifdef linux
 static void linux_read_tcp_stat (struct tcp_mib *);
+#endif
+#ifdef freebsd4
+static unsigned int hz;
 #endif
 
 	/*********************
@@ -171,13 +187,16 @@ struct variable13 tcp_variables[] = {
 };
 
 /* Define the OID pointer to the top of the mib tree that we're
-   registering underneath */
-oid tcp_variables_oid[] = { 1,3,6,1,2,1,6 };
+   registering underneath, and the OID for the MIB module */
+oid tcp_variables_oid[] = { SNMP_OID_MIB2,6 };
+oid tcp_module_oid[]    = { SNMP_OID_MIB2,49 };
 
 void init_tcp(void)
 {
   /* register ourselves with the agent to handle our mib tree */
   REGISTER_MIB("mibII/tcp", tcp_variables, variable13, tcp_variables_oid);
+  REGISTER_SYSOR_ENTRY( tcp_module_oid,
+		"The MIB module for managing TCP implementations");
 
 #ifdef TCPSTAT_SYMBOL
   auto_nlist( TCPSTAT_SYMBOL,0,0 );
@@ -185,7 +204,9 @@ void init_tcp(void)
 #ifdef TCP_SYMBOL
   auto_nlist( TCP_SYMBOL,0,0 );
 #endif
-
+#if freebsd4
+  hz=sysconf(_SC_CLK_TCK); /* get ticks/s from system */
+#endif
 }
 
 /*
@@ -203,27 +224,25 @@ void init_tcp(void)
 static int
 header_tcp(struct variable *vp,
 	   oid *name,
-	   int *length,
+	   size_t *length,
 	   int exact,
-	   int *var_len,
+	   size_t *var_len,
 	   WriteMethod **write_method)
 {
 #define TCP_NAME_LENGTH	8
     oid newname[MAX_OID_LEN];
     int result;
-    char c_oid[SPRINT_MAX_LEN];
 
-    if (snmp_get_do_debugging()) {
-      sprint_objid (c_oid, name, *length);
-      DEBUGMSGTL(("mibII/tcp", "var_tcp: %s %d\n", c_oid, exact));
-    }
+    DEBUGMSGTL(("mibII/tcp", "var_tcp: "));
+    DEBUGMSGOID(("mibII/tcp", name, *length));
+    DEBUGMSG(("mibII/tcp"," %d\n", exact));
 
-    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
+    memcpy( (char *)newname,(char *)vp->name, vp->namelen * sizeof(oid));
     newname[TCP_NAME_LENGTH] = 0;
-    result = snmp_oid_compare(name, *length, newname, (int)vp->namelen + 1);
+    result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
     if ((exact && (result != 0)) || (!exact && (result >= 0)))
         return(MATCH_FAILED);
-    memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
+    memcpy( (char *)name,(char *)newname, (vp->namelen + 1) * sizeof(oid));
     *length = vp->namelen + 1;
 
     *write_method = 0;
@@ -244,12 +263,19 @@ header_tcp(struct variable *vp,
 u_char *
 var_tcp(struct variable *vp,
 	oid *name,
-	int *length,
+	size_t *length,
 	int exact,
-	int *var_len,
+	size_t *var_len,
 	WriteMethod **write_method)
 {
     static struct kna tcpipstats;
+#ifdef TCPTV_NEEDS_HZ
+    /*
+     * I don't know of any such system now, but maybe they'll figure
+     * it out some day.
+     */
+    int hz = 1000;
+#endif
 
     /*
      *	Allow for a kernel w/o TCP
@@ -263,7 +289,7 @@ var_tcp(struct variable *vp,
 	 */
 
     if (sysmp (MP_SAGET, MPSA_TCPIPSTATS, &tcpipstats, sizeof tcpipstats) == -1) {
-	perror ("sysmp(MP_SAGET)(MPSA_TCPIPSTATS)");
+	snmp_log_perror ("sysmp(MP_SAGET)(MPSA_TCPIPSTATS)");
     }
 #define tcpstat tcpipstats.tcpstat
 
@@ -282,20 +308,31 @@ var_tcp(struct variable *vp,
 		return (u_char *) &long_return;
 	    case TCPRTOMIN:
 #ifndef linux
+#ifdef TCPTV_NEEDS_HZ
+		long_return = TCPTV_MIN;
+#else
 		long_return = TCPTV_MIN / PR_SLOWHZ * 1000;
+#endif
 		return (u_char *) &long_return;
 #else
 		return (u_char *) &tcpstat.TcpRtoMin;
 #endif
 	    case TCPRTOMAX:
 #ifndef linux
+#ifdef TCPTV_NEEDS_HZ
+		long_return = TCPTV_REXMTMAX;
+#else
 		long_return = TCPTV_REXMTMAX / PR_SLOWHZ * 1000;
+#endif
 		return (u_char *) &long_return;
 #else
 		return (u_char *) &tcpstat.TcpRtoMax;
 #endif
 	    case TCPMAXCONN:
 #ifndef linux
+#if NO_DUMMY_VALUES
+		return NULL;
+#endif
 		long_return = -1;
 		return (u_char *) &long_return;
 #else
@@ -311,6 +348,9 @@ var_tcp(struct variable *vp,
 #ifdef hpux
 		long_return = MIB_tcpcounter[7];
 #else
+#if NO_DUMMY_VALUES
+		return NULL;
+#endif
 		long_return = tcpstat.tcps_conndrops;	/* XXX */
 #endif
 		return (u_char *) &long_return;
@@ -318,6 +358,9 @@ var_tcp(struct variable *vp,
 #ifdef hpux
 		long_return = MIB_tcpcounter[8];
 #else
+#if NO_DUMMY_VALUES
+		return NULL;
+#endif
 		long_return = tcpstat.tcps_drops;	/* XXX */
 #endif
 		return (u_char *) &long_return;
@@ -359,7 +402,7 @@ var_tcp(struct variable *vp,
 		return (u_char *) &long_return;
 #endif /* linux */
 	    default:
-		ERROR_MSG("");
+		DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_tcp\n", vp->magic));
 	}
     return NULL;
 }
@@ -369,15 +412,18 @@ var_tcp(struct variable *vp,
 u_char *
 var_tcp(struct variable *vp,
 	oid *name,
-	int *length,
+	size_t *length,
 	int exact,
-	int *var_len,
+	size_t *var_len,
 	WriteMethod **write_method)
 {
     static struct tcpstat tcpstat;
 #ifdef hpux
     static	counter MIB_tcpcounter[MIB_tcpMAXCTR+1];
 #endif
+#ifdef TCPTV_NEEDS_HZ
+    int hz = 1000;
+#endif
 
     /*
      *	Allow for a kernel w/o TCP
@@ -432,20 +478,31 @@ var_tcp(struct variable *vp,
 		return (u_char *) &long_return;
 	    case TCPRTOMIN:
 #ifndef linux
+#ifdef TCPTV_NEEDS_HZ
+		long_return = TCPTV_MIN;
+#else
 		long_return = TCPTV_MIN / PR_SLOWHZ * 1000;
+#endif
 		return (u_char *) &long_return;
 #else
 		return (u_char *) &tcpstat.TcpRtoMin;
 #endif
 	    case TCPRTOMAX:
 #ifndef linux
+#ifdef TCPTV_NEEDS_HZ
+		long_return = TCPTV_REXMTMAX;
+#else
 		long_return = TCPTV_REXMTMAX / PR_SLOWHZ * 1000;
+#endif
 		return (u_char *) &long_return;
 #else
 		return (u_char *) &tcpstat.TcpRtoMax;
 #endif
 	    case TCPMAXCONN:
 #ifndef linux
+#if NO_DUMMY_VALUES
+		return NULL;
+#endif
 		long_return = -1;
 		return (u_char *) &long_return;
 #else
@@ -461,6 +518,9 @@ var_tcp(struct variable *vp,
 #ifdef hpux
 		long_return = MIB_tcpcounter[7];
 #else
+#if NO_DUMMY_VALUES
+		return NULL;
+#endif
 		long_return = tcpstat.tcps_conndrops;	/* XXX */
 #endif
 		return (u_char *) &long_return;
@@ -468,6 +528,9 @@ var_tcp(struct variable *vp,
 #ifdef hpux
 		long_return = MIB_tcpcounter[8];
 #else
+#if NO_DUMMY_VALUES
+		return NULL;
+#endif
 		long_return = tcpstat.tcps_drops;	/* XXX */
 #endif
 		return (u_char *) &long_return;
@@ -509,7 +572,7 @@ var_tcp(struct variable *vp,
 		return (u_char *) &long_return;
 #endif /* linux */
 	    default:
-		ERROR_MSG("");
+		DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_tcp\n", vp->magic));
 	}
     return NULL;
 }
@@ -519,9 +582,9 @@ var_tcp(struct variable *vp,
 u_char *
 var_tcpEntry(struct variable *vp,
 	     oid *name,
-	     int *length,
+	     size_t *length,
 	     int exact,
-	     int *var_len,
+	     size_t *var_len,
 	     WriteMethod **write_method)
 {
     int i;
@@ -588,7 +651,7 @@ LowState = -1;	    /* Don't have one yet */
 	    }
 	}
 	if (LowState < 0) return(NULL);
-	memcpy( (char *)name,(char *)lowest, ((int)vp->namelen + 10) * sizeof(oid));
+	memcpy( (char *)name,(char *)lowest, (vp->namelen + 10) * sizeof(oid));
 	*length = vp->namelen + 10;
 	*write_method = 0;
 	*var_len = sizeof(long);
@@ -631,9 +694,9 @@ TCP_Cmp(void *addr, void *ep)
 u_char *
 var_tcp(struct variable *vp,
 	oid *name,
-	int *length,
+	size_t *length,
 	int exact,
-	int *var_len,
+	size_t *var_len,
 	WriteMethod **write_method)
 {
   mib2_tcp_t tcpstat;
@@ -691,7 +754,7 @@ var_tcp(struct variable *vp,
       long_return = ipstat.tcpInErrs;
       return(u_char *) &long_return;
     default:
-      ERROR_MSG("");
+      DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_tcp\n", vp->magic));
       return (NULL);
     }
 }
@@ -700,9 +763,9 @@ var_tcp(struct variable *vp,
 u_char *
 var_tcpEntry(struct variable *vp,
 	     oid *name,
-	     int *length,
+	     size_t *length,
 	     int exact,
-	     int *var_len,
+	     size_t *var_len,
 	     WriteMethod **write_method)
 {
   oid newname[MAX_OID_LEN], lowest[MAX_OID_LEN], *op;
@@ -718,7 +781,7 @@ var_tcpEntry(struct variable *vp,
     int			Found = 0;
     
     memset (&Lowentry, 0, sizeof (Lowentry));
-    memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
+    memcpy( (char *)newname,(char *)vp->name, vp->namelen * sizeof(oid));
     if (*length == TCP_CONN_LENGTH) /* Assume that the input name is the lowest */
       memcpy( (char *)lowest,(char *)name, TCP_CONN_LENGTH * sizeof(oid));
     for (Nextentry.tcpConnLocalAddress = (u_long)-1, req_type = GET_FIRST;
@@ -757,7 +820,7 @@ var_tcpEntry(struct variable *vp,
     if (Found == 0)
       return(NULL);
     memcpy((char *)name, (char *)lowest,
-	  ((int)vp->namelen + TCP_CONN_LENGTH - TCP_LOCADDR_OFF) * sizeof(oid));
+	  (vp->namelen + TCP_CONN_LENGTH - TCP_LOCADDR_OFF) * sizeof(oid));
     *length = vp->namelen + TCP_CONN_LENGTH - TCP_LOCADDR_OFF;
     *write_method = 0;
     *var_len = sizeof(long);
@@ -778,7 +841,7 @@ var_tcpEntry(struct variable *vp,
       long_return = Lowentry.tcpConnRemPort;
       return(u_char *) &long_return;
     default:
-      ERROR_MSG("");
+      DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_tcpEntry\n", vp->magic));
       return (NULL);
     }
 }
@@ -862,7 +925,7 @@ Again:	/*
 		next = inpcb.INP_NEXT_SYMBOL;
 
 		if((klookup((unsigned long)next, (char *)&inpcb, sizeof (inpcb)) == 0)) {
-		    perror("TCP_Count_Connections - inpcb");
+		    snmp_log_perror("TCP_Count_Connections - inpcb");
 		    break;
 		}
 #if !(defined(freebsd2) || defined(netbsd1) || defined(openbsd2))
@@ -878,7 +941,7 @@ Again:	/*
 			continue;
 		}
 		if(klookup((unsigned long)inpcb.inp_ppcb, (char *)&tcpcb, sizeof (tcpcb)) == 0) {
-		    perror("TCP_Count_Connections - tcpcb");
+		    snmp_log_perror("TCP_Count_Connections - tcpcb");
 		    break;
 		}
 
@@ -908,7 +971,7 @@ static struct xinpgen *xig = NULL;
 
 void TCP_Scan_Init (void)
 {
-#if  !defined(CAN_USE_SYSCTL) || !defined(TCPCTL_PCBLIST)
+#if !defined(CAN_USE_SYSCTL) || !defined(TCPCTL_PCBLIST)
 #ifdef PCB_TABLE
     struct inpcbtable table;
 #endif
@@ -944,7 +1007,7 @@ void TCP_Scan_Init (void)
 
     if (! (in = fopen ("/proc/net/tcp", "r")))
       {
-	fprintf (stderr, "snmpd: cannot open /proc/net/tcp ...\n");
+ snmp_log(LOG_ERR, "snmpd: cannot open /proc/net/tcp ...\n");
 	tcp_prev = NULL;
 	return;
       }
@@ -981,6 +1044,7 @@ void TCP_Scan_Init (void)
 	pcb.uid = uid;
     
 	nnew = (struct inpcb *) malloc (sizeof (struct inpcb));
+	if (nnew == NULL) break;
 	*nnew = pcb;
 	nnew->INP_NEXT_SYMBOL = 0;
 
