@@ -3,19 +3,17 @@
  *
  */
 
-#include "mib_module_config.h"
-
 #include <config.h>
-#if STDC_HEADERS
+#if HAVE_STRING_H
 #include <string.h>
-#include <stdlib.h>
-#else
+#endif
 #if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#endif
 #include <sys/types.h>
+#if HAVE_SYS_PARAM_H
 #include <sys/param.h>
+#endif
 #include <sys/socket.h>
 
 #if TIME_WITH_SYS_TIME
@@ -57,11 +55,12 @@
 #if HAVE_SYS_STREAM_H
 #include <sys/stream.h>
 #endif
-#ifdef HAVE_NET_ROUTE_H
+#if HAVE_NET_ROUTE_H
 #include <net/route.h>
 #endif
-
+#if HAVE_NETINET_IN_SYSTM_H
 #include <netinet/in_systm.h>
+#endif
 #include <netinet/ip.h>
 #if HAVE_SYS_QUEUE_H
 #include <sys/queue.h>
@@ -82,16 +81,23 @@
 #if HAVE_INET_MIB2_H
 #include <inet/mib2.h>
 #endif
+
+#if HAVE_DMALLOC_H
+#include <dmalloc.h>
+#endif
+
+#include "mibincl.h"
+
 #ifdef solaris2
 #include "kernel_sunos5.h"
 #else
 #include "kernel.h"
 #endif
-#include "../../../snmplib/system.h"
+
+#include "system.h"
 #include "asn1.h"
 #include "snmp_debug.h"
 
-#include "mibincl.h"
 #include "auto_nlist.h"
 
 #ifdef hpux
@@ -104,6 +110,7 @@
 #include "tcp.h"
 #endif
 #include "udp.h"
+#include "sysORTable.h"
 
 #ifdef CAN_USE_SYSCTL
 #include <sys/sysctl.h>
@@ -140,14 +147,17 @@ struct variable8 udp_variables[] = {
 };
 
 /* Define the OID pointer to the top of the mib tree that we're
-   registering underneath */
-oid udp_variables_oid[] = { 1,3,6,1,2,1,7 };
+   registering underneath, and the OID for the MIB module */
+oid udp_variables_oid[] = { SNMP_OID_MIB2,7 };
+oid udp_module_oid[]    = { SNMP_OID_MIB2,50 };
 
 void init_udp(void)
 {
 
   /* register ourselves with the agent to handle our mib tree */
   REGISTER_MIB("mibII/udp", udp_variables, variable8, udp_variables_oid);
+  REGISTER_SYSOR_ENTRY( udp_module_oid,
+		"The MIB module for managing UDP implementations");
 
 #ifdef UDPSTAT_SYMBOL
   auto_nlist( UDPSTAT_SYMBOL,0,0 );
@@ -172,20 +182,18 @@ void init_udp(void)
 static int
 header_udp(struct variable *vp,
 	   oid *name,
-	   int *length,
+	   size_t *length,
 	   int exact,
-	   int *var_len,
+	   size_t *var_len,
 	   WriteMethod **write_method)
 {
 #define UDP_NAME_LENGTH	8
     oid newname[MAX_OID_LEN];
     int result;
-    char c_oid[SPRINT_MAX_LEN];
 
-    if (snmp_get_do_debugging()) {
-      sprint_objid (c_oid, name, *length);
-      DEBUGMSGTL(("mibII/udp", "var_udp: %s %d\n", c_oid, exact));
-    }
+    DEBUGMSGTL(("mibII/udp", "var_udp: "));
+    DEBUGMSGOID(("mibII/udp", name, *length));
+    DEBUGMSG(("mibII/udp"," %d\n", exact));
 
     memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));
     newname[UDP_NAME_LENGTH] = 0;
@@ -214,9 +222,9 @@ header_udp(struct variable *vp,
 u_char *
 var_udp(struct variable *vp,
 	oid *name,
-	int *length,
+	size_t *length,
 	int exact,
-	int *var_len,
+	size_t *var_len,
 	WriteMethod **write_method)
 {
 #ifdef linux
@@ -258,6 +266,9 @@ var_udp(struct variable *vp,
 #if defined(linux)
 	    long_return = udpstat.UdpInDatagrams;
 #else
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = 0;
 #endif
 #endif
@@ -269,6 +280,9 @@ var_udp(struct variable *vp,
 #if defined(linux)
 	    long_return = udpstat.UdpNoPorts;
 #else
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = 0;
 #endif
 #endif
@@ -280,6 +294,9 @@ var_udp(struct variable *vp,
 #if defined(linux)
 	    long_return = udpstat.UdpOutDatagrams;
 #else
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = 0;
 #endif
 #endif
@@ -297,7 +314,7 @@ var_udp(struct variable *vp,
 #endif /* linux */
 
 	default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_udp\n", vp->magic));
     }
     return NULL;
 }
@@ -308,9 +325,9 @@ var_udp(struct variable *vp,
 u_char *
 var_udp(struct variable *vp,
 	oid *name,
-	int *length,
+	size_t *length,
 	int exact,
-	int *var_len,
+	size_t *var_len,
 	WriteMethod **write_method)
 {
     static struct kna tcpipstats;
@@ -319,7 +336,7 @@ var_udp(struct variable *vp,
 	return NULL;
 
     if (sysmp (MP_SAGET, MPSA_TCPIPSTATS, &tcpipstats, sizeof tcpipstats) == -1) {
-	perror ("sysmp(MP_SAGET)(MPSA_TCPIPSTATS)");
+	snmp_log_perror ("sysmp(MP_SAGET)(MPSA_TCPIPSTATS)");
     }
 #define udpstat tcpipstats.udpstat
 
@@ -338,7 +355,7 @@ var_udp(struct variable *vp,
 	      udpstat.udps_badlen;
 	    return (u_char *) &long_return;
 	default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_udp\n", vp->magic));
     }
     return NULL;
 }
@@ -350,9 +367,9 @@ var_udp(struct variable *vp,
 u_char *
 var_udp(struct variable *vp,
 	oid *name,
-	int *length,
+	size_t *length,
 	int exact,
-	int *var_len,
+	size_t *var_len,
 	WriteMethod **write_method)
 {
     static struct udpstat udpstat;
@@ -388,7 +405,7 @@ var_udp(struct variable *vp,
 	    return (u_char *) &long_return;
 
 	default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_udp\n", vp->magic));
     }
     return NULL;
 }
@@ -400,9 +417,9 @@ var_udp(struct variable *vp,
 u_char *
 var_udpEntry(struct variable *vp,
 	     oid *name,
-	     int *length,
+	     size_t *length,
 	     int exact,
-	     int *var_len,
+	     size_t *var_len,
 	     WriteMethod **write_method)
 {
     int i;
@@ -460,7 +477,7 @@ LowState = -1;		/* UDP doesn't have 'State', but it's a useful flag */
 		long_return = ntohs(Lowinpcb.inp_lport);
 		return (u_char *) &long_return;
 	    default:
-		ERROR_MSG("");
+		DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_udpEntry\n", vp->magic));
 	}
     return  NULL;
 }
@@ -470,9 +487,9 @@ LowState = -1;		/* UDP doesn't have 'State', but it's a useful flag */
 u_char *
 var_udp(struct variable *vp,
 	oid *name,
-	int *length,
+	size_t *length,
 	int exact,
-	int *var_len,
+	size_t *var_len,
 	WriteMethod **write_method)
 {
     mib2_udp_t udpstat;
@@ -506,7 +523,7 @@ var_udp(struct variable *vp,
       		break;
 	default:
 		ret = NULL;
-		ERROR_MSG("");
+		DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_udp\n", vp->magic));
     }
     return (ret);
 }
@@ -514,9 +531,9 @@ var_udp(struct variable *vp,
 u_char *
 var_udpEntry(struct variable *vp,
 	     oid *name,
-	     int *length,
+	     size_t *length,
 	     int exact,
-	     int *var_len,
+	     size_t *var_len,
 	     WriteMethod **write_method)
 {
     return NULL;
@@ -583,7 +600,7 @@ static void UDP_Scan_Init(void)
 
     if (! (in = fopen ("/proc/net/udp", "r")))
       {
-	fprintf (stderr, "snmpd: cannot open /proc/net/udp ...\n");
+ snmp_log(LOG_ERR, "snmpd: cannot open /proc/net/udp ...\n");
 	udp_prev = 0;
 	return;
       }
@@ -616,6 +633,7 @@ static void UDP_Scan_Init(void)
 	pcb.inp_fport = htons (pcb.inp_fport);
 
 	nnew = (struct inpcb *) malloc (sizeof (struct inpcb));
+	if (nnew == NULL) break;
 	*nnew = pcb;
 	nnew->inp_next = 0;
 
