@@ -27,18 +27,33 @@
 #include <arpa/inet.h>
 #endif
 
+#if HAVE_DMALLOC_H
+#include <dmalloc.h>
+#endif
+
+#if HAVE_NETDB_H
 #include <netdb.h>
+#endif
+#if HAVE_WINSOCK_H
+#include <winsock.h>
+#endif
 
 #include "mibincl.h"
 #include "read_config.h"
 #include "agent_read_config.h"
-#include "../../../snmplib/system.h"
+#include "system.h"
 #include "vacm.h"
+#include "callback.h"
+#include "agent_registry.h"
+#include "agent_callbacks.h"
 #include "vacm_vars.h"
-#include "../../mib_module_config.h"
 #ifdef USING_MIBII_SYSORTABLE_MODULE
 #if TIME_WITH_SYS_TIME
-# include <sys/time.h>
+# ifdef WIN32
+#  include <sys/timeb.h>
+# else
+#  include <sys/time.h>
+# endif
 # include <time.h>
 #else
 # if HAVE_SYS_TIME_H
@@ -50,30 +65,23 @@
 #include "sysORTable.h"
 #endif
 
-#include "vacm.h"
-
 void
 init_vacm_vars (void) 
 {
 
 #ifdef USING_MIBII_SYSORTABLE_MODULE
-  static oid reg[] = {1,3,6,1,6,3,13};
+  static oid reg[] = {SNMP_OID_SNMPMODULES,16,2,2,1};
 #endif
   
 #define PRIVRW	(SNMPV2ANY | 0x5000)
 
   struct variable2 vacm_sec2group[] = {
-    {SECURITYMODEL, ASN_INTEGER, PRIVRW, var_vacm_sec2group, 1, {1}},
-    {SECURITYNAME, ASN_OCTET_STR, PRIVRW, var_vacm_sec2group, 1, {2}},
     {SECURITYGROUP, ASN_OCTET_STR, PRIVRW, var_vacm_sec2group, 1, {3}},
     {SECURITYSTORAGE, ASN_INTEGER, PRIVRW, var_vacm_sec2group, 1, {4}},
     {SECURITYSTATUS, ASN_INTEGER, PRIVRW, var_vacm_sec2group, 1, {5}},
   };
 
   struct variable2 vacm_access[] = {
-    {ACCESSPREFIX, ASN_OCTET_STR, PRIVRW, var_vacm_access, 1, {1}},
-    {ACCESSMODEL, ASN_INTEGER, PRIVRW, var_vacm_access, 1, {2}},
-    {ACCESSLEVEL, ASN_INTEGER, PRIVRW, var_vacm_access, 1, {3}},
     {ACCESSMATCH, ASN_INTEGER, PRIVRW, var_vacm_access, 1, {4}},
     {ACCESSREAD, ASN_OCTET_STR, PRIVRW, var_vacm_access, 1, {5}},
     {ACCESSWRITE, ASN_OCTET_STR, PRIVRW, var_vacm_access, 1, {6}},
@@ -83,8 +91,6 @@ init_vacm_vars (void)
   };
 
   struct variable2 vacm_view[] = {
-    {VIEWNAME, ASN_OCTET_STR, PRIVRW, var_vacm_view, 1, {1}},
-    {VIEWSUBTREE, ASN_OBJECT_ID, PRIVRW, var_vacm_view, 1, {2}},
     {VIEWMASK, ASN_OCTET_STR, PRIVRW, var_vacm_view, 1, {3}},
     {VIEWTYPE, ASN_INTEGER, PRIVRW, var_vacm_view, 1, {4}},
     {VIEWSTORAGE, ASN_INTEGER, PRIVRW, var_vacm_view, 1, {5}},
@@ -106,26 +112,47 @@ init_vacm_vars (void)
   snmpd_register_config_handler("com2sec", vacm_parse_security,
                                 vacm_free_security,"name source community");
   snmpd_register_config_handler("group", vacm_parse_group, vacm_free_group,
-                                "name v1|v2c|usm|any security");
+                                "name v1|v2c|usm security");
   snmpd_register_config_handler("access", vacm_parse_access, vacm_free_access,
                             "name context model level prefx read write notify");
   snmpd_register_config_handler("view", vacm_parse_view, vacm_free_view,
                                 "name type subtree [mask]");
+  snmpd_register_config_handler("rwcommunity", vacm_parse_simple,
+                                NULL,"community [default|hostname|network/bits] [oid]");
+  snmpd_register_config_handler("rocommunity", vacm_parse_simple,
+                                NULL,"community [default|hostname|network/bits] [oid]");
+  snmpd_register_config_handler("rwuser", vacm_parse_simple,
+                                NULL,"user [noauth|auth|priv] [oid]");
+  snmpd_register_config_handler("rouser", vacm_parse_simple,
+                                NULL,"user [noauth|auth|priv] [oid]");
 
 #ifdef USING_MIBII_SYSORTABLE_MODULE
-  register_sysORTable(reg,7,"View-based Access Control Model for SNMP.");
+  register_sysORTable(reg,10,"View-based Access Control Model for SNMP.");
 #endif
+
+  /* register ourselves to handle access control */
+  snmp_register_callback(SNMP_CALLBACK_APPLICATION, SNMPD_CALLBACK_ACM_CHECK,
+                         vacm_in_view_callback, NULL);
+  snmp_register_callback(SNMP_CALLBACK_APPLICATION,
+                         SNMPD_CALLBACK_ACM_CHECK_INITIAL,
+                         vacm_in_view_callback, NULL);
 }
 
 static struct vacm_securityEntry *securityFirst =0, *securityLast =0;
 
-void vacm_parse_security (char *token, 
+#define EXAMPLE_NETWORK		"NETWORK"
+#define EXAMPLE_COMMUNITY	"COMMUNITY"
+
+void vacm_parse_security (const char *token, 
 			  char *param)
 {
-    char *name, *source, *community, *mask;
+    char *name, *source, *community;
+    const char *mask;
     char *cp;
     struct vacm_securityEntry *sp, se;
     int maskLength, maskBit;
+    struct sockaddr_in *srcIp, *srcMask;
+    char null[] = "";
 
     memset (&se, 0 , sizeof se);
     name = strtok(param, "\t\n ");
@@ -138,33 +165,43 @@ void vacm_parse_security (char *token,
 	config_perror("missing SOURCE parameter");
 	return;
     }
+    if ( !strncmp( source, EXAMPLE_NETWORK, strlen(EXAMPLE_NETWORK)) ) {
+	config_perror("Example config NETWORK not properly configured");
+	return;		/* or exit(1); */
+    }
     community = strtok(NULL, "\t\n ");
     if (!community) {
 	config_perror("missing COMMUNITY parameter");
 	return;
     }
+    if ( !strncmp( community, EXAMPLE_COMMUNITY, strlen(EXAMPLE_COMMUNITY)) ) {
+	config_perror("Example config COMMUNITY not properly configured");
+	return;		/* or exit(1); */
+    }
+    srcIp   = (struct sockaddr_in*)&(se.sourceIp);
+    srcMask = (struct sockaddr_in*)&(se.sourceMask);
     cp = strchr(source, '/');
-    if (cp == NULL) cp = "";
+    if (cp == NULL) cp = null;
     else *cp++ = 0;
     mask = cp;
     if (strcmp("default", source) == 0 || strcmp("0.0.0.0", source) == 0) {
-	memset(&se.sourceIp.sin_addr, 0, sizeof(struct in_addr));
+	memset(&(srcIp->sin_addr), 0, sizeof(struct in_addr));
 	mask = "0.0.0.0";
     }
-    else if ((se.sourceIp.sin_addr.s_addr = inet_addr (source)) == (unsigned) -1) {
+    else if ((srcIp->sin_addr.s_addr = inet_addr (source)) == (unsigned) -1) {
 	struct hostent *hp = gethostbyname(source);
 	if (hp != NULL) {
-	    memcpy(&se.sourceIp.sin_addr , hp->h_addr, 4);
+	    memcpy(&(srcIp->sin_addr), hp->h_addr, 4);
 	}
 	else {
 	    config_perror ("bad source address");
 	    return;
 	}
     }
-    if (*mask == 0) memset (&se.sourceMask.sin_addr, 0xff, sizeof(struct in_addr));
+    if (*mask == 0) memset (&(srcMask->sin_addr), 0xff, sizeof(struct in_addr));
     else {
 	if (strchr(mask, '.')) {
-	    if ((se.sourceMask.sin_addr.s_addr = inet_addr(mask)) == (unsigned)-1) {
+	    if ((srcMask->sin_addr.s_addr = inet_addr(mask)) == (unsigned)-1) {
 		config_perror("bad mask");
 		return;
 	    }
@@ -176,21 +213,33 @@ void vacm_parse_security (char *token,
 		return;
 	    }
 	    maskBit = 0x80000000L;
-	    se.sourceMask.sin_addr.s_addr = 0;
+	    srcMask->sin_addr.s_addr = 0;
 	    while (maskLength--) {
-		se.sourceMask.sin_addr.s_addr |= maskBit;
+		srcMask->sin_addr.s_addr |= maskBit;
 		maskBit >>= 1;
 	    }
-	    se.sourceMask.sin_addr.s_addr = htonl(se.sourceMask.sin_addr.s_addr);
+	    srcMask->sin_addr.s_addr = htonl(srcMask->sin_addr.s_addr);
 	}
     }
-    if ((se.sourceIp.sin_addr.s_addr & ~se.sourceMask.sin_addr.s_addr) != 0) {
+    if ((srcIp->sin_addr.s_addr & ~srcMask->sin_addr.s_addr) != 0) {
 	config_perror("source/mask mismatch");
 	return;
     }
+    if (strlen(name)+1 > sizeof(se.securityName)) {
+    	config_perror("security name too long");
+	return;
+    }
+    if (strlen(community)+1 > sizeof(se.community)) {
+    	config_perror("community name too long");
+	return;
+    }
     strcpy(se.securityName, name);
     strcpy(se.community, community);
     sp = (struct vacm_securityEntry *)malloc (sizeof *sp);
+    if (sp == NULL) {
+    	config_perror("memory error");
+	return;
+    }
     *sp = se;
     if (securityFirst != NULL) {
 	securityLast->next = sp;
@@ -210,7 +259,7 @@ void vacm_free_security (void)
     }
 }
 
-void vacm_parse_group (char *token, 
+void vacm_parse_group (const char *token, 
 		       char *param)
 {
     char *group, *model, *security;
@@ -233,13 +282,19 @@ void vacm_parse_group (char *token,
 	config_perror("missing SECURITY parameter");
 	return;
     }
-    if (strcasecmp(model, "any") == 0) imodel = SNMP_SEC_MODEL_ANY;
-    else if (strcasecmp(model, "v1") == 0) imodel = SNMP_SEC_MODEL_SNMPv1;
+    if (strcasecmp(model, "v1") == 0) imodel = SNMP_SEC_MODEL_SNMPv1;
     else if (strcasecmp(model, "v2c") == 0) imodel = SNMP_SEC_MODEL_SNMPv2c;
-    else if (strcasecmp(model, "v2p") == 0) imodel = SNMP_SEC_MODEL_SNMPv2p;
     else if (strcasecmp(model, "usm") == 0) imodel = SNMP_SEC_MODEL_USM;
+    else if (strcasecmp(model, "any") == 0) {
+	config_perror("bad security model \"any\" should be: v1, v2c or usm - installing anyway");
+	imodel = SNMP_SEC_MODEL_ANY;
+    }
     else {
-	config_perror("bad security model");
+	config_perror("bad security model, should be: v1, v2c or usm");
+	return;
+    }
+    if (strlen(security)+1 > sizeof(gp->groupName)) {
+    	config_perror("security name too long");
 	return;
     }
     gp = vacm_createGroupEntry(imodel, security);
@@ -255,10 +310,10 @@ void vacm_free_group (void)
     vacm_destroyAllGroupEntries();
 }
 
-void vacm_parse_access (char *token, char *param)
+void vacm_parse_access (const char *token, char *param)
 {
     char *name, *context, *model, *level, *prefix, *readView, *writeView, *notify;
-    int imodel, ilevel;
+    int imodel, ilevel, iprefix;
     struct vacm_accessEntry *ap;
 
     name = strtok(param, " \t\n");
@@ -305,26 +360,48 @@ void vacm_parse_access (char *token, char *param)
     if (strcasecmp(model, "any") == 0) imodel = SNMP_SEC_MODEL_ANY;
     else if (strcasecmp(model, "v1") == 0) imodel = SNMP_SEC_MODEL_SNMPv1;
     else if (strcasecmp(model, "v2c") == 0) imodel = SNMP_SEC_MODEL_SNMPv2c;
-    else if (strcasecmp(model, "v2p") == 0) imodel = SNMP_SEC_MODEL_SNMPv2p;
     else if (strcasecmp(model, "usm") == 0) imodel = SNMP_SEC_MODEL_USM;
     else {
-	config_perror("bad security model");
+	config_perror("bad security model (any, v1, v2c, usm)");
 	return;
     }
     if (strcasecmp(level, "noauth") == 0) ilevel = SNMP_SEC_LEVEL_NOAUTH;
     else if (strcasecmp(level, "noauthnopriv") == 0) ilevel = SNMP_SEC_LEVEL_NOAUTH;
     else if (strcasecmp(level, "auth") == 0) ilevel = SNMP_SEC_LEVEL_AUTHNOPRIV;
     else if (strcasecmp(level, "authnopriv") == 0) ilevel = SNMP_SEC_LEVEL_AUTHNOPRIV;
-    else if (strcasecmp(model, "priv") == 0) ilevel = SNMP_SEC_LEVEL_AUTHPRIV;
-    else if (strcasecmp(model, "authpriv") == 0) ilevel = SNMP_SEC_LEVEL_AUTHPRIV;
+    else if (strcasecmp(level, "priv") == 0) ilevel = SNMP_SEC_LEVEL_AUTHPRIV;
+    else if (strcasecmp(level, "authpriv") == 0) ilevel = SNMP_SEC_LEVEL_AUTHPRIV;
     else {
-	config_perror("bad security level");
+	config_perror("bad security level (noauthnopriv, authnopriv, authpriv)");
+	return;
+    }
+    if (strcmp(prefix,"exact") == 0) iprefix = 1;
+    else if (strcmp(prefix,"prefix") == 0) iprefix = 2;
+    else if (strcmp(prefix,"0") == 0) {
+	config_perror("bad prefix match parameter \"0\", should be: exact or prefix - installing anyway");
+	iprefix = 1;
+    }
+    else {
+	config_perror("bad prefix match parameter, should be: exact or prefix");
+	return;
+    }
+    if (strlen(readView)+1 > sizeof(ap->readView)) {
+    	config_perror("readView too long");
+	return;
+    }
+    if (strlen(writeView)+1 > sizeof(ap->writeView)) {
+    	config_perror("writeView too long");
+	return;
+    }
+    if (strlen(notify)+1 > sizeof(ap->notifyView)) {
+    	config_perror("notifyView too long");
 	return;
     }
     ap = vacm_createAccessEntry (name, context, imodel, ilevel);
     strcpy(ap->readView, readView);
     strcpy(ap->writeView, writeView);
     strcpy(ap->notifyView, notify);
+    ap->contextMatch = iprefix;
     ap->storageType = SNMP_STORAGE_PERMANENT;
     ap->status = SNMP_ROW_ACTIVE;
     free (ap->reserved);
@@ -336,14 +413,14 @@ void vacm_free_access (void)
     vacm_destroyAllAccessEntries();
 }
 
-void vacm_parse_view (char *token, 
+void vacm_parse_view (const char *token, 
 		      char *param)
 {
     char *name, *type, *subtree, *mask;
     int inclexcl;
     struct vacm_viewEntry *vp;
     oid suboid[MAX_OID_LEN];
-    int suboid_len = 0;
+    size_t suboid_len = 0;
     u_char viewMask[sizeof (vp->viewMask)];
     int i;
 
@@ -379,7 +456,7 @@ void vacm_parse_view (char *token,
     if (mask) {
 	int val;
 	i = 0;
-	for (mask = strtok(mask, "."); mask; mask = strtok(NULL, ":")) {
+	for (mask = strtok(mask, ".:"); mask; mask = strtok(NULL, ".:")) {
 	    if (i >= sizeof(viewMask)) {
 		config_perror("MASK too long");
 		return;
@@ -410,84 +487,243 @@ void vacm_free_view (void)
     vacm_destroyAllViewEntries();
 }
 
-int vacm_in_view (struct packet_info *pi,
+void vacm_parse_simple(const char *token, char *confline) {
+  char line[SPRINT_MAX_LEN];
+  char community[COMMUNITY_MAX_LEN];
+  char theoid[SPRINT_MAX_LEN];
+  char viewname[SPRINT_MAX_LEN];
+  char addressname[SPRINT_MAX_LEN];
+  const char *rw = "none";
+  const char *model = "any";
+  char *cp;
+  static int num = 0;
+  char secname[SPRINT_MAX_LEN];
+  char authtype[SPRINT_MAX_LEN];
+
+  /* community name or user name */
+  cp = copy_word(confline, community);
+
+  if (strcmp(token,"rouser") == 0 || strcmp(token,"rwuser") == 0) {
+    /* authentication type */
+    if (cp && *cp)
+      cp = copy_word(cp, authtype);
+    else
+      strcpy(authtype, "auth");
+    DEBUGMSGTL((token, "setting auth type: \"%s\"\n",authtype));
+    model = "usm";
+  } else {
+    /* source address */
+    if (cp && *cp) {
+      cp = copy_word(cp, addressname);
+    } else {
+      strcpy(addressname, "default");
+    }
+    /* authtype has to be noauth */
+    strcpy(authtype, "noauth");
+  }
+
+  /* oid they can touch */
+  if (cp && *cp) {
+    cp = copy_word(cp, theoid);
+  } else {
+    strcpy(theoid, ".1");
+  }
+
+  if (strcmp(token,"rwcommunity") == 0 || strcmp(token,"rwuser") == 0)
+    rw = viewname;
+
+  if (strcmp(token,"rwcommunity") == 0 || strcmp(token,"rocommunity") == 0) {
+    /* com2sec mapping */
+    /* com2sec anonymousSecNameNUM    ADDRESS  COMMUNITY */
+    sprintf(secname, "anonymousSecName%03d", num);
+    sprintf(line,"%s %s %s", secname, addressname, community);
+    DEBUGMSGTL((token,"passing: %s %s\n", "com2sec", line));
+    vacm_parse_security("com2sec",line);
+
+    /* sec->group mapping */
+    /* group   anonymousGroupNameNUM  any      anonymousSecNameNUM */
+    sprintf(line,"anonymousGroupName%03d v1 %s", num, secname);
+    DEBUGMSGTL((token,"passing: %s %s\n", "group", line));
+    vacm_parse_group("group",line);
+    sprintf(line,"anonymousGroupName%03d v2c %s", num, secname);
+    DEBUGMSGTL((token,"passing: %s %s\n", "group", line));
+    vacm_parse_group("group",line);
+  } else {
+    strcpy(secname, community);
+
+    /* sec->group mapping */
+    /* group   anonymousGroupNameNUM  any      anonymousSecNameNUM */
+    sprintf(line,"anonymousGroupName%03d usm %s", num, secname);
+    DEBUGMSGTL((token,"passing: %s %s\n", "group", line));
+    vacm_parse_group("group",line);
+  }
+
+
+  /* view definition */
+  /* view    anonymousViewNUM       included OID */
+  sprintf(viewname,"anonymousView%03d",num);
+  sprintf(line,"%s included %s", viewname, theoid);
+  DEBUGMSGTL((token,"passing: %s %s\n", "view", line));
+  vacm_parse_view("view",line);
+
+  /* map everything together */
+  /* access  anonymousGroupNameNUM  "" MODEL AUTHTYPE exact anonymousViewNUM [none/anonymousViewNUM] [none/anonymousViewNUM] */
+  sprintf(line, "anonymousGroupName%03d  \"\" %s %s exact %s %s %s", num,
+          model, authtype, viewname, rw, rw);
+  DEBUGMSGTL((token,"passing: %s %s\n", "access", line));
+  vacm_parse_access("access",line);
+  num++;
+}
+
+int
+vacm_in_view_callback(int majorID, int minorID, void *serverarg,
+                      void *clientarg) {
+  struct view_parameters *view_parms = (struct view_parameters *) serverarg;
+  int retval;
+  
+  if (view_parms == NULL)
+    return 1;
+  retval = vacm_in_view(view_parms->pdu, view_parms->name,
+                        view_parms->namelen);
+  if (retval != 0)
+    view_parms->errorcode = retval;
+  return retval;
+}
+
+
+/*******************************************************************-o-******
+ * vacm_in_view
+ *
+ * Parameters:
+ *	*pdu
+ *	*name
+ *	 namelen
+ *      
+ * Returns:
+ *	0	On success.
+ *	1	Missing security name.
+ *	2	Missing group
+ *	3	Missing access
+ *	4	Missing view
+ *	5	Not in view
+ *
+ * Debug output listed as follows:
+ *	<securityName> <groupName> <viewName> <viewType>
+ */
+int vacm_in_view (struct snmp_pdu *pdu,
 		  oid *name,
-		  int namelen)
+		  size_t namelen)
 {
     struct vacm_securityEntry *sp = securityFirst;
     struct vacm_accessEntry *ap;
     struct vacm_groupEntry *gp;
     struct vacm_viewEntry *vp;
+    struct sockaddr_in *pduIp = (struct sockaddr_in*)&(pdu->address);
+    struct sockaddr_in *srcIp, *srcMask;
     char *vn;
     char *sn;
 
-    if (pi->sec_model == SNMP_SEC_MODEL_SNMPv1 || pi->sec_model == SNMP_SEC_MODEL_SNMPv2c) {
-	DEBUGMSGTL(("mibII/vacm_vars", "vacm_in_view: ver=%d, source=%.8x, community=%s\n", pi->version, pi->source.sin_addr.s_addr, pi->community));
+    if (pdu->version == SNMP_VERSION_1 || pdu->version == SNMP_VERSION_2c) {
+	if (snmp_get_do_debugging()) {
+            char *buf;
+            if (pdu->community) {
+                buf = malloc(1+ pdu->community_len);
+                memcpy(buf, pdu->community, pdu->community_len);
+                buf[pdu->community_len] = '\0';
+            } else {
+                DEBUGMSGTL(("mibII/vacm_vars", "NULL community"));
+                buf = strdup("NULL");
+            }
+            
+	    DEBUGMSGTL(("mibII/vacm_vars", "vacm_in_view: ver=%d, source=%.8x, community=%s\n", pdu->version, pduIp->sin_addr.s_addr, buf));
+	    free (buf);
+	}
 
 	/* allow running without snmpd.conf */
 	if (sp == NULL) {
 	    DEBUGMSGTL(("mibII/vacm_vars", "vacm_in_view: accepted with no com2sec entries\n"));
-	    switch (pi->pdutype) {
+	    switch (pdu->command) {
 	    case SNMP_MSG_GET:
 	    case SNMP_MSG_GETNEXT:
 	    case SNMP_MSG_GETBULK:
-		return 1;
-	    default:
 		return 0;
+	    default:
+		return 1;
 	    }
 	}
 	while (sp) {
-	    if ((pi->source.sin_addr.s_addr & sp->sourceMask.sin_addr.s_addr)
-		    == sp->sourceIp.sin_addr.s_addr
-		&& strcmp(sp->community, (char *)pi->community) == 0)
+	    srcIp   = (struct sockaddr_in *)&(sp->sourceIp);
+	    srcMask = (struct sockaddr_in *)&(sp->sourceMask);
+	    if ((pduIp->sin_addr.s_addr & srcMask->sin_addr.s_addr)
+		    == srcIp->sin_addr.s_addr
+                && strlen(sp->community) == pdu->community_len
+		&& !strncmp(sp->community, (char *)pdu->community, pdu->community_len))
 		break;
 	    sp = sp->next;
 	}
-	if (sp == NULL) return 0;
+	if (sp == NULL) return 1;
 	sn = sp->securityName;
-    }
-    else {
+    } else if (pdu->securityModel == SNMP_SEC_MODEL_USM) {
+      DEBUGMSG (("mibII/vacm_vars",
+                 "vacm_in_view: ver=%d, model=%d, secName=%s\n",
+                 pdu->version, pdu->securityModel, pdu->securityName));
+      sn = pdu->securityName;
+    } else {
 	sn = NULL;
     }
 
-    if (sn == NULL) return 0;
-    DEBUGMSGTL(("mibII/vacm_vars", "vacm_in_view: securityName == %s\n", sn));
-    gp = vacm_getGroupEntry(pi->sec_model, sn);
-    if (gp == NULL) return 0;
-    DEBUGMSGTL(("mibII/vacm_vars", "vacm_in_view: groupName == %s\n", gp->groupName));
-    ap = vacm_getAccessEntry(gp->groupName, "", pi->sec_model, pi->sec_level);
-    if (ap == NULL) return 0;
-    switch (pi->pdutype) {
-    case SNMP_MSG_GET:
-    case SNMP_MSG_GETNEXT:
-    case SNMP_MSG_GETBULK:
+    if (sn == NULL) return 1;
+    DEBUGMSGTL(("mibII/vacm_vars", "vacm_in_view: sn=%s", sn));
+
+    gp = vacm_getGroupEntry(pdu->securityModel, sn);
+    if (gp == NULL) { DEBUGMSG(("mibII/vacm_vars", "\n")); return 2; }
+    DEBUGMSG (("mibII/vacm_vars", ", gn=%s", gp->groupName));
+
+    ap = vacm_getAccessEntry(gp->groupName, "", pdu->securityModel,
+                             pdu->securityLevel);
+    if (ap == NULL) { DEBUGMSG(("mibII/vacm_vars", "\n")); return 3; }
+
+    if (name == 0) { /* only check the setup of the vacm for the request */
+        DEBUGMSG(("mibII/vacm_vars", ", Done checking setup\n"));
+        return 0;
+    }
+
+    switch (pdu->command) {
+      case SNMP_MSG_GET:
+      case SNMP_MSG_GETNEXT:
+      case SNMP_MSG_GETBULK:
 	vn = ap->readView;
 	break;
-    case SNMP_MSG_SET:
+      case SNMP_MSG_SET:
 	vn = ap->writeView;
 	break;
-    case SNMP_MSG_TRAP:
-    case SNMP_MSG_TRAP2:
-    case SNMP_MSG_INFORM:
+      case SNMP_MSG_TRAP:
+      case SNMP_MSG_TRAP2:
+      case SNMP_MSG_INFORM:
 	vn = ap->notifyView;
 	break;
-    default:
-	fprintf(stderr,"bad msg type in vacm_in_view: %d\n", pi->pdutype);
+      default:
+        snmp_log(LOG_ERR, "bad msg type in vacm_in_view: %d\n", pdu->command);
 	vn = ap->readView;
     }
-    DEBUGMSGTL(("mibII/vacm_vars", "vacm_in_view: viewName == %s\n", vn));
+    DEBUGMSG (("mibII/vacm_vars", ", vn=%s", vn));
+
     vp = vacm_getViewEntry (vn, name, namelen);
-    if (vp == NULL) return 0;
-    DEBUGMSGTL(("mibII/vacm_vars", "vacm_in_view: viewType == %d\n", vp->viewType));
-    if (vp->viewType == SNMP_VIEW_EXCLUDED) return 0;
-    return 1;
-}
+    if (vp == NULL) { DEBUGMSG(("mibII/vacm_vars", "\n")); return 4; }
+    DEBUGMSG(("mibII/vacm_vars", ", vt=%d\n", vp->viewType));
+
+    if (vp->viewType == SNMP_VIEW_EXCLUDED) return 5;
+
+    return 0;
+
+}  /* end vacm_in_view() */
+
 
 u_char *var_vacm_sec2group(struct variable *vp,
 			   oid *name,
-			   int *length,
+			   size_t *length,
 			   int exact,
-			   int *var_len,
+			   size_t *var_len,
 			   WriteMethod **write_method)
 {
     struct vacm_groupEntry *gp;
@@ -509,7 +745,9 @@ u_char *var_vacm_sec2group(struct variable *vp,
 	groupSubtreeLen = *length - 13;
 	cp = secname;
 	while (groupSubtreeLen-- > 0) {
-	    *cp++ = *groupSubtree++;
+            if (*groupSubtree > 255)
+              return 0; /* illegal value */
+	    *cp++ = (char) *groupSubtree++;
 	}
 	*cp = 0;
 
@@ -521,7 +759,9 @@ u_char *var_vacm_sec2group(struct variable *vp,
 	groupSubtreeLen = *length - 12;
 	cp = secname;
 	while (groupSubtreeLen-- > 0) {
-	    *cp++ = *groupSubtree++;
+            if (*groupSubtree > 255)
+              return 0; /* illegal value */
+	    *cp++ = (char) *groupSubtree++;
 	}
 	*cp = 0;
 	vacm_scanGroupInit();
@@ -565,9 +805,9 @@ u_char *var_vacm_sec2group(struct variable *vp,
 
 u_char *var_vacm_access(struct variable *vp,
 			oid *name,
-			int *length,
+			size_t *length,
 			int exact,
-			int *var_len,
+			size_t *var_len,
 			WriteMethod **write_method)
 {
     struct vacm_accessEntry *gp;
@@ -592,13 +832,17 @@ u_char *var_vacm_access(struct variable *vp,
 	len = *op++;
 	cp = groupName;
 	while (len-- > 0) {
-	    *cp++ = *op++;
+            if (*op > 255)
+              return 0; /* illegal value */
+	    *cp++ = (char) *op++;
 	}
 	*cp = 0;
 	len = *op++;
 	cp = contextPrefix;
 	while (len-- > 0) {
-	    *cp++ = *op++;
+            if (*op > 255)
+              return 0; /* illegal value */
+	    *cp++ = (char) *op++;
 	}
 	*cp = 0;
 	secmodel = *op++;
@@ -620,7 +864,9 @@ u_char *var_vacm_access(struct variable *vp,
 	    len = *op;
 	    cp = groupName;
 	    while (len-- >= 0) {
-		*cp++ = *op++;
+                if (*op > 255)
+                  return 0; /* illegal value */
+		*cp++ = (char) *op++;
 	    }
 	    *cp = 0;
 	}
@@ -630,7 +876,9 @@ u_char *var_vacm_access(struct variable *vp,
 	    len = *op;
 	    cp = contextPrefix;
 	    while (len-- >= 0) {
-		*cp++ = *op++;
+                if (*op > 255)
+                  return 0; /* illegal value */
+		*cp++ = (char) *op++;
 	    }
 	    *cp = 0;
 	}
@@ -708,15 +956,15 @@ u_char *var_vacm_access(struct variable *vp,
 
 u_char *var_vacm_view(struct variable *vp,
 		      oid *name,
-		      int *length,
+		      size_t *length,
 		      int exact,
-		      int *var_len,
+		      size_t *var_len,
 		      WriteMethod **write_method)
 {
     struct vacm_viewEntry *gp;
     char viewName[32];
     oid subtree[MAX_OID_LEN];
-    int subtreeLen = 0;
+    size_t subtreeLen = 0;
     oid *op, *op1;
     int len;
     char *cp;
@@ -734,7 +982,9 @@ u_char *var_vacm_view(struct variable *vp,
 	len = *op++;
 	cp = viewName;
 	while (len-- > 0) {
-	    *cp++ = *op++;
+            if (*op > 255)
+              return 0; /* illegal value */
+	    *cp++ = (char) *op++;
 	}
 	*cp = 0;
 	len = *length - (op - name);
@@ -758,7 +1008,9 @@ u_char *var_vacm_view(struct variable *vp,
 	    len = *op;
 	    cp = viewName;
 	    while (len-- >= 0) {
-		*cp++ = *op++;
+                if (*op > 255)
+                  return 0; /* illegal value */
+		*cp++ = (char) *op++;
 	    }
 	    *cp = 0;
 	}
