@@ -41,7 +41,9 @@ PERFORMANCE OF THIS SOFTWARE.
 #define GATEWAY			/* MultiNet is always configured this way! */
 #include <stdio.h>
 #include <sys/types.h>
+#if HAVE_SYS_PARAM_H
 #include <sys/param.h>
+#endif
 #include <sys/socket.h>
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
@@ -159,8 +161,13 @@ PERFORMANCE OF THIS SOFTWARE.
 # endif
 #endif
 
+#if HAVE_DMALLOC_H
+#include <dmalloc.h>
+#endif
+
 #include "asn1.h"
 #include "snmp_debug.h"
+#include "snmp_logging.h"
 
 #define CACHE_TIME (120)	    /* Seconds */
 
@@ -207,9 +214,9 @@ extern const struct in_addr * get_in_address (const void *, int, int);
 u_char *
 var_ipRouteEntry(struct variable *vp,
 		 oid *name,
-		 int *length,
+		 size_t *length,
 		 int exact,
-		 int *var_len,
+		 size_t *var_len,
 		 WriteMethod **write_method)
 {
   /*
@@ -277,13 +284,13 @@ var_ipRouteEntry(struct variable *vp,
 	break;
       if (rtp->rtm_version != RTM_VERSION)
 	{
-	  fprintf (stderr, "routing socket message version mismatch (%d instead of %d)\n",
+	  snmp_log(LOG_ERR, "routing socket message version mismatch (%d instead of %d)\n",
 		   rtp->rtm_version, RTM_VERSION);
 	  break;
 	}
       if (rtp->rtm_type != RTM_GET)
 	{
-	  fprintf (stderr, "routing socket returned message other than GET (%d)\n",
+	  snmp_log(LOG_ERR, "routing socket returned message other than GET (%d)\n",
 		   rtp->rtm_type);
 	  continue;
 	}
@@ -337,15 +344,27 @@ var_ipRouteEntry(struct variable *vp,
     long_return = (rtp->rtm_flags & RTF_UP) ? 1 : 0;
     return (u_char *)&long_return;
   case IPROUTEMETRIC2:
+#if NO_DUMMY_VALUES
+    return NULL;
+#endif
     long_return = -1;
     return (u_char *)&long_return;
   case IPROUTEMETRIC3:
+#if NO_DUMMY_VALUES
+    return NULL;
+#endif
     long_return = -1;
     return (u_char *)&long_return;
   case IPROUTEMETRIC4:
+#if NO_DUMMY_VALUES
+    return NULL;
+#endif
     long_return = -1;
     return (u_char *)&long_return;
   case IPROUTEMETRIC5:
+#if NO_DUMMY_VALUES
+    return NULL;
+#endif
     long_return = -1;
     return (u_char *)&long_return;
   case IPROUTENEXTHOP:
@@ -362,6 +381,9 @@ var_ipRouteEntry(struct variable *vp,
       ? 2 : (rtp->rtm_flags & RTF_DYNAMIC) ? 4 : 1;
     return (u_char *)&long_return;
   case IPROUTEAGE:
+#if NO_DUMMY_VALUES
+    return NULL;
+#endif
     long_return = 0;
     return (u_char *)&long_return;
   case IPROUTEMASK:
@@ -379,7 +401,7 @@ var_ipRouteEntry(struct variable *vp,
     *var_len = nullOidLen;
     return (u_char *) nullOid;
   default:
-    ERROR_MSG("");
+    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ipRouteEntry\n", vp->magic));
   }
   return NULL;
 }
@@ -414,7 +436,7 @@ static void Route_Scan_Reload (void)
   if (sysctl (name, sizeof (name) / sizeof (int),
 	      0, &size, 0, 0) == -1)
     {
-      fprintf (stderr, "sysctl(CTL_NET,PF_ROUTE,0,0,NET_RT_DUMP,0)\n");
+      snmp_log(LOG_ERR, "sysctl(CTL_NET,PF_ROUTE,0,0,NET_RT_DUMP,0)\n");
     }
   else
     {
@@ -427,7 +449,7 @@ static void Route_Scan_Reload (void)
 	    }
 	  if ((all_routes = malloc (size)) == 0)
 	    {
-	      fprintf (stderr, "out of memory allocating route table\n");
+	      snmp_log(LOG_ERR, "out of memory allocating route table\n");
 	    }
 	  all_routes_size = size;
 	}
@@ -438,7 +460,7 @@ static void Route_Scan_Reload (void)
       if (sysctl (name, sizeof (name) / sizeof (int),
 		  all_routes, &size, 0, 0) == -1)
 	{
-	  fprintf (stderr, "sysctl(CTL_NET,PF_ROUTE,0,0,NET_RT_DUMP,0)\n");
+	  snmp_log(LOG_ERR, "sysctl(CTL_NET,PF_ROUTE,0,0,NET_RT_DUMP,0)\n");
 	}
       all_routes_end = all_routes + size;
     }
@@ -483,9 +505,9 @@ klgetsa(struct sockaddr_in *dst)
 u_char *
 var_ipRouteEntry(struct variable *vp,
 		oid *name,
-		int *length,
+		size_t *length,
 		int exact,
-		int *var_len,
+		size_t *var_len,
 		WriteMethod **write_method)
 {
     /*
@@ -509,6 +531,9 @@ var_ipRouteEntry(struct variable *vp,
      ** this optimisation fails, if there is only a single route avail.
      ** it is a very special case, but better leave it out ...
      **/
+#if NO_DUMMY_VALUES
+      saveNameLen = 0;
+#endif
     if (rtsize <= 1)
       Save_Valid = 0;
     else
@@ -553,12 +578,8 @@ var_ipRouteEntry(struct variable *vp,
 #if NEED_KLGETSA
 	    sa = klgetsa((struct sockaddr_in *) rthead[RtIndex]->rt_dst);
 	    cp = (u_char *) &(sa->sin_addr.s_addr);
-#else
-#ifdef linux
-	    cp = (u_char *) rthead[RtIndex]->rt_dst.sa_data;
 #else
 	    cp = (u_char *)&(((struct sockaddr_in *) &(rthead[RtIndex]->rt_dst))->sin_addr.s_addr);
-#endif
 #endif
 	    op = Current + 10;
 	    *op++ = *cp++;
@@ -595,12 +616,8 @@ var_ipRouteEntry(struct variable *vp,
 #if NEED_KLGETSA
 	    sa = klgetsa((struct sockaddr_in *) rthead[RtIndex]->rt_dst);
 	    return(u_char *) &(sa->sin_addr.s_addr);
-#else
-#ifdef linux
-            return rthead[RtIndex]->rt_dst.sa_data;
 #else
 	    return(u_char *) &((struct sockaddr_in *) &rthead[RtIndex]->rt_dst)->sin_addr.s_addr;
-#endif
 #endif
 	case IPROUTEIFINDEX:
 	    long_return = (u_long)rthead[RtIndex]->rt_unit;
@@ -609,27 +626,35 @@ var_ipRouteEntry(struct variable *vp,
 	    long_return = (rthead[RtIndex]->rt_flags & RTF_GATEWAY) ? 1 : 0;
 	    return (u_char *)&long_return;
 	case IPROUTEMETRIC2:
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = -1;
 	    return (u_char *)&long_return;
 	case IPROUTEMETRIC3:
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = -1;
 	    return (u_char *)&long_return;
 	case IPROUTEMETRIC4:
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = -1;
 	    return (u_char *)&long_return;
 	case IPROUTEMETRIC5:
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = -1;
 	    return (u_char *)&long_return;
 	case IPROUTENEXTHOP:
 #if NEED_KLGETSA
 	    sa = klgetsa((struct sockaddr_in *) rthead[RtIndex]->rt_gateway);
 	    return(u_char *) &(sa->sin_addr.s_addr);
-#else
-#ifdef linux
-	    return(u_char *) rthead[RtIndex]->rt_gateway.sa_data;
 #else
 	    return(u_char *) &((struct sockaddr_in *) &rthead[RtIndex]->rt_gateway)->sin_addr.s_addr;
-#endif /* linux */
 #endif /* *bsd */
 	case IPROUTETYPE:
 	    long_return = (rthead[RtIndex]->rt_flags & RTF_GATEWAY) ? 4 : 3;
@@ -638,19 +663,21 @@ var_ipRouteEntry(struct variable *vp,
 	    long_return = (rthead[RtIndex]->rt_flags & RTF_DYNAMIC) ? 4 : 2;
 	    return (u_char *)&long_return;
 	case IPROUTEAGE:
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = 0;
 	    return (u_char *)&long_return;
 	case IPROUTEMASK:
 #if NEED_KLGETSA
 		/* XXX - Almost certainly not right
 		    but I don't have a suitable system to test this on */
+#if NO_DUMMY_VALUES
+	    return NULL;
+#endif
 	    long_return = 0;
 #else /*  NEED_KLGETSA */
-  #ifndef linux
 	    if ( ((struct sockaddr_in *) &rthead[RtIndex]->rt_dst)->sin_addr.s_addr == 0 )
-  #else
-	    if (rthead[RtIndex]->rt_dst.sa_data == 0)
-  #endif
 		long_return = 0;	/* Default route */
 	    else {
 #ifndef linux
@@ -661,7 +688,8 @@ var_ipRouteEntry(struct variable *vp,
 
 		long_return = rt_ifnetaddr.ia_subnetmask;
 #else /* linux */
-                return (u_char *) rthead[RtIndex]->rt_genmask.sa_data;
+	    cp = (u_char *)&(((struct sockaddr_in *) &(rthead[RtIndex]->rt_dst))->sin_addr.s_addr);
+                return (u_char *) &(((struct sockaddr_in *) &(rthead[RtIndex]->rt_genmask))->sin_addr.s_addr);
 #endif /* linux */
 	    }
 #endif /* NEED_KLGETSA */
@@ -670,7 +698,7 @@ var_ipRouteEntry(struct variable *vp,
 	    *var_len = nullOidLen;
 	    return (u_char *)nullOid;
 	default:
-	    ERROR_MSG("");
+	    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ipRouteEntry\n", vp->magic));
    }
    return NULL;
 }
@@ -705,9 +733,9 @@ IP_Cmp_Route(void *addr, void *ep)
 u_char *
 var_ipRouteEntry(struct variable *vp,
 		 oid *name,
-		 int *length,
+		 size_t *length,
 		 int exact,
-		 int *var_len,
+		 size_t *var_len,
 		 WriteMethod **write_method)
 {
   /*
@@ -726,11 +754,11 @@ var_ipRouteEntry(struct variable *vp,
 
   /* fill in object part of name for current (less sizeof instance part) */
   
-  memcpy( (char *)current,(char *)vp->name, (int)(vp->namelen) * sizeof(oid));
+  memcpy( (char *)current,(char *)vp->name, vp->namelen * sizeof(oid));
   if (*length == IP_ROUTENAME_LENGTH) /* Assume that the input name is the lowest */
     memcpy( (char *)lowest,(char *)name, IP_ROUTENAME_LENGTH * sizeof(oid));
   else
-    name[IP_ROUTEADDR_OFF] = -1; /* Grhhh: to prevent accidental comparison :-( */
+    name[IP_ROUTEADDR_OFF] = (oid)-1; /* Grhhh: to prevent accidental comparison :-( */
   for (Nextentry.ipRouteDest = (u_long)-2, req_type = GET_FIRST;
        ;
        Nextentry = entry, req_type = GET_NEXT) {
@@ -803,7 +831,7 @@ var_ipRouteEntry(struct variable *vp,
     long_return = Lowentry.ipRouteMask;
     return (u_char *)&long_return;
   default:
-    ERROR_MSG("");
+    DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ipRouteEntry\n", vp->magic));
   };
   return NULL;
 }
@@ -857,7 +885,7 @@ load_rtentries(struct radix_node *pt)
 #else
       klookup((unsigned long)ifnet.if_name, name, sizeof name);
       name[sizeof (name) - 1] = '\0';
-      cp = (char *) index(name, '\0');
+      cp = (char *) strchr(name, '\0');
       string_append_int (cp, ifnet.if_unit);
 #endif
       Interface_Scan_Init();
@@ -925,7 +953,7 @@ static void Route_Scan_Reload (void)
   if (!rthead) {
     rthead = (RTENTRY **) malloc(100 * sizeof(RTENTRY *));
     if (!rthead) {
-      ERROR_MSG("malloc");
+      snmp_log(LOG_ERR,"route table malloc fail\n");
       return;
     }
     memset((char *)rthead,(0), 100 * sizeof(RTENTRY *));
@@ -973,7 +1001,7 @@ static void Route_Scan_Reload (void)
           klookup( rt->rt_ifp, (char *)&ifnet, sizeof (ifnet));
           klookup( ifnet.if_name, name, 16);
           name[15] = '\0';
-          cp = (char *) index(name, '\0');
+          cp = (char *) strchr(name, '\0');
 	  string_append_int (cp, ifnet.if_unit);
 
           Interface_Scan_Init();
@@ -1041,7 +1069,7 @@ static void Route_Scan_Reload (void)
 	if (!rthead) {
           rthead = (RTENTRY **) malloc(100 * sizeof(RTENTRY *));
           if (!rthead) {
-		ERROR_MSG("malloc");
+		snmp_log(LOG_ERR,"route table malloc fail\n");
 		return;
 	    }
           memset((char *)rthead,(0), 100 * sizeof(RTENTRY *));
@@ -1078,7 +1106,7 @@ static void Route_Scan_Reload (void)
 				sizeof (ifnet));
 			klookup((unsigned long) ifnet.if_name, name, 16);
 			name[15] = '\0';
-			cp = (char *) index(name, '\0');
+			cp = (char *) strchr(name, '\0');
 			string_append_int (cp, ifnet.if_unit);
 			if (strcmp(name,"lo0") == 0) continue; 
 
@@ -1142,7 +1170,7 @@ static void Route_Scan_Reload (void)
 	if (! rthead) {
 	    rthead = (struct rtentry **) calloc(100, sizeof(struct rtentry *));
 	    if (! rthead) {
-		ERROR_MSG("malloc");
+		snmp_log(LOG_ERR,"route table malloc fail\n");
 		return;
 	    }
 	    rtallocate = 100;
@@ -1156,7 +1184,7 @@ static void Route_Scan_Reload (void)
 
 	if (! (in = fopen ("/proc/net/route", "r")))
 	  {
-	    fprintf (stderr, "cannot open /proc/net/route - burps\n");
+	    snmp_log(LOG_ERR, "cannot open /proc/net/route - burps\n");
 	    return;
 	  }
 
@@ -1176,13 +1204,13 @@ static void Route_Scan_Reload (void)
 	     * Iface Dest GW Flags RefCnt Use Metric Mask MTU Win IRTT
 	     * eth0 0A0A0A0A 00000000 05 0 0 0 FFFFFFFF 1500 0 0 
 	     */
-	    if (8 != sscanf (line, "%s %lx %lx %x %u %d %d %lx %*d %*d %*d\n",
+	    if (8 != sscanf (line, "%s %x %x %x %u %d %d %x %*d %*d %*d\n",
 			     rt->rt_dev,
-			     (unsigned long *) rt->rt_dst.sa_data,
-			     (unsigned long *) rt->rt_gateway.sa_data,
+			     &(((struct sockaddr_in *) &(rtent.rt_dst))->sin_addr.s_addr),
+			     &(((struct sockaddr_in *) &(rtent.rt_gateway))->sin_addr.s_addr),
 /* XXX: fix type of the args */
 			     &flags, &refcnt, &use, &metric,
-			     (unsigned long *) &rt->rt_genmask.sa_data))
+			     &(((struct sockaddr_in *) &(rtent.rt_genmask))->sin_addr.s_addr)))
 	      continue;
 	    
 	    strcpy (name, rt->rt_dev);
@@ -1247,13 +1275,8 @@ static int qsort_compare(RTENTRY **r1,
 	register u_long dst1 = ntohl(klgetsa((struct sockaddr_in *)(*r1)->rt_dst)->sin_addr.s_addr);
 	register u_long dst2 = ntohl(klgetsa((struct sockaddr_in *)(*r2)->rt_dst)->sin_addr.s_addr);
 #else
-#if !defined(linux)
 	register u_long dst1 = ntohl(((struct sockaddr_in *) &((*r1)->rt_dst))->sin_addr.s_addr);
 	register u_long dst2 = ntohl(((struct sockaddr_in *) &((*r2)->rt_dst))->sin_addr.s_addr);
-#else
-	register u_long dst1 = ntohl(*(unsigned long *)&(*r1)->rt_dst.sa_data);
-	register u_long dst2 = ntohl(*(unsigned long *)&(*r2)->rt_dst.sa_data);
-#endif /* defined(linux) */
 #endif /* NEED_KLGETSA */
 
 	/*
@@ -1303,6 +1326,8 @@ static int qsort_compare(RTENTRY **r1,
 #include "interfaces.h"
 #include "struct.h"
 #include "util_funcs.h"
+#include "snmp_logging.h"
+#include "snmp_debug.h"
 
 static TAILQ_HEAD(, snmprt) rthead;
 static char *rtbuf;
@@ -1368,7 +1393,7 @@ rtmsg(struct rt_msghdr *rtm)
 	}
 	if (!gotdest) {
 		/* XXX can't happen if code above is correct */
-		fprintf(stderr, "route no dest?\n");
+	 snmp_log(LOG_ERR, "route no dest?\n");
 		free(rt);
 	} else {
 		/* If no mask provided, it was a host route. */
@@ -1441,9 +1466,9 @@ suck_krt(int force)
 u_char *
 var_ipRouteEntry(struct variable *vp,
 		 oid *name,
-		 int *length,
+		 size_t *length,
 		 int exact,
-		 int *var_len,
+		 size_t *var_len,
 		 WriteMethod **write_method)
 {
 	/*
@@ -1571,6 +1596,9 @@ var_ipRouteEntry(struct variable *vp,
 		return (u_char *)&long_return;
 
 	case IPROUTEAGE:
+#if NO_DUMMY_VALUES
+		return NULL;
+#endif
 		long_return = 0;
 		return (u_char *)&long_return;
 
@@ -1582,7 +1610,7 @@ var_ipRouteEntry(struct variable *vp,
 		*var_len = nullOidLen;
 		return (u_char *)nullOid;
 	default:
-		ERROR_MSG("");
+		DEBUGMSGTL(("snmpd", "unknown sub-id %d in var_ipRouteEntry\n", vp->magic));
 	}
 	return NULL;
 }
@@ -1611,13 +1639,13 @@ init_var_route(void)
 const struct sockaddr *
 get_address (const void * _ap, int addresses, int wanted)
 {
-  const struct sockaddr *ap = (struct sockaddr *) _ap;
-  int index;
+  const struct sockaddr *ap = (const struct sockaddr *) _ap;
+  int iindex;
   int bitmask;
 
-  for (index = 0, bitmask = 1;
-       index < RTAX_MAX;
-       ++index, bitmask <<= 1)
+  for (iindex = 0, bitmask = 1;
+       iindex < RTAX_MAX;
+       ++iindex, bitmask <<= 1)
     {
       if (bitmask == wanted)
 	{
@@ -1632,27 +1660,10 @@ get_address (const void * _ap, int addresses, int wanted)
 	}
       else if (bitmask & addresses)
 	{
-	  unsigned length = 1;
-	  switch (ap->sa_family)
-	    {
-	    case AF_UNIX:
-	      length = sizeof (struct sockaddr_un);
-	      break;
-	    case AF_LINK:
-#ifdef _MAX_SA_LEN
-	      length = _MAX_SA_LEN;
-#else
-	      length = sizeof (struct sockaddr_dl);
-#endif
-	      break;
-	    case AF_INET:
-	    default:
-	      length = sizeof (struct sockaddr_in);
-	      break;
-	    }
+	  unsigned length = (unsigned)snmp_socket_length(ap->sa_family);
 	  while (length % sizeof (long) != 0)
 	    ++length;
-	  ap = (struct sockaddr *) ((char *) ap + length);
+	  ap = (const struct sockaddr *) ((const char *) ap + length);
 	}
     }
   return 0;
@@ -1668,15 +1679,15 @@ get_address (const void * _ap, int addresses, int wanted)
 const struct in_addr *
 get_in_address (const void * ap, int addresses, int wanted)
 {
-  struct sockaddr_in * a;
+  const struct sockaddr_in * a;
 
-  a = (struct sockaddr_in *) get_address (ap, addresses, wanted);
+  a =  (const struct sockaddr_in *)get_address (ap, addresses, wanted);
   if (a == NULL)
     return NULL;
 
   if (a->sin_family != AF_INET)
     {
-      ERROR_MSG("AF_INET sockaddr expected");
+      DEBUGMSGTL(("snmpd", "unknown socket family %d [AF_INET expected] in var_ipRouteEntry.\n", a->sin_family));
     }
   return &a->sin_addr;
 }
