@@ -1,3 +1,7 @@
+/*
+ * disk.c
+ */
+
 #include <config.h>
 
 #include <stdio.h>
@@ -78,6 +82,9 @@
 #endif
 #define statvfs statfs
 #endif
+#if HAVE_VM_VM_H
+#include <vm/vm.h>
+#endif
 #if HAVE_VM_SWAP_PAGER_H
 #include <vm/swap_pager.h>
 #endif
@@ -87,7 +94,7 @@
 #if HAVE_MALLOC_H
 #include <malloc.h>
 #endif
-#if STDC_HEADERS
+#if HAVE_STRING_H
 #include <string.h>
 #endif
 #if HAVE_FSTAB_H
@@ -103,7 +110,11 @@
 #include <netinet/in.h>
 #endif
 #if TIME_WITH_SYS_TIME
-# include <sys/time.h>
+# ifdef WIN32
+#  include <sys/timeb.h>
+# else
+#  include <sys/time.h>
+# endif
 # include <time.h>
 #else
 # if HAVE_SYS_TIME_H
@@ -112,6 +123,13 @@
 #  include <time.h>
 # endif
 #endif
+#if HAVE_WINSOCK_H
+#include <winsock.h>
+#endif
+
+#if HAVE_DMALLOC_H
+#include <dmalloc.h>
+#endif
 
 #include "mibincl.h"
 #include "struct.h"
@@ -119,14 +137,13 @@
 #include "util_funcs.h"
 #include "read_config.h"
 #include "agent_read_config.h"
-#include "mib_module_config.h"
 #include "auto_nlist.h"
 #if USING_UCD_SNMP_ERRORMIB_MODULE
 #include "errormib.h"
 #else
-#define setPerrorstatus(x) perror(x)
+#define setPerrorstatus(x) snmp_log_perror(x)
 #endif
-#include "../../../snmplib/system.h"
+#include "system.h"
 
 int numdisks;
 struct diskpart disks[MAXDISKS];
@@ -141,6 +158,7 @@ struct variable2 extensible_disk_variables[] = {
   {DISKAVAIL, ASN_INTEGER, RONLY, var_extensible_disk, 1, {DISKAVAIL}},
   {DISKUSED, ASN_INTEGER, RONLY, var_extensible_disk, 1, {DISKUSED}},
   {DISKPERCENT, ASN_INTEGER, RONLY, var_extensible_disk, 1, {DISKPERCENT}},
+  {DISKPERCENTNODE, ASN_INTEGER, RONLY, var_extensible_disk, 1, {DISKPERCENTNODE}},
   {ERRORFLAG, ASN_INTEGER, RONLY, var_extensible_disk, 1, {ERRORFLAG }},
   {ERRORMSG, ASN_OCTET_STR, RONLY, var_extensible_disk, 1, {ERRORMSG }}
 };
@@ -152,7 +170,7 @@ oid disk_variables_oid[] = { EXTENSIBLEMIB,DISKMIBNUM,1 };
 void init_disk(void)
 {
   /* register ourselves with the agent to handle our mib tree */
-  REGISTER_MIB("mibII/dis", extensible_disk_variables, variable2, \
+  REGISTER_MIB("ucd-snmp/disk", extensible_disk_variables, variable2, \
                disk_variables_oid);
 
   snmpd_register_config_handler("disk", disk_parse_config, disk_free_config,
@@ -172,7 +190,7 @@ void disk_free_config (void)
   }
 }
 
-void disk_parse_config(char *token, char *cptr)
+void disk_parse_config(const char *token, char *cptr)
 {
 #if HAVE_GETMNTENT
 #if HAVE_SYS_MNTTAB_H
@@ -184,7 +202,7 @@ void disk_parse_config(char *token, char *cptr)
 #else
 #if HAVE_FSTAB_H
   struct fstab *fstab;
-  struct stat stat1, stat2;
+  struct stat stat1;
 #endif
 #endif
   char tmpbuf[1024];
@@ -290,11 +308,11 @@ void disk_parse_config(char *token, char *cptr)
   
 */
 
-unsigned char *var_extensible_disk(struct variable *vp,
+u_char *var_extensible_disk(struct variable *vp,
 				   oid *name,
-				   int *length,
+				   size_t *length,
 				   int exact,
-				   int *var_len,
+				   size_t *var_len,
 				   WriteMethod **write_method)
 {
 
@@ -303,6 +321,9 @@ unsigned char *var_extensible_disk(struct variable *vp,
   double totalblks, free, used, avail, availblks;
 #else
   static long avail;
+#ifdef STRUCT_STATVFS_HAS_F_FILES
+  int percent_inode;
+#endif
 #endif
   static long long_ret;
   static char errmsg[300];
@@ -353,7 +374,7 @@ unsigned char *var_extensible_disk(struct variable *vp,
 #else
   if (statvfs (disks[disknum].path, &vfs) == -1) {
 #endif
-    fprintf(stderr,"Couldn't open device %s\n",disks[disknum].device);
+    snmp_log(LOG_ERR,"Couldn't open device %s\n",disks[disknum].device);
     setPerrorstatus("statvfs dev/disk");
     return NULL;
   }
@@ -378,6 +399,11 @@ unsigned char *var_extensible_disk(struct variable *vp,
   iserror = (disks[disknum].minimumspace >= 0 ?
              avail < disks[disknum].minimumspace :
              100-percent <= disks[disknum].minpercent) ? 1 : 0;
+#ifdef STRUCT_STATVFS_HAS_F_FILES
+  percent_inode = vfs.f_favail <= 0 ? 100 :
+    (int) ((double) (vfs.f_files - vfs.f_ffree) /
+           (double) (vfs.f_files - (vfs.f_ffree - vfs.f_favail)) * 100.0 + 0.5);
+#endif
   switch (vp->magic) {
     case DISKTOTAL:
       long_ret = vfs.f_blocks;
@@ -398,6 +424,11 @@ unsigned char *var_extensible_disk(struct variable *vp,
     case DISKPERCENT:
       long_ret = percent;
       return ((u_char *) (&long_ret));
+#ifdef STRUCT_STATVFS_HAS_F_FILES
+    case DISKPERCENTNODE:
+      long_ret = percent_inode;
+      return ((u_char *) (&long_ret));
+#endif
     case ERRORFLAG:
       long_ret = iserror;
       return((u_char *) (&long_ret));
@@ -420,14 +451,14 @@ unsigned char *var_extensible_disk(struct variable *vp,
 #if HAVE_FSTAB_H
   /* read the disk information */
   if ((file = open(disks[disknum].device,0)) < 0) {
-    fprintf(stderr,"Couldn't open device %s\n",disks[disknum].device);
+    snmp_log(LOG_ERR,"Couldn't open device %s\n",disks[disknum].device);
     setPerrorstatus("open dev/disk");
     return(NULL);
   }
   lseek(file, (long) (SBLOCK * DEV_BSIZE), 0);
   if (read(file,(char *) &filesys, SBSIZE) != SBSIZE) {
     setPerrorstatus("open dev/disk");
-    fprintf(stderr,"Error reading device %s\n",disks[disknum].device);
+    snmp_log(LOG_ERR,"Error reading device %s\n",disks[disknum].device);
     close(file);
     return(NULL);
   }
