@@ -13,7 +13,7 @@ extern FindVarMethod var_extensible_disk;
 
 /* config file parsing routines */
 void disk_free_config (void);
-void disk_parse_config (char *, char *);
+void disk_parse_config (const char *, char *);
 
 #include "mibdefs.h"
 
@@ -24,6 +24,7 @@ void disk_parse_config (char *, char *);
 #define DISKAVAIL 7
 #define DISKUSED 8
 #define DISKPERCENT 9
+#define DISKPERCENTNODE 10
 
 struct diskpart
 {
