@@ -79,20 +79,30 @@
 #endif
 #define statvfs statfs
 #endif
+#if HAVE_VM_VM_H
+#include <vm/vm.h>
+#endif
 #if HAVE_VM_SWAP_PAGER_H
 #include <vm/swap_pager.h>
 #endif
 #if HAVE_SYS_FIXPOINT_H
 #include <sys/fixpoint.h>
 #endif
+#if HAVE_SYS_LOADAVG_H
+#include <sys/loadavg.h>
+#endif
 #if HAVE_MALLOC_H
 #include <malloc.h>
 #endif
-#if STDC_HEADERS
+#if HAVE_STRING_H
 #include <string.h>
 #endif
 #if TIME_WITH_SYS_TIME
-# include <sys/time.h>
+# ifdef WIN32
+#  include <sys/timeb.h>
+# else
+#  include <sys/time.h>
+# endif
 # include <time.h>
 #else
 # if HAVE_SYS_TIME_H
@@ -101,6 +111,9 @@
 #  include <time.h>
 # endif
 #endif
+#if HAVE_WINSOCK_H
+#include <winsock.h>
+#endif
 
 #include "mibincl.h"
 #include "struct.h"
@@ -136,14 +149,14 @@ void init_loadave(void)
   oid loadave_variables_oid[] = { EXTENSIBLEMIB,LOADAVEMIBNUM,1 };
 
   /* register ourselves with the agent to handle our mib tree */
-  REGISTER_MIB("ucd_snmp/loadave", extensible_loadave_variables, variable2, \
+  REGISTER_MIB("ucd-snmp/loadave", extensible_loadave_variables, variable2, \
                loadave_variables_oid);
 
   snmpd_register_config_handler("load", loadave_parse_config,
                                 loadave_free_config, "max1 [max5] [max15]");
 }
 
-void loadave_parse_config(char *token, char* cptr)
+void loadave_parse_config(const char *token, char* cptr)
 {
   int i;
   
@@ -193,7 +206,7 @@ int try_getloadavg(double *r_ave, size_t s_ave)
 #elif defined(linux)
   { FILE *in = fopen("/proc/loadavg", "r");
     if (!in) {
-      fprintf (stderr, "snmpd: cannot open /proc/loadavg\n");
+      snmp_log(LOG_ERR, "snmpd: cannot open /proc/loadavg\n");
       return (-1);
     }
     fscanf(in, "%lf %lf %lf", pave, (pave + 1), (pave + 2));
@@ -205,7 +218,9 @@ int try_getloadavg(double *r_ave, size_t s_ave)
   for(i=0;i<s_ave;i++)
     *(pave+i) = FIX_TO_DBL(favenrun[i]);
 #elif !defined(cygwin)
+#ifdef CAN_USE_NLIST
   if (auto_nlist(LOADAVE_SYMBOL,(char *) pave, sizeof(double)*s_ave) == 0)
+#endif
     return (-1);
 #endif
 /*
@@ -229,11 +244,11 @@ int try_getloadavg(double *r_ave, size_t s_ave)
   return 0;
 }
   
-unsigned char *var_extensible_loadave(struct variable *vp,
+u_char *var_extensible_loadave(struct variable *vp,
 				      oid *name,
-				      int *length,
+				      size_t *length,
 				      int exact,
-				      int *var_len,
+				      size_t *var_len,
 				      WriteMethod **write_method)
 {
 
@@ -241,10 +256,8 @@ unsigned char *var_extensible_loadave(struct variable *vp,
   static float float_ret;
   static char errmsg[300];
   double avenrun[3];
-  
   if (header_simple_table(vp,name,length,exact,var_len,write_method,3))
     return(NULL);
-
   switch (vp->magic) {
     case MIBINDEX:
       long_ret = name[*length-1];
@@ -259,7 +272,8 @@ unsigned char *var_extensible_loadave(struct variable *vp,
     return(0);
   switch (vp->magic) {
     case LOADAVE:
-      sprintf(errmsg,"%.2f",avenrun[name[*length-1]-1]);
+      
+      sprintf(errmsg,"%.2f",avenrun[name[*length-1]-1]); 
       *var_len = strlen(errmsg);
       return((u_char *) (errmsg));
     case LOADMAXVAL:
@@ -271,7 +285,7 @@ unsigned char *var_extensible_loadave(struct variable *vp,
       return((u_char *) (&long_ret));
 #ifdef OPAQUE_SPECIAL_TYPES
     case LOADAVEFLOAT:
-      float_ret = avenrun[name[*length-1]-1];
+      float_ret = (float)avenrun[name[*length-1]-1];
       *var_len = sizeof(float_ret);
       return((u_char *) (&float_ret));
 #endif
