@@ -10,11 +10,11 @@ config_require(ucd-snmp/extensible util_funcs)
 
 extern FindVarMethod var_extensible_pass;
 WriteMethod setPass;
-int pass_compare (void *, void *);
+int pass_compare (const void *, const void *);
 
 /* config file parsing routines */
 void pass_free_config (void);
-void pass_parse_config (char *, char *);
+void pass_parse_config (const char *, char *);
 
 #include "mibdefs.h"
 
