@@ -17,15 +17,15 @@
 #if HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
-#if TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
+#if HAVE_SYS_WAIT_H
+# include <sys/wait.h>
+#endif
+#if HAVE_WINSOCK_H
+#include <winsock.h>
+#endif
+
+#if HAVE_DMALLOC_H
+#include <dmalloc.h>
 #endif
 
 #include <signal.h>
@@ -37,7 +37,7 @@
 #include "extensible.h"
 #include "util_funcs.h"
 #include "read_config.h"
-#include "../../../snmplib/system.h"
+#include "system.h"
 
 struct extensible *persistpassthrus=NULL;
 int numpersistpassthrus=0;
@@ -47,10 +47,14 @@ struct persist_pipe_type {
   int pid;
 } *persist_pipes = ( struct persist_pipe_type * ) NULL;
 static int init_persist_pipes (void);
-static int close_persist_pipe (int iindex);
+static void close_persist_pipe (int iindex);
 static int open_persist_pipe (int iindex, char *command);
 static void destruct_persist_pipes (void);
-static int write_persist_pipe (int iindex, char *data);
+static int write_persist_pipe (int iindex, const char *data);
+
+/*  These are defined in pass.c */
+extern int asc2bin(char *p);
+extern int bin2asc(char *p, size_t n);
 
 /* the relocatable extensible commands variables */
 struct variable2 extensible_persist_passthru_variables[] = {
@@ -64,7 +68,7 @@ void init_pass_persist(void)
                                 pass_persist_free_config,"miboid program");
 }
 
-void pass_persist_parse_config(char *token, char* cptr)
+void pass_persist_parse_config(const char *token, char* cptr)
 {
   struct extensible **ppass = &persistpassthrus, **etmp, *ptmp;
   char *tcptr;
@@ -80,6 +84,8 @@ void pass_persist_parse_config(char *token, char* cptr)
   while(*ppass != NULL)
     ppass = &((*ppass)->next);
   (*ppass) = (struct extensible *) malloc(sizeof(struct extensible));
+  if (*ppass == NULL)
+    return;
   (*ppass)->type = PASSTHRU_PERSIST;
 
   (*ppass)->miblen = parse_miboid(cptr,(*ppass)->miboid);
@@ -106,17 +112,14 @@ void pass_persist_parse_config(char *token, char* cptr)
   if (numpersistpassthrus > 0) {
     etmp = (struct extensible **)
       malloc(((sizeof(struct extensible *)) * numpersistpassthrus));
+    if (etmp == NULL)
+      return;
     for(i=0,ptmp = (struct extensible *) persistpassthrus;
         i < numpersistpassthrus && ptmp != 0;
         i++, ptmp = ptmp->next)
       etmp[i] = ptmp;
     qsort(etmp, numpersistpassthrus, sizeof(struct extensible *),
-#ifdef __STDC__
-         (int (*)(const void *, const void *)) pass_persist_compare
-#else
-          pass_persist_compare
-#endif
-      );
+          pass_persist_compare);
     persistpassthrus = (struct extensible *) etmp[0];
     ptmp = (struct extensible *) etmp[0];
 
@@ -146,11 +149,11 @@ void pass_persist_free_config (void)
   numpersistpassthrus = 0;
 }
 
-unsigned char *var_extensible_pass_persist(struct variable *vp,
+u_char *var_extensible_pass_persist(struct variable *vp,
 					   oid *name,
-					   int *length,
+					   size_t *length,
 					   int exact,
-					   int *var_len,
+					   size_t *var_len,
 					   WriteMethod **write_method)
 {
 
@@ -239,17 +242,26 @@ unsigned char *var_extensible_pass_persist(struct variable *vp,
           return((unsigned char *) buf2);
         } else if (!strncasecmp(buf,"integer",7)) {
           *var_len = sizeof(long_ret);
-          long_ret = atoi(buf2);
+          long_ret = strtol(buf2, NULL, 10);
           vp->type = ASN_INTEGER;
           return((unsigned char *) &long_ret);
+        } else if (!strncasecmp(buf,"unsigned",7)) {
+          *var_len = sizeof(long_ret);
+          long_ret = strtoul(buf2, NULL, 10);
+          vp->type = ASN_UNSIGNED;
+          return((unsigned char *) &long_ret);
         } else if (!strncasecmp(buf,"counter",7)) {
           *var_len = sizeof(long_ret);
-          long_ret = atoi(buf2);
+          long_ret = strtoul(buf2, NULL, 10);
           vp->type = ASN_COUNTER;
           return((unsigned char *) &long_ret);
+        } else if (!strncasecmp(buf,"octet",5)) {
+          *var_len = asc2bin(buf2);
+          vp->type = ASN_OCTET_STR;
+          return((unsigned char *) buf2);
         } else if (!strncasecmp(buf,"gauge",5)) {
           *var_len = sizeof(long_ret);
-          long_ret = atoi(buf2);
+          long_ret = strtoul(buf2, NULL, 10);
           vp->type = ASN_GAUGE;
           return((unsigned char *) &long_ret);
         } else if (!strncasecmp(buf,"objectid",8)) {
@@ -259,18 +271,19 @@ unsigned char *var_extensible_pass_persist(struct variable *vp,
           return((unsigned char *) objid);
         } else if (!strncasecmp(buf,"timetick",8)) {
           *var_len = sizeof(long_ret);
-          long_ret = atoi(buf2);
+          long_ret = strtoul(buf2, NULL, 10);
           vp->type = ASN_TIMETICKS;
           return((unsigned char *) &long_ret);
         } else if (!strncasecmp(buf,"ipaddress",9)) {
           newlen = parse_miboid(buf2,objid);
           if (newlen != 4) {
-            fprintf(stderr,"invalid ipaddress returned:  %s\n",buf2);
+            snmp_log(LOG_ERR,"invalid ipaddress returned:  %s\n",buf2);
             *var_len = 0;
             return(NULL);
           }
           long_ret = (objid[0] << (8*3)) + (objid[1] << (8*2)) +
             (objid[2] << 8) + objid[3];
+	  long_ret = htonl(long_ret);
           *var_len = sizeof(long_ret);
           vp->type = ASN_IPADDRESS;
           return((unsigned char *) &long_ret);
@@ -290,10 +303,10 @@ int
 setPassPersist(int action,
 	       u_char *var_val,
 	       u_char var_val_type,
-	       int var_val_len,
+	       size_t var_val_len,
 	       u_char *statP,
 	       oid *name,
-	       int name_len)
+	       size_t name_len)
 {
   int i, j, rtest, tmplen=1000, last;
   struct extensible *persistpassthru;
@@ -334,8 +347,7 @@ setPassPersist(int action,
         case ASN_COUNTER:
         case ASN_GAUGE:
         case ASN_TIMETICKS:
-          asn_parse_int(var_val,&tmplen,&var_val_type, &tmp,
-                        sizeof(tmp));
+          tmp = *((long *) var_val);
           switch (var_val_type) {
             case ASN_INTEGER:
               sprintf(buf,"integer %d",(int) tmp);
@@ -352,8 +364,8 @@ setPassPersist(int action,
           }
           break;
         case ASN_IPADDRESS:
-          asn_parse_unsigned_int(var_val,&tmplen,&var_val_type, &utmp,
-                                 sizeof(utmp));
+          utmp = *((u_long *) var_val);
+	  utmp = ntohl(utmp);
           sprintf(buf,"ipaddress %d.%d.%d.%d",
                   (int) ((utmp & 0xff000000) >> (8*3)),
                   (int) ((utmp & 0xff0000) >> (8*2)),
@@ -361,14 +373,17 @@ setPassPersist(int action,
                   (int) ((utmp & 0xff)));
           break;
         case ASN_OCTET_STR:
-          itmp = sizeof(buf);
-          memset(buf2,(0),itmp);
-          asn_parse_string(var_val,&tmplen,&var_val_type,buf2,&itmp);
-          sprintf(buf,"string %s",buf2);
+	  itmp = sizeof(buf2);
+	  memset(buf2, 0, itmp);
+          memcpy(buf2, var_val, var_val_len);
+          if (bin2asc(buf2, itmp) == (int)itmp)
+              sprintf(buf,"string %s",buf2);
+          else
+              sprintf(buf,"octet %s",buf2);
           break;
         case ASN_OBJECT_ID:
-          itmp = sizeof(objid);
-          asn_parse_objid(var_val,&tmplen,&var_val_type,objid,&itmp);
+          itmp = var_val_len/sizeof(oid);
+          memcpy(objid, var_val, var_val_len);
           sprint_mib_oid(buf2, objid, itmp);
           sprintf(buf,"objectid \"%s\"",buf2);
           break;
@@ -406,11 +421,11 @@ setPassPersist(int action,
   return SNMP_ERR_NOSUCHNAME;
 }
 
-int pass_persist_compare(void *a, void *b)
+int pass_persist_compare(const void *a, const void *b)
 {
-  struct extensible **ap, **bp;
-  ap = (struct extensible **) a;
-  bp = (struct extensible **) b;
+  const struct extensible * const *ap, * const *bp;
+  ap = (const struct extensible * const *) a;
+  bp = (const struct extensible * const *) b;
   return snmp_oid_compare((*ap)->miboid,(*ap)->miblen,(*bp)->miboid,(*bp)->miblen);
 }
 
@@ -526,13 +541,15 @@ static int open_persist_pipe(int iindex, char *command)
   return 1;
 }
 
+#if STRUCT_SIGACTION_HAS_SA_SIGACTION
 /* Generic handler */
 void sigpipe_handler (int sig, siginfo_t *sip, void *uap)
 {
   return;
 }
+#endif
 
-static int write_persist_pipe(int iindex,  char *data)
+static int write_persist_pipe(int iindex, const char *data)
 {
   struct sigaction sa, osa;
   int wret = 0, werrno = 0;
@@ -544,7 +561,9 @@ static int write_persist_pipe(int iindex,  char *data)
 
   /* Setup our signal action to catch SIGPIPEs */
   sa.sa_handler = NULL;
+#if STRUCT_SIGACTION_HAS_SA_SIGACTION
   sa.sa_sigaction = &sigpipe_handler;
+#endif
   sigemptyset(&sa.sa_mask);
   sa.sa_flags = 0;
   if( sigaction( SIGPIPE, &sa, &osa ) ) {
@@ -572,7 +591,7 @@ static int write_persist_pipe(int iindex,  char *data)
   return 1;
 }
 
-static int close_persist_pipe(int iindex)
+static void close_persist_pipe(int iindex)
 {
 
   /* Check and nix every item */
