@@ -2,7 +2,11 @@
 
 #include <sys/types.h>
 #if TIME_WITH_SYS_TIME
-# include <sys/time.h>
+# ifdef WIN32
+#  include <sys/timeb.h>
+# else
+#  include <sys/time.h>
+# endif
 # include <time.h>
 #else
 # if HAVE_SYS_TIME_H
@@ -14,6 +18,9 @@
 #if HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
+#if HAVE_WINSOCK_H
+#include <winsock.h>
+#endif
 
 #include "mibincl.h"
 #include "struct.h"
@@ -21,7 +28,7 @@
 #include "util_funcs.h"
 #include "read_config.h"
 #include "agent_read_config.h"
-#include "../../../snmplib/system.h"
+#include "system.h"
 
 #include "version.h"
 
@@ -54,11 +61,11 @@ void init_versioninfo(void)
 }
 
 
-unsigned char *var_extensible_version(struct variable *vp,
+u_char *var_extensible_version(struct variable *vp,
 				      oid *name,
-				      int *length,
+				      size_t *length,
 				      int exact,
-				      int *var_len,
+				      size_t *var_len,
 				      WriteMethod **write_method)
 {
 
@@ -66,15 +73,13 @@ unsigned char *var_extensible_version(struct variable *vp,
   static char errmsg[300];
   char *cptr;
   time_t curtime;
-  char c_oid[SPRINT_MAX_LEN];
 #ifdef CONFIGURE_OPTIONS
-  static char *config_opts = CONFIGURE_OPTIONS;
+  static char config_opts[] = CONFIGURE_OPTIONS;
 #endif
 
-  if (snmp_get_do_debugging()) {
-    sprint_objid (c_oid, name, *length);
-    DEBUGMSGTL(("ucd-snmp/versioninfo", "versionMib: %s %d\n", c_oid, exact));
-  }
+    DEBUGMSGTL(("ucd-snmp/versioninfo", "var_extensible_version: "));
+    DEBUGMSGOID(("ucd-snmp/versioninfo", name, *length));
+    DEBUGMSG(("ucd-snmp/versioninfo"," %d\n", exact));
 
   if (header_generic(vp,name,length,exact,var_len,write_method))
     return(NULL);
@@ -88,7 +93,7 @@ unsigned char *var_extensible_version(struct variable *vp,
       *var_len = strlen(errmsg);
       return((u_char *) errmsg); 
     case VERDATE:
-      sprintf(errmsg,"$Date: 1999/02/12 00:10:25 $");
+      sprintf(errmsg,"$Date: 2000/01/13 05:09:40 $");
       *var_len = strlen(errmsg);
       return((u_char *) errmsg); 
     case VERCDATE:
@@ -98,7 +103,7 @@ unsigned char *var_extensible_version(struct variable *vp,
       *var_len = strlen(errmsg);
       return((u_char *) errmsg);
     case VERIDENT:
-      sprintf(errmsg,"$Id: versioninfo.c,v 1.8 1999/02/12 00:10:25 hardaker Exp $");
+      sprintf(errmsg,"$Id: versioninfo.c,v 1.18 2000/01/13 05:09:40 mslifcak Exp $");
       *var_len = strlen(errmsg);
       return((u_char *) errmsg);
     case VERCONFIG:
@@ -134,21 +139,20 @@ int
 update_hook(int action,
 	    u_char *var_val,
 	    u_char var_val_type,
-	    int var_val_len,
+	    size_t var_val_len,
 	    u_char *statP,
 	    oid *name,
-	    int name_len)
+	    size_t name_len)
 {
   long tmp=0;
-  int tmplen=1000;
 
   if (var_val_type != ASN_INTEGER) {
-    printf("Wrong type != int\n");
+    snmp_log(LOG_ERR, "Wrong type != int\n");
     return SNMP_ERR_WRONGTYPE;
   }
-  asn_parse_int(var_val,&tmplen,&var_val_type,&tmp,sizeof(int));
+  tmp = *((long *) var_val);
   if (tmp == 1 && action == COMMIT) {
-    update_config(0);
+    update_config();
   } 
   return SNMP_ERR_NOERROR;
 }
@@ -157,19 +161,18 @@ int
 debugging_hook(int action,
 	       u_char *var_val,
 	       u_char var_val_type,
-	       int var_val_len,
+	       size_t var_val_len,
 	       u_char *statP,
 	       oid *name,
-	       int name_len)
+	       size_t name_len)
 {
   long tmp=0;
-  int tmplen=1000;
 
   if (var_val_type != ASN_INTEGER) {
-    printf("Wrong type != int\n");
+    DEBUGMSGTL(("versioninfo", "Wrong type != int\n"));
     return SNMP_ERR_WRONGTYPE;
   }
-  asn_parse_int(var_val,&tmplen,&var_val_type,&tmp,sizeof(int));
+  tmp = *((long *) var_val);
   if (action == COMMIT) {
     snmp_set_do_debugging(tmp);
   } 
