@@ -11,6 +11,7 @@ config_require(ucd-snmp/file)
 config_arch_require(linux, ucd-snmp/memory)
 config_arch_require(hpux9, ucd-snmp/memory)
 config_arch_require(hpux10, ucd-snmp/memory)
+config_arch_require(solaris2, ucd-snmp/vmstat_solaris2)
 config_arch_require(solaris2, ucd-snmp/memory_solaris2)
 config_arch_require(hpux9, ucd-snmp/hpux)
 config_arch_require(hpux10, ucd-snmp/hpux)
@@ -19,4 +20,7 @@ config_arch_require(bsdi3, ucd-snmp/memory)
 config_arch_require(linux, ucd-snmp/vmstat)
 config_arch_require(freebsd2, ucd-snmp/vmstat_freebsd2)
 config_arch_require(freebsd2, ucd-snmp/memory_freebsd2)
+config_arch_require(freebsd4, ucd-snmp/vmstat_freebsd2)
+config_arch_require(freebsd4, ucd-snmp/memory_freebsd2)
 config_add_mib(UCD-SNMP-MIB) 
+config_add_mib(UCD-DEMO-MIB) 
