@@ -35,19 +35,24 @@ struct variable;
 typedef int (WriteMethod)(int action,
         u_char  *var_val,
         u_char   var_val_type,
-        int      var_val_len,
+        size_t   var_val_len,
         u_char  *statP,
         oid     *name,
-        int      length);
+        size_t   length);
 
 /* Function pointer called by the master agent for mib information retrieval */
 typedef u_char *(FindVarMethod)(struct variable *vp,
         oid     *name,
-        int     *length,
+        size_t  *length,
         int      exact,
-        int     *var_len,
+        size_t  *var_len,
         WriteMethod   **write_method);
 
+/* Function pointer called by the master agent for setting up subagent requests */
+struct agent_snmp_session; 
+typedef int (AddVarMethod)(struct agent_snmp_session *asp,
+        struct variable_list     *vbp);
+
 struct nlist;
 
 extern long long_return;
@@ -68,34 +73,16 @@ struct variable {
     oid		    name[MAX_OID_LEN];	    /* object identifier of variable */
 };
 
+void init_agent(const char *);
 int subtree_old_size (void);
 void sort_tree (void);
-struct subtree *find_subtree (oid *, int, struct subtree *);
-struct subtree *find_subtree_next (oid *, int, struct subtree *);
-void register_mib (char *, struct variable *, int , int , oid *, int);
-void unregister_mib (oid *, int);
-struct subtree *unregister_mib_tree (oid *, int, struct subtree *);
-struct subtree *free_subtree (struct subtree *);
-void init_agent(void);
-int compare_tree (oid *, int, oid *, int);
-
-/* REGISTER_MIB(): This macro simply loads register_mib with less pain:
-
-   descr:   A short description of the mib group being loaded.
-   var:     The variable structure to load.
-   vartype: The variable structure used to define it (variable2, variable4, ...)
-   theoid:  A *initialized* *exact length* oid pointer.
-            (sizeof(theoid) *must* return the number of elements!) 
-*/
-#define REGISTER_MIB(descr, var, vartype, theoid)                      \
-  register_mib(descr, (struct variable *) var, sizeof(struct vartype), \
-               sizeof(var)/sizeof(struct vartype),                     \
-               theoid, sizeof(theoid)/sizeof(oid));
 
+#ifndef _AGENT_REGISTRY_H
+#include "agent_registry.h"
+#endif
 
 /* fail overloads non-negative integer value. it must be -1 ! */
 #define MATCH_FAILED	(-1)
 #define MATCH_SUCCEEDED	0
 
 #endif /* _SNMP_VARS_H_ */
-
