@@ -64,7 +64,8 @@
 #endif
 #if HAVE_WINSOCK_H
 #include <winsock.h>
-#else
+#endif
+#if HAVE_NETDB_H
 #include <netdb.h>
 #endif
 #if HAVE_ARPA_INET_H
@@ -78,11 +79,14 @@
 #include "mib.h"
 #include "snmp.h"
 #include "system.h"
+#include "int64.h"
 #include "snmp_parse_args.h"
+#include "snmp_logging.h"
+#include "default_store.h"
 
 #define MAX_ARGS 256
     
-char *SumFile = (char*)"Sum";
+const char *SumFile = "Sum";
 char *Argv[MAX_ARGS];
 int Argc;
 
@@ -90,9 +94,11 @@ int Argc;
 struct varInfo {
   char *name;
   oid *info_oid;
-  int oidlen;
+  int type;
+  size_t oidlen;
   char descriptor[64];
   u_int value;
+  struct counter64 c64value;
   float max;
   time_t time;
   int peak_count;
@@ -112,6 +118,7 @@ void usage (void)
   fprintf(stderr, "  -f config\tload configuration from file\n");
   fprintf(stderr, "  -p period\tspecifies the poll period\n");
   fprintf(stderr, "  -P peaks\treporting period in poll periods\n");
+  fprintf(stderr, "  -v vars/pkt\tnumber of variables per packet\n");
   fprintf(stderr, "  -k\t\tkeep seconds in output time\n");
   fprintf(stderr, "  -m\t\tshow max values\n");
   fprintf(stderr, "  -S\t\tlog to a sum file\n");
@@ -189,7 +196,7 @@ void sprint_descriptor(char *buffer,
 void processFileArgs(char *fileName)
 {
   FILE *fp;
-  char buf[257], *cp;
+  char buf[260], *cp;
   int blank, linenumber = 0;
 
   fp = fopen(fileName, "r");
@@ -197,7 +204,7 @@ void processFileArgs(char *fileName)
     return;
   while (fgets(buf, sizeof(buf), fp)){
     linenumber++;
-    if (strlen(buf) > 256){
+    if (strlen(buf) > (sizeof(buf)-2)){
       fprintf(stderr, "Line too long on line %d of %s\n",
 	      linenumber, fileName);
       exit(1);
@@ -264,14 +271,14 @@ void wait_for_period(int period)
     case -1:
       /* FALLTHRU */
     default:
-      perror("select");
+      snmp_log_perror("select");
       break;
     }
   }
 }
 
 oid sysUpTimeOid[9] = { 1, 3, 6, 1, 2, 1, 1, 3, 0 };
-int sysUpTimeLen = 9;
+size_t sysUpTimeLen = 9;
 
 int main(int argc, char *argv[])
 {
@@ -283,7 +290,8 @@ int main(int argc, char *argv[])
 
   int	count, current_name = 0;
   struct varInfo varinfo[128], *vip;
-  u_int value;
+  u_int value=0;
+  struct counter64 c64value;
   float printvalue;
   int period = 1;
   int deltat = 0, timestamp = 0, fileout = 0, dosum = 0, printmax = 0;
@@ -292,7 +300,7 @@ int main(int argc, char *argv[])
   time_t last_time = 0;
   time_t this_time;
   time_t delta_time;
-  int sum;
+  int sum; /* what the heck is this for, its never used? */
   char filename[128];
   struct timeval tv;
   struct tm tm;
@@ -304,7 +312,7 @@ int main(int argc, char *argv[])
   int print = 1;
   int exit_code = 0;
     
-  arg = snmp_parse_args(argc, argv, &session);
+  arg = snmp_parse_args(argc, argv, &session, NULL, NULL);
   gateway = session.peername;
 
   /*
@@ -380,10 +388,12 @@ int main(int argc, char *argv[])
   }
 
   SOCK_STARTUP;
-  snmp_synch_setup(&session);
+
+  /* open an SNMP session */
   ss = snmp_open(&session);
   if (ss == NULL){
-    snmp_perror("snmpdelta");
+    /* diagnose snmp_open errors with the input struct snmp_session pointer */
+    snmp_sess_perror("snmpdelta", &session);
     SOCK_CLEANUP;
     exit(1);
   }
@@ -397,7 +407,7 @@ int main(int argc, char *argv[])
       vip->oidlen = MAX_OID_LEN;
       vip->info_oid = (oid *)malloc(sizeof(oid) * vip->oidlen);
       if (snmp_parse_oid(vip->name, vip->info_oid, &vip->oidlen) == NULL) {
-	fprintf(stderr, "Invalid object identifier: %s\n", vip->name);
+	snmp_perror(vip->name);
 	SOCK_CLEANUP;
 	exit(1);
       }
@@ -408,6 +418,7 @@ int main(int argc, char *argv[])
       strcpy(vip->descriptor, SumFile);
     }
     vip->value = 0;
+    zeroU64(&vip->c64value);
     vip->time = 0;
     vip->max = 0;
     if (peaks){
@@ -480,8 +491,15 @@ int main(int argc, char *argv[])
 	      fprintf(stderr, "Missing variable in reply\n");
 	      break;
 	    }
-	    value = *(vars->val.integer) - vip->value;
-	    vip->value = *(vars->val.integer);
+	    vip->type = vars->type;
+            if (vars->type == ASN_COUNTER64) {
+              u64Subtract(vars->val.counter64, &vip->c64value, &c64value);
+              memcpy(&vip->c64value, &vars->val.counter64,
+                     sizeof(struct counter64));
+            } else {
+              value = *(vars->val.integer) - vip->value;
+              vip->value = *(vars->val.integer);
+            }
 	    vars = vars->next_variable;
 	  } else {
 	    value = sum;
@@ -495,8 +513,9 @@ int main(int argc, char *argv[])
 	  if (last_time == 0)
 	    continue;
 
-	  if (vip->oidlen)
-	    sum += value;
+	  if (vip->oidlen && vip->type != ASN_COUNTER64) {
+            sum += value;
+          }
 
 	  if (tableForm) {
 	    if (count == begin) sprintf(outstr, "%s", timestring+1);
@@ -506,13 +525,18 @@ int main(int argc, char *argv[])
 	    sprintf(outstr, "%s %s", timestring, vip->descriptor);
 
 	  if (deltat || tableForm){
-	    printvalue = ((float)value * 100) / delta_time;
-	    if (tableForm) sprintf(valueStr, "\t%.2f", printvalue);
-	    else sprintf(valueStr, " /sec: %.2f", printvalue);
+            if (vip->type == ASN_COUNTER64) {
+              fprintf(stderr, "time delta and table form not supported for counter64s\n");
+              exit(1);
+            } else {
+              printvalue = ((float)value * 100) / delta_time;
+              if (tableForm) sprintf(valueStr, "\t%.2f", printvalue);
+              else sprintf(valueStr, " /sec: %.2f", printvalue);
+            }
 	  } else {
 	    printvalue = (float)value;
-	    sprintf(valueStr, " /%d sec: %.0f",
-		    period, printvalue);
+	    sprintf(valueStr, " /%d sec: ", period);
+            printU64(valueStr+strlen(valueStr), &c64value);
 	  }
 
 	  if (!peaks){
@@ -607,7 +631,12 @@ int main(int argc, char *argv[])
 	  exit_code = 1;
 	  break;
 	}
-	if ((pdu = snmp_fix_pdu(response, SNMP_MSG_GET)) != NULL)
+
+        /* retry if the errored variable was successfully removed */
+        pdu = snmp_fix_pdu(response, SNMP_MSG_GET);
+        snmp_free_pdu(response);
+	response = NULL;
+        if (pdu != NULL)
 	  goto retry;
       }
 	    
@@ -616,7 +645,7 @@ int main(int argc, char *argv[])
       response = 0;
       break;
     } else {    /* status == STAT_ERROR */
-      snmp_perror("snmpdelta");
+      snmp_sess_perror("snmpdelta", ss);
       response = 0;
       break;
     }
