@@ -61,7 +61,8 @@ SOFTWARE.
 #endif
 #if HAVE_WINSOCK_H
 #include <winsock.h>
-#else
+#endif
+#if HAVE_NETDB_H
 #include <netdb.h>
 #endif
 #if HAVE_ARPA_INET_H
@@ -77,13 +78,12 @@ SOFTWARE.
 #include "system.h"
 #include "snmp_parse_args.h"
 
-
 int failures=0;
 
 void
 usage (void)
 {
-  fprintf(stderr,"Usage:\n  snmpget ");
+  fprintf(stderr,"Usage: snmpget ");
   snmp_parse_args_usage(stderr);
   fprintf(stderr," [<objectID> ...]\n\n");
   snmp_parse_args_descriptions(stderr);
@@ -100,11 +100,17 @@ int main(int argc, char *argv[])
     int current_name = 0;
     char *names[128];
     oid name[MAX_OID_LEN];
-    int name_length;
+    size_t name_length;
     int status;
 
     /* get the common command line arguments */
-    arg = snmp_parse_args(argc, argv, &session);
+    arg = snmp_parse_args(argc, argv, &session, NULL, NULL);
+
+    if (arg >= argc) {
+      fprintf(stderr, "Missing object name\n");
+      usage();
+      exit(1);
+    }
 
     /* get the object names */
     for(; arg < argc; arg++)
@@ -112,21 +118,27 @@ int main(int argc, char *argv[])
     
     SOCK_STARTUP;
 
-    /* open an SNMP session */
-    snmp_synch_setup(&session);
+
+    /* 
+     * Open an SNMP session.
+     */
     ss = snmp_open(&session);
     if (ss == NULL){
-      snmp_perror("snmpget");
+      /* diagnose snmp_open errors with the input struct snmp_session pointer */
+      snmp_sess_perror("snmpget", &session);
       SOCK_CLEANUP;
       exit(1);
     }
 
-    /* create PDU for GET request and add object names to request */
+
+    /* 
+     * Create PDU for GET request and add object names to request.
+     */
     pdu = snmp_pdu_create(SNMP_MSG_GET);
     for(count = 0; count < current_name; count++){
       name_length = MAX_OID_LEN;
       if (!snmp_parse_oid(names[count], name, &name_length)) {
-        fprintf(stderr, "Invalid object identifier: %s\n", names[count]);
+        snmp_perror(names[count]);
         failures++;
       } else
         snmp_add_null_var(pdu, name, name_length);
@@ -136,44 +148,64 @@ int main(int argc, char *argv[])
       exit(1);
     }
 
-    /* do the request */
+
+    /* 
+     * Perform the request.
+     *
+     * If the Get Request fails, note the OID that caused the error,
+     * "fix" the PDU (removing the error-prone OID) and retry.
+     */
 retry:
     status = snmp_synch_response(ss, pdu, &response);
     if (status == STAT_SUCCESS){
       if (response->errstat == SNMP_ERR_NOERROR){
         for(vars = response->variables; vars; vars = vars->next_variable)
           print_variable(vars->name, vars->name_length, vars);
+
       } else {
         fprintf(stderr, "Error in packet\nReason: %s\n",
                 snmp_errstring(response->errstat));
+
         if (response->errstat == SNMP_ERR_NOSUCHNAME){
           fprintf(stderr, "This name doesn't exist: ");
           for(count = 1, vars = response->variables; 
                 vars && count != response->errindex;
                 vars = vars->next_variable, count++)
-            ;
+            /*EMPTY*/ ;
           if (vars)
             fprint_objid(stderr, vars->name, vars->name_length);
           fprintf(stderr, "\n");
         }
-        if ((pdu = snmp_fix_pdu(response, SNMP_MSG_GET)) != NULL)
+
+        /* retry if the errored variable was successfully removed */
+        pdu = snmp_fix_pdu(response, SNMP_MSG_GET);
+        snmp_free_pdu(response);
+	response = NULL;
+        if (pdu != NULL)
           goto retry;
-      }
+
+      }  /* endif -- SNMP_ERR_NOERROR */
+
     } else if (status == STAT_TIMEOUT){
 	fprintf(stderr,"Timeout: No Response from %s.\n", session.peername);
 	snmp_close(ss);
 	SOCK_CLEANUP;
 	exit(1);
+
     } else {    /* status == STAT_ERROR */
-      snmp_perror("snmpget");
+      snmp_sess_perror("snmpget", ss);
       snmp_close(ss);
       SOCK_CLEANUP;
       exit(1);
-    }
+
+    }  /* endif -- STAT_SUCCESS */
+
 
     if (response)
       snmp_free_pdu(response);
     snmp_close(ss);
     SOCK_CLEANUP;
-    exit (0);
-}
+    return 0;
+
+}  /* end main() */
+
