@@ -61,7 +61,8 @@ SOFTWARE.
 #endif
 #if HAVE_WINSOCK_H
 #include <winsock.h>
-#else
+#endif
+#if HAVE_NETDB_H
 #include <netdb.h>
 #endif
 #if HAVE_ARPA_INET_H
@@ -80,7 +81,7 @@ SOFTWARE.
 
 void usage(void)
 {
-  fprintf(stderr,"Usage:\n  snmpgetnext ");
+  fprintf(stderr,"Usage: snmpgetnext ");
   snmp_parse_args_usage(stderr);
   fprintf(stderr," [<objectID> ...]\n\n");
   snmp_parse_args_descriptions(stderr);
@@ -96,12 +97,18 @@ int main(int argc, char *argv[])
     int   current_name = 0;
     char  *names[128];
     oid   name[MAX_OID_LEN];
-    int   name_length;
+    size_t name_length;
     int   status;
     int   failures = 0;
 
     /* get the common command line arguments */
-    arg = snmp_parse_args(argc, argv, &session);
+    arg = snmp_parse_args(argc, argv, &session, NULL, NULL);
+
+    if (arg >= argc) {
+      fprintf(stderr, "Missing object name\n");
+      usage();
+      exit(1);
+    }
 
     /* get the object names */
     for(; arg < argc; arg++)
@@ -110,10 +117,10 @@ int main(int argc, char *argv[])
     SOCK_STARTUP;
 
     /* open an SNMP session */
-    snmp_synch_setup(&session);
     ss = snmp_open(&session);
     if (ss == NULL){
-      snmp_perror("snmpgetnext");
+      /* diagnose snmp_open errors with the input struct snmp_session pointer */
+      snmp_sess_perror("snmpgetnext", &session);
       SOCK_CLEANUP;
       exit(1);
     }
@@ -124,7 +131,7 @@ int main(int argc, char *argv[])
     for(count = 0; count < current_name; count++){
       name_length = MAX_OID_LEN;
       if (snmp_parse_oid(names[count], name, &name_length) == NULL) {
-        fprintf(stderr, "Invalid object identifier: %s\n", names[count]);
+        snmp_perror(names[count]);
         failures++;
       } else
         snmp_add_null_var(pdu, name, name_length);
@@ -154,18 +161,23 @@ retry:
             fprint_objid(stderr, vars->name, vars->name_length);
           fprintf(stderr, "\n");
         }
-        if ((pdu = snmp_fix_pdu(response, SNMP_MSG_GETNEXT)) != NULL)
+
+        /* retry if the errored variable was successfully removed */
+        pdu = snmp_fix_pdu(response, SNMP_MSG_GETNEXT);
+        snmp_free_pdu(response);
+	response = NULL;
+        if (pdu != NULL)
           goto retry;
       }
     } else if (status == STAT_TIMEOUT){
       fprintf(stderr, "Timeout: No Response from %s.\n", session.peername);
     } else {    /* status == STAT_ERROR */
-      snmp_perror("snmpgetnext");
+      snmp_sess_perror("snmpgetnext", ss);
     }
 
     if (response)
       snmp_free_pdu(response);
     snmp_close(ss);
     SOCK_CLEANUP;
-    exit (0);
+    return 0;
 }
