@@ -59,8 +59,11 @@ SOFTWARE.
 
 #if HAVE_WINSOCK_H
 #include <winsock.h>
-#else
+#endif
+#if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#if HAVE_NETDB_H
 #include <netdb.h>
 #endif
 
@@ -86,6 +89,7 @@ static oid oid_ifinucastpkts[] ={1, 3, 6, 1, 2, 1, 2, 2, 1,11, 1};
 static oid oid_cfg_nnets[] =	{1, 3, 6, 1, 2, 1, 2, 1, 0};
 static oid oid_ipadentaddr[] =	{1, 3, 6, 1, 2, 1, 4,20, 1, 1, 0, 0, 0, 0};
 
+#define IFINDEX		1
 #define IFNAME		2
 #define IFMTU		4
 #define IFOPERSTATUS	8
@@ -100,7 +104,7 @@ static oid oid_ipadentaddr[] =	{1, 3, 6, 1, 2, 1, 4,20, 1, 1, 0, 0, 0, 0};
 #define OUTQLEN		21
 
 #define IPADDR		1
-#define	IFINDEX		2
+#define	IPIFINDEX	2
 #define IPNETMASK	3
 
 
@@ -111,22 +115,27 @@ void
 intpr(int interval)
 {
 	oid varname[MAX_OID_LEN], *instance, *ifentry;
-	int varname_len;
+	size_t varname_len;
 	int ifnum, cfg_nnets;
 	oid curifip [4];
 	struct variable_list *var;
 	struct snmp_pdu *request, *response;
 	int status;
-	int ifindex;
+	int ifindex, oldindex = 0;
 	struct _if_info {
 	    char name[128];
 	    char ip[128], route[128];
 	    int mtu;
-	    int ipkts, ierrs, opkts, oerrs, operstatus, outqueue;
+	    int ifindex;
+	    char s_ipkts[20], s_ierrs[20], s_opkts[20], s_oerrs[20], s_outq[20];
+	    unsigned long ipkts, opkts;
+	    int operstatus;
 	    u_long netmask;
 	    struct in_addr ifip, ifroute;
 	} *if_table, *cur_if;
-	int max_name = 0, max_ip = 0, max_route = 0, i;
+	int max_name = 4, max_ip = 7, max_route = 7, max_ipkts = 5,
+	    max_ierrs = 5, max_opkts = 5, max_oerrs = 5, max_outq = 5;
+	int i;
 
 	if (interval) {
 		sidewaysintpr((unsigned)interval);
@@ -155,7 +164,7 @@ intpr(int interval)
 		varname_len = sizeof (oid_ipadentaddr) / sizeof (oid);
 		instance = varname + 9;
 		memmove (varname + 10, curifip, sizeof (curifip));
-		*instance = IFINDEX;
+		*instance = IPIFINDEX;
 		snmp_add_null_var (request, varname, varname_len);
 		*instance = IPADDR;
 		snmp_add_null_var (request, varname, varname_len);
@@ -165,17 +174,22 @@ intpr(int interval)
 		status = snmp_synch_response (Session, request, &response);
 		if (status != STAT_SUCCESS || response->errstat != SNMP_ERR_NOERROR) {
 		    fprintf (stderr, "SNMP request failed after %d out of %d interfaces (IP)\n", ifnum, cfg_nnets);
+		    if (snmp_get_do_debugging()) {
+			fprintf(stderr, "status = %d, errstat = %ld, errindex = %ld\n",
+				status, response->errstat, response->errindex);
+		    }
 		    cfg_nnets = ifnum;
 		    break;
 		}
 		for (var = response->variables; var; var = var->next_variable) {
-                  if (snmp_get_do_debugging()) {
-		    print_variable (var->name, var->name_length, var);
-                  }
+		    if (snmp_get_do_debugging()) {
+			print_variable (var->name, var->name_length, var);
+		    }
 		    switch (var->name [9]) {
-		    case IFINDEX:
+		    case IPIFINDEX:
 			ifindex = *var->val.integer;
-			cur_if = if_table + ifindex - 1;
+			for (cur_if = if_table; cur_if->ifindex != ifindex && cur_if->ifindex != 0; cur_if++) ;
+			cur_if->ifindex = ifindex;
 			break;
 		    case IPADDR:
 			memmove (curifip, var->name+10, sizeof (curifip));
@@ -187,7 +201,7 @@ intpr(int interval)
 		}
 		cur_if->ifroute.s_addr = cur_if->ifip.s_addr & cur_if->netmask;
 		if (cur_if->ifroute.s_addr)
-			strcpy(cur_if->route, netname (cur_if->ifroute, 0));
+			strcpy(cur_if->route, netname (cur_if->ifroute, cur_if->netmask));
 		else strcpy(cur_if->route, "none");
 		if ((i = strlen(cur_if->route)) > max_route) max_route = i;
 		if (cur_if->ifip.s_addr)
@@ -201,9 +215,11 @@ intpr(int interval)
 		varname_len = sizeof(oid_ifname) / sizeof(oid);
 		ifentry = varname + 9;
 		instance = varname + 10;
-		request = snmp_pdu_create (SNMP_MSG_GET);
+		request = snmp_pdu_create (SNMP_MSG_GETNEXT);
 
-		*instance = ifnum;
+		*instance = oldindex;
+		*ifentry = IFINDEX;
+		snmp_add_null_var (request, varname, varname_len);
 		*ifentry = IFNAME;
 		snmp_add_null_var (request, varname, varname_len);
 		*ifentry = IFMTU;
@@ -225,7 +241,13 @@ intpr(int interval)
 		*ifentry = OUTQLEN;
 		snmp_add_null_var (request, varname, varname_len);
 
-		status = snmp_synch_response (Session, request, &response);
+		while ((status = snmp_synch_response (Session, request, &response)) == STAT_SUCCESS) {
+		    if (response->errstat != SNMP_ERR_NOSUCHNAME)
+			break;
+		    if ((request = snmp_fix_pdu(response, SNMP_MSG_GETNEXT)) == NULL)
+			break;
+		    snmp_free_pdu(response);
+		}
 		if (status != STAT_SUCCESS || response->errstat != SNMP_ERR_NOERROR) {
 		    fprintf (stderr, "SNMP request failed after %d out of %d interfaces (IF)\n", ifnum, cfg_nnets);
 		    cfg_nnets = ifnum;
@@ -233,28 +255,60 @@ intpr(int interval)
 		}
 		cur_if = if_table + ifnum - 1;
 		for (var = response->variables; var; var = var->next_variable) {
-                  if (snmp_get_do_debugging()) {
-		    print_variable (var->name, var->name_length, var);
-                  }
-		    if (var->val.integer)
+		    if (snmp_get_do_debugging()) {
+			print_variable (var->name, var->name_length, var);
+		    }
+		    if (!var->val.integer) continue;
+		    if (memcmp(var->name, oid_ifname, 8*sizeof(oid))) continue;
 		    switch (var->name [9]) {
+		    case IFINDEX:
+			ifindex = *var->val.integer;
+			for (cur_if = if_table; cur_if->ifindex != ifindex && cur_if->ifindex != 0; cur_if++) ;
+			cur_if->ifindex = ifindex;
+			break;
 		    case OUTQLEN:
-			cur_if->outqueue = *var->val.integer; break;
+			sprintf(cur_if->s_outq, "%lu", *var->val.integer);
+			i = strlen(cur_if->s_outq);
+			if (i > max_outq) max_outq = i;
+			break;
 		    case OUTERRORS:
-			cur_if->oerrs = *var->val.integer; break;
+			sprintf(cur_if->s_oerrs, "%lu", *var->val.integer);
+			i = strlen(cur_if->s_oerrs);
+			if (i > max_oerrs) max_oerrs = i;
+			break;
 		    case INERRORS:
-			cur_if->ierrs = *var->val.integer; break;
+			sprintf(cur_if->s_ierrs, "%lu", *var->val.integer);
+			i = strlen(cur_if->s_ierrs);
+			if (i > max_ierrs) max_ierrs = i;
+			break;
 		    case IFMTU:
 			cur_if->mtu = *var->val.integer; break;
 		    case INUCASTPKTS:
-			cur_if->ipkts += *var->val.integer; break;
+			cur_if->ipkts += *var->val.integer;
+			sprintf(cur_if->s_ipkts, "%lu", cur_if->ipkts);
+			i = strlen(cur_if->s_ipkts);
+			if (i > max_ipkts) max_ipkts = i;
+			break;
 		    case INNUCASTPKTS:
-			cur_if->ipkts += *var->val.integer; break;
+			cur_if->ipkts += *var->val.integer;
+			sprintf(cur_if->s_ipkts, "%lu", cur_if->ipkts);
+			i = strlen(cur_if->s_ipkts);
+			if (i > max_ipkts) max_ipkts = i;
+			break;
 		    case OUTUCASTPKTS:
-			cur_if->opkts += *var->val.integer; break;
+			cur_if->opkts += *var->val.integer;
+			sprintf(cur_if->s_opkts, "%lu", cur_if->opkts);
+			i = strlen(cur_if->s_opkts);
+			if (i > max_opkts) max_opkts = i;
+			break;
 		    case OUTNUCASTPKTS:
-			cur_if->opkts += *var->val.integer; break;
+			cur_if->opkts += *var->val.integer;
+			sprintf(cur_if->s_opkts, "%lu", cur_if->opkts);
+			i = strlen(cur_if->s_opkts);
+			if (i > max_opkts) max_opkts = i;
+			break;
 		    case IFNAME:
+			oldindex = var->name[10];
 			if (var->val_len >= sizeof(cur_if->name))
 			    var->val_len = sizeof(cur_if->name) - 1;
 			memmove (cur_if->name, var->val.string, var->val_len);
@@ -279,21 +333,25 @@ intpr(int interval)
 		}
 	}
 
-	printf("%*.*s %5.5s %*.*s %*.*s %9.9s %5.5s %9.9s %5.5s %5.5s",
+	printf("%*.*s %5.5s %*.*s %*.*s %*s %*s %*s %*s %*s",
 		-max_name, max_name, "Name", "Mtu",
 		-max_route, max_route, "Network",
-		-max_ip, max_ip, "Address", "Ipkts", "Ierrs",
-		"Opkts", "Oerrs", "Queue");
+		-max_ip, max_ip, "Address",
+		max_ipkts, "Ipkts",
+		max_ierrs, "Ierrs",
+		max_opkts, "Opkts",
+		max_oerrs, "Oerrs",
+		max_outq, "Queue");
 	putchar('\n');
 	for (ifnum = 0, cur_if = if_table; ifnum < cfg_nnets; ifnum++, cur_if++) {
 		if (cur_if->name [0] == 0) continue;
-		printf("%-*.*s %5d ", max_name, max_name, cur_if->name, cur_if->mtu);
-		printf("%-*.*s ", max_route, max_route, cur_if->route);
-		printf("%-*.*s ", max_ip, max_ip, cur_if->ip);
-		printf("%9d %5d %9d %5d %5d",
-		    cur_if->ipkts, cur_if->ierrs,
-		    cur_if->opkts, cur_if->oerrs,
-		    cur_if->outqueue);
+		printf("%*.*s %5d ", -max_name, max_name, cur_if->name, cur_if->mtu);
+		printf("%*.*s ", -max_route, max_route, cur_if->route);
+		printf("%*.*s ", -max_ip, max_ip, cur_if->ip);
+		printf("%*s %*s %*s %*s %*s",
+			max_ipkts, cur_if->s_ipkts, max_ierrs, cur_if->s_ierrs,
+			max_opkts, cur_if->s_opkts, max_oerrs, cur_if->s_oerrs,
+			max_outq, cur_if->s_outq);
 		putchar('\n');
 	}
 	free(if_table);
@@ -305,21 +363,24 @@ void
 intpro(int interval)
 {
 	oid varname[MAX_OID_LEN], *instance, *ifentry;
-	int varname_len;
+	size_t varname_len;
 	int ifnum, cfg_nnets;
 	oid curifip [4];
 	struct variable_list *var;
 	struct snmp_pdu *request, *response;
 	int status;
-	int ifindex;
+	int ifindex, oldindex = 0;
 	struct _if_info {
+	    int ifindex;
 	    char name[128];
 	    char ip[128], route[128];
-	    unsigned long ioctets, ierrs, ooctets, oerrs, operstatus, outqueue;
+	    char ioctets[20], ierrs[20], ooctets[20], oerrs[20], outqueue[20];
+	    int operstatus;
 	    u_long netmask;
 	    struct in_addr ifip, ifroute;
 	} *if_table, *cur_if;
-	int max_name = 0, max_route = 0, max_ip = 0, i;
+	int max_name = 4, max_route = 7, max_ip = 7, max_ioctets = 7, max_ooctets = 7;
+	int i;
 
 	if (interval) {
 		sidewaysintpr((unsigned)interval);
@@ -348,7 +409,7 @@ intpro(int interval)
 		varname_len = sizeof (oid_ipadentaddr) / sizeof (oid);
 		instance = varname + 9;
 		memmove (varname + 10, curifip, sizeof (curifip));
-		*instance = IFINDEX;
+		*instance = IPIFINDEX;
 		snmp_add_null_var (request, varname, varname_len);
 		*instance = IPADDR;
 		snmp_add_null_var (request, varname, varname_len);
@@ -366,9 +427,10 @@ intpro(int interval)
 		    print_variable (var->name, var->name_length, var);
                   }
 		    switch (var->name [9]) {
-		    case IFINDEX:
+		    case IPIFINDEX:
 			ifindex = *var->val.integer;
-			cur_if = if_table + ifindex - 1;
+			for (cur_if = if_table; cur_if->ifindex != ifindex && cur_if->ifindex != 0; cur_if++);
+			cur_if->ifindex = ifindex;
 			break;
 		    case IPADDR:
 			memmove (curifip, var->name+10, sizeof (curifip));
@@ -380,7 +442,7 @@ intpro(int interval)
 		}
 		cur_if->ifroute.s_addr = cur_if->ifip.s_addr & cur_if->netmask;
 		if (cur_if->ifroute.s_addr)
-			strcpy(cur_if->route, netname (cur_if->ifroute, 0));
+			strcpy(cur_if->route, netname (cur_if->ifroute, cur_if->netmask));
 		else strcpy(cur_if->route, "none");
 		if ((i = strlen(cur_if->route)) > max_route) max_route = i;
 		if (cur_if->ifip.s_addr)
@@ -394,9 +456,11 @@ intpro(int interval)
 		varname_len = sizeof(oid_ifname) / sizeof(oid);
 		ifentry = varname + 9;
 		instance = varname + 10;
-		request = snmp_pdu_create (SNMP_MSG_GET);
+		request = snmp_pdu_create (SNMP_MSG_GETNEXT);
 
-		*instance = ifnum;
+		*instance = oldindex;
+		*ifentry = IFINDEX;
+		snmp_add_null_var (request, varname, varname_len);
 		*ifentry = IFNAME;
 		snmp_add_null_var (request, varname, varname_len);
 		*ifentry = IFOPERSTATUS;
@@ -406,29 +470,41 @@ intpro(int interval)
 		*ifentry = OUTOCTETS;
 		snmp_add_null_var (request, varname, varname_len);
 
-		status = snmp_synch_response (Session, request, &response);
+		while ((status = snmp_synch_response (Session, request, &response)) == STAT_SUCCESS) {
+		    if (response->errstat != SNMP_ERR_NOSUCHNAME)
+			break;
+		    if ((request = snmp_fix_pdu(response, SNMP_MSG_GETNEXT)) == NULL)
+			break;
+		    snmp_free_pdu(response);
+		}
 		if (status != STAT_SUCCESS || response->errstat != SNMP_ERR_NOERROR) {
 		    fprintf (stderr, "SNMP request failed for interface %d, variable %ld out of %d interfaces (IF)\n", ifnum, response->errindex, cfg_nnets);
 		    cfg_nnets = ifnum;
 		    break;
 		}
-		cur_if = if_table + ifnum - 1;
 		for (var = response->variables; var; var = var->next_variable) {
-                  if (snmp_get_do_debugging()) {
-		    print_variable (var->name, var->name_length, var);
-                  }
+		    if (snmp_get_do_debugging()) {
+			print_variable (var->name, var->name_length, var);
+		    }
+		    if (!var->val.integer) continue;
 		    switch (var->name [9]) {
-		    case OUTQLEN:
-			cur_if->outqueue = *var->val.integer; break;
-		    case OUTERRORS:
-			cur_if->oerrs = *var->val.integer; break;
-		    case INERRORS:
-			cur_if->ierrs = *var->val.integer; break;
+		    case IFINDEX:
+			ifindex = *var->val.integer;
+			for (cur_if = if_table; cur_if->ifindex != ifindex && cur_if->ifindex != 0; cur_if++);
+			cur_if->ifindex = ifindex;
+			break;
 		    case INOCTETS:
-			cur_if->ioctets += *var->val.integer; break;
+			sprintf(cur_if->ioctets, "%lu",  *var->val.integer);
+			i = strlen(cur_if->ioctets);
+			if (i > max_ioctets) max_ioctets = i;
+			break;
 		    case OUTOCTETS:
-			cur_if->ooctets += *var->val.integer; break;
+			sprintf(cur_if->ooctets, "%lu",  *var->val.integer);
+			i = strlen(cur_if->ooctets);
+			if (i > max_ooctets) max_ooctets = i;
+			break;
 		    case IFNAME:
+			oldindex = var->name[10];
 			if (var->val_len >= sizeof(cur_if->name))
 			    var->val_len = sizeof(cur_if->name) - 1;
 			memmove (cur_if->name, var->val.string, var->val_len);
@@ -447,25 +523,26 @@ intpro(int interval)
 			continue;
 		}
 		if (cur_if->operstatus != MIB_IFSTATUS_UP) {
-			cp = (char *) strchr(cur_if->name, '\0');
+			cp = strchr(cur_if->name, '\0');
 			*cp++ = '*';
 			*cp = '\0';
 		}
 	}
 
-	printf("%*.*s %*.*s %*.*s %12.12s %12.12s ",
+	printf("%*.*s %*.*s %*.*s %*.*s %*.*s ",
 		-max_name, max_name, "Name",
 		-max_route, max_route, "Network",
-		-max_ip, max_ip, "Address", "Ioctets", "Ooctets");
+		-max_ip, max_ip, "Address",
+		max_ioctets, max_ioctets, "Ioctets",
+		max_ooctets, max_ooctets, "Ooctets");
 	putchar('\n');
 	for (ifnum = 0, cur_if = if_table; ifnum < cfg_nnets; ifnum++, cur_if++) {
 		if (cur_if->name [0] == 0) continue;
 		printf("%*.*s ", -max_name, max_name, cur_if->name);
 		printf("%*.*s ", -max_route, max_route, cur_if->route);
 		printf("%*.*s ", -max_ip, max_ip, cur_if->ip);
-		printf("%12lu %12lu",
-		    cur_if->ioctets,
-		    cur_if->ooctets);
+		printf("%*s %*s", max_ioctets, cur_if->ioctets,
+		    max_ioctets, cur_if->ooctets);
 		putchar('\n');
 	}
 	free(if_table);
@@ -473,12 +550,12 @@ intpro(int interval)
 
 #define	MAXIF	128
 struct	iftot {
-	char	ift_name[128];		/* interface name */
-	int	ift_ip;			/* input packets */
-	int	ift_ie;			/* input errors */
-	int	ift_op;			/* output packets */
-	int	ift_oe;			/* output errors */
-	int	ift_co;			/* collisions */
+	char		ift_name[128];		/* interface name */
+	unsigned int	ift_ip;			/* input packets */
+	unsigned int	ift_ie;			/* input errors */
+	unsigned int	ift_op;			/* output packets */
+	unsigned int	ift_oe;			/* output errors */
+	unsigned int	ift_co;			/* collisions */
 } iftot[MAXIF];
 
 u_char	signalled;			/* set if alarm goes off "early" */
@@ -574,7 +651,7 @@ sidewaysintpr(unsigned int interval)
 	struct iftot *lastif, *sum, *interesting, ifnow, *now = &ifnow;
 	struct variable_list *var;
 	oid varname[MAX_OID_LEN], *instance, *ifentry;
-	int varname_len;
+	size_t varname_len;
 	int ifnum, cfg_nnets;
 
 	lastif = iftot;
@@ -619,9 +696,9 @@ sidewaysintpr(unsigned int interval)
 	timerSet(interval);
 
 banner:
-	printf("    input   %-6.6s    output       ", interesting->ift_name);
+	printf("     input   %-6.6s     output       ", interesting->ift_name);
 	if (lastif - iftot > 0)
-		printf("     input  (Total)    output");
+		printf("                 input  (Total)     output");
 	for (ip = iftot; ip < iftot + MAXIF; ip++) {
 		ip->ift_ip = 0;
 		ip->ift_ie = 0;
@@ -630,10 +707,10 @@ banner:
 		ip->ift_co = 0;
 	}
 	putchar('\n');
-	printf("%8.8s %5.5s %8.8s %5.5s %5.5s ",
+	printf("%10.10s %8.8s %10.10s %8.8s %8.8s ",
 		"packets", "errs", "packets", "errs", "colls");
 	if (lastif - iftot > 0)
-		printf("%8.8s %5.5s %8.8s %5.5s %5.5s ",
+		printf("%10.10s %8.8s %10.10s %8.8s %8.8s ",
 			"packets", "errs", "packets", "errs", "colls");
 	putchar('\n');
 	fflush(stdout);
@@ -689,7 +766,7 @@ loop:
 		}
 
 		if (ip == interesting)
-			printf("%8d %5d %8d %5d %5d ",
+			printf("%10d %8d %10d %8d %8d ",
 				now->ift_ip - ip->ift_ip,
 				now->ift_ie - ip->ift_ie,
 				now->ift_op - ip->ift_op,
@@ -707,7 +784,7 @@ loop:
 		sum->ift_co += ip->ift_co;
 	}
 	if (lastif - iftot > 0)
-		printf("%8d %5d %8d %5d %5d ",
+		printf("%10d %8d %10d %8d %8d ",
 			sum->ift_ip - total->ift_ip,
 			sum->ift_ie - total->ift_ie,
 			sum->ift_op - total->ift_op,
