@@ -62,8 +62,11 @@ SOFTWARE.
 #if HAVE_WINSOCK_H
 #include <winsock.h>
 #include "winstub.h"
-#else
+#endif
+#if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#if HAVE_NETDB_H
 #include <netdb.h>
 #endif
 
@@ -201,30 +204,30 @@ static oid oid_tcpconntable[] = {1, 3, 6, 1, 2, 1, 6, 13, 1};
 static oid oid_udptable[] = {1, 3, 6, 1, 2, 1, 7, 5, 1};
 #define UDP_ENTRY 9
 
-char *tcpstates[] = {
-    (char*)"",
-    (char*)"CLOSED",
-    (char*)"LISTEN",
-    (char*)"SYNSENT",
-    (char*)"SYNRECEIVED",
-    (char*)"ESTABLISHED",
-    (char*)"FINWAIT1",
-    (char*)"FINWAIT2",
-    (char*)"CLOSEWAIT",
-    (char*)"LASTACK",
-    (char*)"CLOSING",
-    (char*)"TIMEWAIT"
+const char *tcpstates[] = {
+    "",
+    "CLOSED",
+    "LISTEN",
+    "SYNSENT",
+    "SYNRECEIVED",
+    "ESTABLISHED",
+    "FINWAIT1",
+    "FINWAIT2",
+    "CLOSEWAIT",
+    "LASTACK",
+    "CLOSING",
+    "TIMEWAIT"
 };
 #define TCP_NSTATES 11
 
-int validUShortAssign( unsigned short *, int, char *);
+int validUShortAssign( unsigned short *, int, const char *);
 
 /*
  * Print a summary of connections related to an Internet
  * protocol (currently only TCP).  For TCP, also give state of connection.
  */
 void
-protopr (char *name)
+protopr (const char *name)
 {
     struct tcpconn_entry *tcpconn = NULL, *tcplast = NULL, *tp, *newtp;
     struct udp_entry *udpconn = NULL, *udplast = NULL, *up, *newup;
@@ -247,7 +250,7 @@ protopr (char *name)
     response = NULL;
 	status = snmp_synch_response(Session, request, &response);
 	if (status != STAT_SUCCESS || response->errstat != SNMP_ERR_NOERROR){
-	    snmp_perror("SNMP request failed");
+	    snmp_sess_perror("SNMP request failed", Session);
 	    break;
 	}
 	vp = response->variables;
@@ -266,11 +269,11 @@ protopr (char *name)
 		    break;
 	}
 	if (tp == NULL){
-	    tp = (struct tcpconn_entry *)malloc(sizeof(struct tcpconn_entry));
+	    tp = (struct tcpconn_entry *)calloc(1, sizeof(struct tcpconn_entry));
+	    if (tp == NULL) break;
 	    if (tcplast != NULL) tcplast->next = tp;
 	    tcplast = tp;
 	    if (tcpconn == NULL) tcpconn = tp;
-	    memset(tp, 0, sizeof(*tp));
 	    memmove(tp->instance, instance, sizeof(tp->instance));
 	}
 
@@ -367,12 +370,11 @@ protopr (char *name)
 		    break;
 	}
 	if (up == NULL){
-	    up = (struct udp_entry *)malloc(sizeof(struct udp_entry));
+	    up = (struct udp_entry *)calloc(1, sizeof(struct udp_entry));
+	    if (up == NULL) break;
 	    if (udplast != NULL) udplast->next = up;
 	    udplast = up;
 	    if (udpconn == NULL) udpconn = up;
-	    memset(up, 0, sizeof(*up));
-	    up->next = NULL;
 	    memmove(up->instance, instance, sizeof(up->instance));
 	}
 
@@ -413,7 +415,7 @@ protopr (char *name)
 }
 
 int
-validUShortAssign( unsigned short * pushort, int ival, char * errstr)
+validUShortAssign( unsigned short * pushort, int ival, const char * errstr)
 {
     u_long ulval = (u_long)ival;
     if (ival > 65535)
@@ -434,7 +436,7 @@ void
 udp_stats(void)
 {
     oid varname[MAX_OID_LEN], *udpentry;
-    int varname_len;
+    size_t varname_len;
     struct variable_list *var;
     int count;
     struct stat_table *sp = udp_stattab;
@@ -465,7 +467,7 @@ void
 tcp_stats(void)
 {
     oid varname[MAX_OID_LEN], *tcpentry;
-    int varname_len;
+    size_t varname_len;
     struct variable_list *var;
     int count;
     struct stat_table *sp = tcp_stattab;
@@ -496,7 +498,7 @@ void
 ip_stats(void)
 {
     oid varname[MAX_OID_LEN], *ipentry;
-    int varname_len;
+    size_t varname_len;
     struct variable_list *var;
     int count;
     struct stat_table *sp = ip_stattab;
@@ -527,7 +529,7 @@ void
 icmp_stats(void)
 {
     oid varname[MAX_OID_LEN], *icmpentry;
-    int varname_len;
+    size_t varname_len;
     struct variable_list *var;
     int count, first;
     struct stat_table *sp;
@@ -596,7 +598,7 @@ icmp_stats(void)
 void
 inetprint(struct in_addr *in,
 	  u_short port,
-	  char *proto)
+	  const char *proto)
 {
 	struct servent *sp = 0;
 	char line[80], *cp;
