@@ -75,8 +75,11 @@ char copyright[] =
 #if HAVE_WINSOCK_H
 #include <winsock.h>
 #include "winstub.h"
-#else
+#endif
+#if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#if HAVE_NETDB_H
 #include <netdb.h>
 #endif
 
@@ -96,27 +99,28 @@ char copyright[] =
 #include "snmp_api.h"
 #include "snmp_impl.h"
 #include "snmp_client.h"
-#include "party.h"
-#include "context.h"
-#include "view.h"
-#include "acl.h"
 #include "version.h"
 #include "snmp_debug.h"
 #include "system.h"
+#include "default_store.h"
 
 #include "netstat.h"
 
 #define NULLPROTOX	((struct protox *) 0)
 struct protox {
 	u_char	pr_wanted;		/* 1 if wanted, 0 otherwise */
-	void	(*pr_cblocks) (char *);	/* control blocks printing routine */
+	void	(*pr_cblocks) (const char *);	/* control blocks printing routine */
 	void	(*pr_stats) (void);	/* statistics printing routine */
-	char	*pr_name;		/* well-known name */
+	const char	*pr_name;	/* well-known name */
 } protox[] = {
-	{ 0,	protopr,    tcp_stats,	(char*)"tcp" },
-	{ 0,	protopr,    udp_stats,	(char*)"udp" },
-	{ 0,	0,	    ip_stats,	(char*)"ip" },
-	{ 0,	0,	    icmp_stats,	(char*)"icmp" },
+	{ 0,	protopr,    tcp_stats,	"tcp" },
+	{ 0,	protopr,    udp_stats,	"udp" },
+#ifdef INET6
+	{ 0,	protopr6,   tcp_stats,	"tcp6" },
+	{ 0,	protopr6,   udp_stats,	"udp6" },
+#endif
+	{ 0,	0,	    ip_stats,	"ip" },
+	{ 0,	0,	    icmp_stats,	"icmp" },
 	{ 0,	0,	    0,		0 }
 };
 
@@ -132,11 +136,12 @@ char	*intrface;
 struct snmp_session *Session;
 int print_errors = 0;
 
+static struct protoent *getprotoent46(void);
+
 void usage(void)
 {
     fprintf(stderr, "Usage: snmpnetstat [-v 1 | -v 2c] [-q] [-D] hostname community [-ainrs] [-P proto] [-I interface] [interval]      or:\n");
-    fprintf(stderr, "Usage: snmpnetstat [-v 2p] [-q] [-D] hostname noAuth [-ainrs] [-P proto] [-I interface] [interval]       or:\n");
-    fprintf(stderr, "Usage: snmpnetstat [-v 2p] [-q] [-D] hostname srcParty dstParty context [-ainrs] [-P proto] [-I interface] [interval]\n");
+    fprintf(stderr, "Usage: snmpnetstat [-q] [-D] hostname noAuth [-ainrs] [-P proto] [-I interface] [interval]\n");
 }
 
 int main(int argc, char *argv[])
@@ -147,28 +152,18 @@ int main(int argc, char *argv[])
     int allprotos = 1;
     char *community = NULL;
     struct snmp_session session;
-    char ctmp[128];
     int dest_port = SNMP_PORT;
     int clock_flag = 0;
     u_long	srcclock = 0, dstclock = 0;
     int version = SNMP_VERSION_1;
-    struct partyEntry *pp;
-    struct contextEntry *cxp;
-    oid src[MAX_OID_LEN], dst[MAX_OID_LEN], context[MAX_OID_LEN];
-    int srclen = 0, dstlen = 0, contextlen = 0;
+    size_t srclen = 0, dstlen = 0, contextlen = 0;
     int trivialSNMPv2 = FALSE;
-    struct hostent *hp;
-    in_addr_t destAddr;
     int arg;
-#ifdef _DEBUG_MALLOC_INC
-    unsigned long histid1, histid2, orig_size, current_size;
-#endif
 
     init_mib();
     /*
      * Usage: snmpnetstatwalk -v 1 [-q] hostname community ...      or:
-     * Usage: snmpnetstat [-v 2 ] [-q] hostname noAuth     ...      or:
-     * Usage: snmpnetstat [-v 2 ] [-q] hostname srcParty dstParty context ...
+     * Usage: snmpnetstat [-v 2 ] [-q] hostname noAuth     ...
      */
     for(arg = 1; arg < argc; arg++){
 	if (argv[arg][0] == '-'){
@@ -210,8 +205,6 @@ int main(int argc, char *argv[])
 		    version = SNMP_VERSION_1;
 		else if (!strcmp(community,"2c"))
 		    version = SNMP_VERSION_2c;
-		else if (!strcmp(community,"2p"))
-		    version = SNMP_VERSION_2p;
 		else {
 		    fprintf(stderr, "Invalid version: %s\n", community);
 		    usage();
@@ -280,81 +273,6 @@ int main(int argc, char *argv[])
 	} else if ((version == SNMP_VERSION_1 || version == SNMP_VERSION_2c)
                    && community == NULL){
 	    community = argv[arg]; 
-	} else if (version == SNMP_VERSION_2p && srclen == 0 && !trivialSNMPv2){
-	    sprintf(ctmp, "%s/party.conf", SNMPSHAREPATH);
-	    if (read_party_database(ctmp) > 0){
-		snmp_perror(argv[0]);
-		exit(1);
-	    }
-	    sprintf(ctmp, "%s/context.conf", SNMPSHAREPATH);
-	    if (read_context_database(ctmp) > 0){
-		snmp_perror(argv[0]);
-		exit(1);
-	    }
-	    sprintf(ctmp, "%s/acl.conf", SNMPSHAREPATH);
-	    if (read_acl_database(ctmp) > 0){
-		snmp_perror(argv[0]);
-		exit(1);
-	    }
-	    
-	    if (!strcasecmp(argv[arg], "noauth")){
-		trivialSNMPv2 = TRUE;
-	    } else {
-		party_scanInit();
-		for(pp = party_scanNext(); pp; pp = party_scanNext()){
-		    if (!strcasecmp(pp->partyName, argv[arg])){
-			srclen = pp->partyIdentityLen;
-			memmove(src, pp->partyIdentity, srclen * sizeof(oid));
-			break;
-		    }
-		}
-		if (!pp){
-		    srclen = MAX_OID_LEN;
-		    if (!read_objid(argv[arg], src, &srclen)){
-			printf("Invalid source party: %s\n", argv[arg]);
-			srclen = 0;
-			usage();
-			exit(1);
-		    }
-		}
-	    }
-	} else if (version == SNMP_VERSION_2p && dstlen == 0 && !trivialSNMPv2){
-	    dstlen = MAX_OID_LEN;
-	    party_scanInit();
-	    for(pp = party_scanNext(); pp; pp = party_scanNext()){
-		if (!strcasecmp(pp->partyName, argv[arg])){
-		    dstlen = pp->partyIdentityLen;
-		    memmove(dst, pp->partyIdentity, dstlen * sizeof(oid));
-		    break;
-		}
-	    }
-	    if (!pp){
-		if (!read_objid(argv[arg], dst, &dstlen)){
-		    printf("Invalid destination party: %s\n", argv[arg]);
-		    dstlen = 0;
-		    usage();
-		    exit(1);
-		}
-	    }
-	} else if (version == SNMP_VERSION_2p && contextlen == 0 && !trivialSNMPv2){
-	    contextlen = MAX_OID_LEN;
-	    context_scanInit();
-	    for(cxp = context_scanNext(); cxp; cxp = context_scanNext()){
-		if (!strcasecmp(cxp->contextName, argv[arg])){
-		    contextlen = cxp->contextIdentityLen;
-		    memmove(context, cxp->contextIdentity,
-			  contextlen * sizeof(oid));
-		    break;
-		}
-	    }
-	    if (!cxp){
-		if (!read_objid(argv[arg], context, &contextlen)){
-		    printf("Invalid context: %s\n", argv[arg]);
-		    contextlen = 0;
-		    usage();
-		    exit(1);
-		}
-	    }
 	} else if (isdigit(argv[arg][0])) {
             interval = atoi(argv[arg]);
             if (interval <= 0){
@@ -369,77 +287,31 @@ int main(int argc, char *argv[])
     }
     
     if (!hostname ||
-	((version == SNMP_VERSION_1 || version == SNMP_VERSION_2c) && !community)
-	|| (version == SNMP_VERSION_2p && (!srclen || !dstlen || !contextlen)
-	    && !trivialSNMPv2)){
+	((version == SNMP_VERSION_1 || version == SNMP_VERSION_2c) && !community)) {
 	usage();
 	exit(1);
     }
-    
-    if (trivialSNMPv2){
-	if ((destAddr = inet_addr(hostname)) == -1){
-	    hp = gethostbyname(hostname);
-	    if (hp == NULL){
-		fprintf(stderr, "unknown host: %s\n", hostname);
-		exit(1);
-	    } else {
-              memmove(&destAddr, hp->h_addr, hp->h_length);
-	    }
-	}
-	srclen = dstlen = contextlen = MAX_OID_LEN;
-	ms_party_init(destAddr, src, &srclen, dst, &dstlen,
-		      context, &contextlen);
-    }
-    
-    if (clock_flag){
-	pp = party_getEntry(src, srclen);
-	if (pp){
-            pp->partyAuthClock = srcclock;
-            gettimeofday(&pp->tv, (struct timezone *)0);
-            pp->tv.tv_sec -= pp->partyAuthClock;
-	}
-	pp = party_getEntry(dst, dstlen);
-	if (pp){
-            pp->partyAuthClock = dstclock;
-            gettimeofday(&pp->tv, (struct timezone *)0);
-            pp->tv.tv_sec -= pp->partyAuthClock;
-	}
-    }
-    
-    
-    memset(&session, 0, sizeof(struct snmp_session));
+
+    snmp_sess_init(&session);
     session.peername = hostname;
     session.remote_port = dest_port;
     if (version == SNMP_VERSION_1 || version == SNMP_VERSION_2c){
         session.version = version;
         session.community = (u_char *)community;
         session.community_len = strlen((char *)community);
-    } else if (version == SNMP_VERSION_2p){
-        session.version = version;
-        session.srcParty = src;
-        session.srcPartyLen = srclen;
-        session.dstParty = dst;
-        session.dstPartyLen = dstlen;
-        session.context = context;
-        session.contextLen = contextlen;
     }
 
     SOCK_STARTUP;
-    session.retries = SNMP_DEFAULT_RETRIES;
-    session.timeout = SNMP_DEFAULT_TIMEOUT;
-    session.authenticator = NULL;
-    snmp_synch_setup(&session);
+
+    /* open an SNMP session */
     Session = snmp_open(&session);
     if (Session == NULL){
-        snmp_perror("snmpnetstat: Couldn't open snmp");
+      /* diagnose snmp_open errors with the input struct snmp_session pointer */
+        snmp_sess_perror("snmpnetstat", &session);
         SOCK_CLEANUP;
 	exit(1);
     }
 
-#ifdef _DEBUG_MALLOC_INC
-    orig_size = malloc_inuse(&histid1);
-#endif
-    
     /*
      * Keep file descriptors open to avoid overhead
      * of open/close on each call to get* routines.
@@ -466,10 +338,11 @@ int main(int argc, char *argv[])
 	;
     else {
 
-    while ((p = getprotoent())) {
-	for (tp = protox; tp->pr_name; tp++)
+    while ((p = getprotoent46())) {
+	for (tp = protox; tp->pr_name; tp++) {
 	    if (strcmp(tp->pr_name, p->p_name) == 0)
 		break;
+	}
 	if (tp->pr_name == 0 || (tp->pr_wanted == 0 && allprotos == 0))
 	    continue;
 	if (sflag) {
@@ -488,27 +361,22 @@ int main(int argc, char *argv[])
 
     snmp_close(Session);
 
-#ifdef _DEBUG_MALLOC_INC
-    current_size = malloc_inuse(&histid2);
-    if (current_size != orig_size) malloc_list(2, histid1, histid2);
-#endif
-
     SOCK_CLEANUP;
-    exit(0);
+    return 0;
 }
 
-char *
+const char *
 plural(int n)
 {
 
-	return (n != 1 ? (char *)"s" : (char *)"");
+	return (n != 1 ? "s" : "");
 }
 
 /*
  * Find the protox for the given "well-known" name.
  */
 struct protox * 
-knownname(char *name)
+knownname(const char *name)
 {
 	struct protox *tp;
 	
@@ -522,7 +390,7 @@ knownname(char *name)
  * Find the protox corresponding to name.
  */
 struct protox *
-name2protox(char *name)
+name2protox(const char *name)
 {
 	struct protox *tp;
 	char **alias;			/* alias from p->aliases */
@@ -536,7 +404,7 @@ name2protox(char *name)
 		return(tp);
 		
 	setprotoent(1);			/* make protocol lookup cheaper */
-	while ((p = getprotoent())) {
+	while ((p = getprotoent46())) {
 		/* assert: name not same as p->name */
 		for (alias = p->p_aliases; *alias; alias++)
 			if (strcasecmp(name, *alias) == 0) {
@@ -547,3 +415,36 @@ name2protox(char *name)
 	endprotoent();
 	return(NULLPROTOX);
 }
+
+static struct protoent *
+getprotoent46(void)
+{
+#ifdef INET6
+	static enum {NONE, V4, V6} state = NONE;
+	static struct protoent v;
+	struct protoent *p;
+	int l;
+
+	switch (state) {
+	case NONE:
+		p = getprotoent();
+		if (!p)
+			return p;
+		memcpy(&v, p, sizeof(v));
+		state = V4;
+		break;
+	case V4:
+		strcat(v.p_name, "4");
+		state = V6;
+		break;
+	case V6:
+		l = strlen(v.p_name);
+		v.p_name[l - 1] = '6';
+		state = NONE;
+		break;
+	}
+	return &v;
+#else
+	return getprotoent();
+#endif
+}
