@@ -61,7 +61,8 @@ SOFTWARE.
 #endif
 #if HAVE_WINSOCK_H
 #include <winsock.h>
-#else
+#endif
+#if HAVE_NETDB_H
 #include <netdb.h>
 #endif
 #if HAVE_ARPA_INET_H
@@ -82,7 +83,7 @@ int failures = 0;
 
 void usage(void)
 {
-  fprintf(stderr,"Usage:\n  snmpset ");
+  fprintf(stderr,"Usage: snmpset ");
   snmp_parse_args_usage(stderr);
   fprintf(stderr," [<objectID> <type> <value> ...]\n\n");
   snmp_parse_args_descriptions(stderr);
@@ -102,7 +103,7 @@ void usage(void)
 int main(int argc, char *argv[])
 {
     struct snmp_session session, *ss;
-    struct snmp_pdu *pdu, *response;
+    struct snmp_pdu *pdu, *response = NULL;
     struct variable_list *vars;
     int arg;
     int count;
@@ -113,11 +114,17 @@ int main(int argc, char *argv[])
     char types[128];
     char *values[128];
     oid name[MAX_OID_LEN];
-    int name_length;
+    size_t name_length;
     int status;
 
     /* get the common command line arguments */
-    arg = snmp_parse_args(argc, argv, &session);
+    arg = snmp_parse_args(argc, argv, &session, NULL, NULL);
+
+    if (arg >= argc) {
+      fprintf(stderr, "Missing object name\n");
+      usage();
+      exit(1);
+    }
 
     /* get object names, types, and values */
     for(; arg < argc; arg++){
@@ -162,10 +169,11 @@ int main(int argc, char *argv[])
     SOCK_STARTUP;
 
     /* open an SNMP session */
-    snmp_synch_setup(&session);
     ss = snmp_open(&session);
     if (ss == NULL){
-      snmp_perror("snmpset");
+      /* diagnose snmp_open errors with the input struct snmp_session pointer */
+      snmp_sess_perror("snmpset", &session);
+	  SOCK_CLEANUP;
       exit(1);
     }
 
@@ -174,11 +182,11 @@ int main(int argc, char *argv[])
     for(count = 0; count < current_name; count++){
       name_length = MAX_OID_LEN;
       if (snmp_parse_oid(names[count], name, &name_length) == NULL) {
-        fprintf(stderr, "Invalid object identifier: %s\n", names[count]);
+        snmp_perror(names[count]);
         failures++;
       } else
         if (snmp_add_var(pdu, name, name_length, types[count], values[count])) {
-          snmp_perror("snmpset");
+          snmp_perror(names[count]);
           failures++;
         }
     }
@@ -208,7 +216,12 @@ retry:
               fprint_objid(stderr, vars->name, vars->name_length);
             fprintf(stderr, "\n");
           }
-          if ((pdu = snmp_fix_pdu(response, SNMP_MSG_SET)) != NULL)
+
+          /* retry if the errored variable was successfully removed */
+          pdu = snmp_fix_pdu(response, SNMP_MSG_SET);
+          snmp_free_pdu(response);
+	  response = NULL;
+          if (pdu != NULL)
             goto retry;
         }
       } else if (status == STAT_TIMEOUT){
@@ -217,7 +230,7 @@ retry:
 	SOCK_CLEANUP;
 	exit(1);
       } else {    /* status == STAT_ERROR */
-        snmp_perror("snmpset");
+        snmp_sess_perror("snmpset", ss);
 	snmp_close(ss);
 	SOCK_CLEANUP;
 	exit(1);
@@ -227,5 +240,5 @@ retry:
       snmp_free_pdu(response);
     snmp_close(ss);
     SOCK_CLEANUP;
-    exit (0);
+    return 0;
 }
