@@ -61,7 +61,8 @@ SOFTWARE.
 #endif
 #if HAVE_WINSOCK_H
 #include <winsock.h>
-#else
+#endif
+#if HAVE_NETDB_H
 #include <netdb.h>
 #endif
 #if HAVE_ARPA_INET_H
@@ -75,67 +76,43 @@ SOFTWARE.
 #include "mib.h"
 #include "snmp.h"
 #include "system.h"
+#include "tools.h"
 #include "snmp_parse_args.h"
 
 oid	objid_sysDescr[] = {1, 3, 6, 1, 2, 1, 1, 1, 0};
-int	length_sysDescr = sizeof(objid_sysDescr)/sizeof(oid);
+size_t	length_sysDescr = sizeof(objid_sysDescr)/sizeof(oid);
 oid	objid_sysUpTime[] = {1, 3, 6, 1, 2, 1, 1, 3, 0};
-int	length_sysUpTime = sizeof(objid_sysUpTime)/sizeof(oid);
+size_t	length_sysUpTime = sizeof(objid_sysUpTime)/sizeof(oid);
 oid	objid_ifOperStatus[] = {1, 3, 6, 1, 2, 1, 2, 2, 1, 8};
-int	length_ifOperStatus = sizeof(objid_ifOperStatus)/sizeof(oid);
+size_t	length_ifOperStatus = sizeof(objid_ifOperStatus)/sizeof(oid);
 oid	objid_ifInUCastPkts[] = {1, 3, 6, 1, 2, 1, 2, 2, 1, 11};
-int	length_ifInUCastPkts = sizeof(objid_ifInUCastPkts)/sizeof(oid);
+size_t	length_ifInUCastPkts = sizeof(objid_ifInUCastPkts)/sizeof(oid);
 oid	objid_ifInNUCastPkts[] = {1, 3, 6, 1, 2, 1, 2, 2, 1, 12};
-int	length_ifInNUCastPkts = sizeof(objid_ifInNUCastPkts)/sizeof(oid);
+size_t	length_ifInNUCastPkts = sizeof(objid_ifInNUCastPkts)/sizeof(oid);
 oid	objid_ifOutUCastPkts[] = {1, 3, 6, 1, 2, 1, 2, 2, 1, 17};
-int	length_ifOutUCastPkts = sizeof(objid_ifOutUCastPkts)/sizeof(oid);
+size_t	length_ifOutUCastPkts = sizeof(objid_ifOutUCastPkts)/sizeof(oid);
 oid	objid_ifOutNUCastPkts[] = {1, 3, 6, 1, 2, 1, 2, 2, 1, 18};
-int	length_ifOutNUCastPkts = sizeof(objid_ifOutNUCastPkts)/sizeof(oid);
+size_t	length_ifOutNUCastPkts = sizeof(objid_ifOutNUCastPkts)/sizeof(oid);
 oid	objid_ipInReceives[] = {1, 3, 6, 1, 2, 1, 4, 3, 0};
-int	length_ipInReceives = sizeof(objid_ipInReceives)/sizeof(oid);
+size_t	length_ipInReceives = sizeof(objid_ipInReceives)/sizeof(oid);
 oid	objid_ipOutRequests[] = {1, 3, 6, 1, 2, 1, 4, 10, 0};
-int	length_ipOutRequests = sizeof(objid_ipOutRequests)/sizeof(oid);
+size_t	length_ipOutRequests = sizeof(objid_ipOutRequests)/sizeof(oid);
 
 
 void usage(void)
 {
-  fprintf(stderr,"Usage:\n  snmpstatus ");
+  fprintf(stderr,"Usage: snmpstatus ");
   snmp_parse_args_usage(stderr);
   fprintf(stderr,"\n\n");
   snmp_parse_args_descriptions(stderr);
 }
 
-#ifndef CMU_COMPATIBLE
-char *uptime_string(u_long timeticks,
-		    char *buf)
-{
-    int  seconds, minutes, hours, days;
-
-    timeticks /= 100;
-    days = timeticks / (60 * 60 * 24);
-    timeticks %= (60 * 60 * 24);
-
-    hours = timeticks / (60 * 60);
-    timeticks %= (60 * 60);
-
-    minutes = timeticks / 60;
-    seconds = timeticks % 60;
-
-    if (days == 0){
-      sprintf(buf, "%d:%02d:%02d", hours, minutes, seconds);
-    } else if (days == 1) {
-      sprintf(buf, "%d day, %d:%02d:%02d", days, hours, minutes, seconds);
-    } else {
-      sprintf(buf, "%d days, %d:%02d:%02d", days, hours, minutes, seconds);
-    }
-    return buf;
-}
-#endif /* !CMU_COMPATIBLE */
 
 int main(int argc, char *argv[])
 {
     struct snmp_session session, *ss;
     struct snmp_pdu *pdu, *response;
+    struct sockaddr_in *respIp;
     struct variable_list *vars;
     char *sysdescr = NULL;
     u_long uptime = 0;
@@ -148,15 +125,15 @@ int main(int argc, char *argv[])
     int count;
 
     /* get the common command line arguments */
-    snmp_parse_args(argc, argv, &session);
+    snmp_parse_args(argc, argv, &session, NULL, NULL);
 
     SOCK_STARTUP;
 
     /* open an SNMP session */
-    snmp_synch_setup(&session);
     ss = snmp_open(&session);
     if (ss == NULL){
-      snmp_perror("snmpstatus");
+      /* diagnose snmp_open errors with the input struct snmp_session pointer */
+      snmp_sess_perror("snmpstatus", &session);
       SOCK_CLEANUP;
       exit(1);
     }
@@ -206,7 +183,12 @@ retry:
             fprint_objid(stderr, vars->name, vars->name_length);
           fprintf(stderr, "\n");
         }
-        if ((pdu = snmp_fix_pdu(response, SNMP_MSG_GET)) != NULL)
+
+        /* retry if the errored variable was successfully removed */
+        pdu = snmp_fix_pdu(response, SNMP_MSG_GET);
+        snmp_free_pdu(response);
+	response = NULL;
+        if (pdu != NULL)
           goto retry;
       }
     } else if (status == STAT_TIMEOUT){
@@ -214,12 +196,13 @@ retry:
       SOCK_CLEANUP;
       exit(1);
     } else {    /* status == STAT_ERROR */
-      snmp_perror("snmpstatus");
+      snmp_sess_perror("snmpstatus", ss);
       SOCK_CLEANUP;
       exit(2);
     }
 
-    printf("[%s]=>[%s] Up: %s\n", inet_ntoa(response->address.sin_addr),
+    respIp = (struct sockaddr_in *)&(response->address);
+    printf("[%s]=>[%s] Up: %s\n", inet_ntoa(respIp->sin_addr),
             sysdescr, uptime_string(uptime, buf));
 
     if (response)
@@ -295,7 +278,7 @@ retry:
       } else if (status == STAT_TIMEOUT){
         fprintf(stderr,"Timeout: No Response from %s\n", session.peername);
       } else {    /* status == STAT_ERROR */
-        snmp_perror("snmpstatus");
+        snmp_sess_perror("snmpstatus", ss);
       }
 
       if (response)
@@ -310,5 +293,5 @@ retry:
 
     snmp_close(ss);
     SOCK_CLEANUP;
-    exit (0);
+    return 0;
 }
