@@ -1,6 +1,10 @@
 /*
  * snmptable.c - walk a table and print it nicely
  *
+ * Update: 1999-10-26 <rs-snmp@revelstone.com>
+ * Added ability to use MIB to query tables with non-sequential column OIDs
+ * Added code to handle sparse tables
+ *
  * Update: 1998-07-17 <jhy@gsu.edu>
  * Added text <special options> to usage().
  */
@@ -58,15 +62,15 @@ SOFTWARE.
 #include <stdio.h>
 #if HAVE_WINSOCK_H
 #include <winsock.h>
-#else
+#endif
+#if HAVE_NETDB_H
 #include <netdb.h>
 #endif
 #if HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
-#if HAVE_GETOPT_H
+
 #include <getopt.h>
-#endif
 
 #include "asn1.h"
 #include "snmp_api.h"
@@ -74,11 +78,14 @@ SOFTWARE.
 #include "mib.h"
 #include "snmp.h"
 #include "snmp_impl.h"
+#include "default_store.h"
 #include "system.h"
 #include "apps/snmp_parse_args.h"
+#include "parse.h"
 
 struct column {
   int width;
+  int subid;
   char *label;
   char *fmt;
 } *column = NULL;
@@ -97,60 +104,28 @@ static int show_index = 0;
 static char *field_separator = NULL;
 static char *table_name;
 static oid name[MAX_OID_LEN];
-static int name_length;
+static size_t name_length;
 static oid root[MAX_OID_LEN];
-static int rootlen;
+static size_t rootlen;
 static int localdebug;
-static struct snmp_session session;
+static int nonsequential = 1;
 
-void get_field_names (void);
-void get_table_entries (void);
+void get_field_names (char *);
+void get_table_entries( struct snmp_session *ss );
 void print_table (void);
 
-void usage(void)
+static void optProc(int argc, char *const *argv, int opt)
 {
-  fprintf(stderr,"Usage:\n  snmptable ");
-  snmp_parse_args_usage(stderr);
-  fprintf(stderr," [<special options]> [<objectID>]\n\n");
-  snmp_parse_args_descriptions(stderr);
-  fprintf(stderr,"\n<special options>\n");
-  fprintf(stderr,"  -w <W>        print table in parts of W characters width\n");
-  fprintf(stderr,"  -f <F>        print an F delimited table\n");
-  fprintf(stderr,"  -h            print only the table headings\n");
-  fprintf(stderr,"  -H            don't print the table headings\n");
-  fprintf(stderr,"  -b            brief field names\n");
-  fprintf(stderr,"  -x            print index value\n");
-  exit(1);
-}
-
-int main(int argc, char *argv[])
-{
-  int   arg;
-#ifdef _DEBUG_MALLOC_INC
-  unsigned long histid1, histid2, orig_size, current_size;
-#endif
-
-  setvbuf(stdout, NULL, _IOLBF, 1024);
-  snmp_set_quick_print(1);
-
-  /* get the common command line arguments */
-  arg = snmp_parse_args(argc, argv, &session);
-  snmp_set_suffix_only(0);
-  
-  for(; (arg < argc) && (argv[arg][0] == '-') ; arg++){
-      switch(argv[arg][1]){
+    switch (opt) {
     case 'w':
-      max_width = atoi(argv[++arg]);
+      max_width = atoi(optarg);
       if (max_width == 0) {
-	fprintf(stderr, "Bad -w option: %s\n", argv[++arg]);
+	fprintf(stderr, "Bad -w option: %s\n", optarg);
 	usage();
       }
       break;
     case 'f':
-      if (argv[arg][2])
-        field_separator = &argv[arg][2];
-      else
-        field_separator = argv[++arg];
+      field_separator = optarg;
       break;
     case 'h':
       headers_only = 1;
@@ -158,47 +133,87 @@ int main(int argc, char *argv[])
     case 'H':
       no_headers = 1;
       break;
+    case 'C':
+      nonsequential = 0;
+      break;
     case 'b':
       brief = 1;
       break;
-    case 'x':
+    case 'i':
       show_index = 1;
       break;
-    default:
-      usage();
     }
-  }
+}
+
+void usage(void)
+{
+  fprintf(stdout,"Usage: snmptable ");
+  snmp_parse_args_usage(stdout);
+  fprintf(stdout," <objectID>\n\n");
+  snmp_parse_args_descriptions(stdout);
+  fprintf(stdout,"  -w <W>\tprint table in parts of W characters width\n");
+  fprintf(stdout,"  -f <F>\tprint an F delimited table\n");
+  fprintf(stdout,"  -b\t\tbrief field names\n");
+  fprintf(stdout,"  -i\t\tprint index value\n");
+  exit(1);
+}
+
+int main(int argc, char *argv[])
+{
+  struct snmp_session session, *ss;
+  char *tblname;
+
+  setvbuf(stdout, NULL, _IOLBF, 1024);
+  snmp_set_quick_print(1);
+
+  /* get the common command line arguments */
+  snmp_parse_args(argc, argv, &session, "w:f:ChHbi", optProc);
 
   /* get the initial object and subtree */
   /* specified on the command line */
-  if (arg+1 == argc) {
-    rootlen = MAX_OID_LEN;
-    if (!snmp_parse_oid(argv[arg], root, &rootlen)){
-      fprintf(stderr, "Invalid object identifier: %s\n", argv[arg]);
-      exit(1);
-    }
-    localdebug = snmp_get_dump_packet();
-  } else {
-    fprintf(stderr,"Missing table name\n");
+  if (optind+1 != argc) {
+    fprintf(stderr,"Must have exactly one table name\n");
     usage();
   }
 
-#ifdef _DEBUG_MALLOC_INC
-  orig_size = malloc_inuse(&histid1);
-#endif
+  rootlen = MAX_OID_LEN;
+  if (!snmp_parse_oid(argv[optind], root, &rootlen)){
+    snmp_perror(argv[optind]);
+    exit(1);
+  }
+  localdebug = snmp_get_dump_packet();
+  if( nonsequential ){
+    tblname = strrchr( argv[optind], '.' );
+    if (!tblname)
+      tblname = strrchr( argv[optind], ':' );
+    if( tblname )
+      ++tblname;
+    else
+      tblname = argv[optind];
+  }
+  else
+    tblname = NULL;
 
-  get_field_names();
+  get_field_names( tblname );
 
-  if (headers_only == 0) get_table_entries();
+  /* open an SNMP session */
+  SOCK_STARTUP;
+  ss = snmp_open(&session);
+  if (ss == NULL){
+    /* diagnose snmp_open errors with the input struct snmp_session pointer */
+    snmp_sess_perror("snmptable", &session);
+    SOCK_CLEANUP;
+    exit(1);
+  }
+
+  if (headers_only == 0) get_table_entries(ss);
+
+  snmp_close(ss);
+  SOCK_CLEANUP;
 
   print_table();
 
-#ifdef _DEBUG_MALLOC_INC
-  current_size = malloc_inuse(&histid2);
-  if (current_size != orig_size) malloc_list(2, histid1, histid2);
-#endif
-
-  exit (0);
+  return 0;
 }
 
 void print_table (void)
@@ -252,35 +267,68 @@ void print_table (void)
   }
 }
 
-void get_field_names(void)
+void get_field_names( char* tblname )
 {
   char string_buf[SPRINT_MAX_LEN];
   char *name_p;
+  struct tree *tbl = NULL;
+  int going = 1;
+
+  name_p = string_buf;
+  strcpy(string_buf, "");
+
+  if( tblname )
+    tbl = find_tree_node( tblname, -1 );
+  if( tbl )
+    tbl = tbl->child_list;
+
+  if( tbl ) {
+    root[rootlen++] = tbl->subid;
+    tbl = tbl->child_list;
+  }
+  else
+    root[rootlen++] = 1;
 
-  root[rootlen++] = 1;
   fields = 0;
-  while (1) {
+  while (going) {
     fields++;
-    root[rootlen] = fields;
+    if( tbl ) {
+      if (tbl->access == MIB_ACCESS_NOACCESS) {
+	fields--;
+	tbl = tbl->next_peer;
+	continue;
+      }
+      root[ rootlen ] = tbl->subid;
+      tbl = tbl->next_peer;
+      if (!tbl) going = 0;
+    }
+    else
+      root[rootlen] = fields;
     sprint_objid(string_buf, root, rootlen+1);
     name_p = strrchr(string_buf, '.');
-    if (localdebug) printf("%s %c\n", string_buf, name_p[1]);
-    if ('0' <= name_p[1] && name_p[1] <= '9')
+    if (!name_p) name_p = strrchr(string_buf, ':');
+    if (!name_p) name_p = string_buf;
+    else name_p++;
+    if (localdebug) printf("%s %c\n", string_buf, name_p[0]);
+    if ('0' <= name_p[0] && name_p[0] <= '9') {
+      fields--;
       break;
+    }
     if (fields == 1) column = (struct column *)malloc(sizeof (*column));
     else column = (struct column *)realloc(column, fields*sizeof(*column));
-    column[fields-1].label = strdup(name_p+1);
-    column[fields-1].width = strlen(name_p+1);
+    column[fields-1].label = strdup(name_p);
+    column[fields-1].width = strlen(name_p);
+    column[fields-1].subid = root[ rootlen ];
   }
-  if (fields == 1) {
+  if (fields == 0) {
     fprintf(stderr, "Was that a table? %s\n", string_buf);
     exit(1);
   }
-  fields--;
   *name_p = 0;
   memmove(name, root, rootlen * sizeof(oid));
   name_length = rootlen+1;
   name_p = strrchr(string_buf, '.');
+  if (!name_p) name_p = strrchr(string_buf, ':');
   if (name_p) *name_p = 0;
   table_name = strdup(string_buf);
   if (brief) {
@@ -302,10 +350,9 @@ void get_field_names(void)
   }
 }
 
-void get_table_entries(void)
+void get_table_entries( struct snmp_session *ss )
 {
   int running = 1;
-  struct snmp_session *ss;
   struct snmp_pdu *pdu, *response;
   struct variable_list *vars;
   int   count;
@@ -315,23 +362,21 @@ void get_table_entries(void)
   char  string_buf[SPRINT_MAX_LEN];
   char  *name_p = NULL;
   char  **dp;
+  int end_of_table = 0;
+  int have_current_index;
 
-  SOCK_STARTUP;
-
-  /* open an SNMP session */
-  snmp_synch_setup(&session);
-  ss = snmp_open(&session);
-  if (ss == NULL){
-    snmp_perror("snmptable");
-    SOCK_CLEANUP;
-    exit(1);
-  }
+  /*
+   * TODO:
+   *   1) Deal with multiple index fields
+   *   2) Deal with variable length index fields
+   *   3) optimize to remove a sparse column from get-requests
+   */
 
   while (running) {
     /* create PDU for GETNEXT request and add object name to request */
     pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
     for (i = 1; i <= fields; i++) {
-      name[rootlen] = i;
+      name[rootlen] = column[i-1].subid;
       snmp_add_null_var(pdu, name, name_length);
     }
 
@@ -341,24 +386,6 @@ void get_table_entries(void)
       if (response->errstat == SNMP_ERR_NOERROR){
 	/* check resulting variables */
 	vars = response->variables;
-	if (vars) {
-	  name[rootlen] = 1;
-	  if (vars->name_length <= rootlen ||
-	      memcmp(name, vars->name, (rootlen+1) * sizeof(oid)) != 0) {
-	    /* not part of this subtree */
-	    if (localdebug) {
-	      printf("End of table: ");
-	      print_variable(vars->name, vars->name_length, vars);
-	    }
-	    running = 0;
-	    continue;
-	  }
-	  name_length = vars->name_length;
-	  memcpy(name, vars->name, name_length*sizeof(oid));
-	  sprint_objid(string_buf, vars->name, vars->name_length); 
-	  name_p = strrchr(string_buf, '.');
-	  if (localdebug) printf("Index: %s\n", name_p+1);
-	}
 	entries++;
 	if (entries >= allocated) {
 	  if (allocated == 0) {
@@ -375,32 +402,57 @@ void get_table_entries(void)
 	    if (show_index) indices = (char **)realloc(indices, allocated*sizeof(char *));
 	  }
 	}
-	if (show_index) {
-	  name_p = string_buf + strlen(table_name)+1;
-	  name_p = strchr(name_p, '.')+1;
-	  name_p = strchr(name_p, '.')+1;
-	  indices[entries-1] = strdup(name_p);
-	  i = strlen(name_p);
-	  if (i > index_width) index_width = i;
-	}
 	dp = data+(entries-1)*fields;
 	col = -1;
+	end_of_table = 1; /* assume end of table */
+	have_current_index = 0;
+	name_length = rootlen+1;
 	for (vars = response->variables; vars; vars = vars->next_variable) {
 	  col++;
-	  i = name[rootlen] = vars->name[rootlen];
+	  name[rootlen] = column[col].subid;
 	  if (localdebug) sprint_variable(string_buf, vars->name, vars->name_length, vars);
-	  if (vars->name_length != name_length ||
+	  if( (vars->name_length < name_length) || ((int)vars->name[rootlen] != column[col].subid) ||
 	      memcmp(name, vars->name, name_length * sizeof(oid)) != 0) {
 	    /* not part of this subtree */
 	    if (localdebug) printf("%s => ignored\n", string_buf);
 	    continue;
 	  }
+	  
+	  /* save index off */
+	  if ( ! have_current_index ) {
+	    end_of_table = 0;
+	    have_current_index = 1;
+	    name_length = vars->name_length;
+	    memcpy(name, vars->name, name_length*sizeof(oid));
+	    sprint_objid(string_buf, vars->name, vars->name_length); 
+	    i = vars->name_length - rootlen + 1;
+	    name_p = string_buf + strlen(table_name)+1;
+	    if (localdebug || show_index ) {
+	      name_p = strchr(name_p, '.')+1;
+	    }
+	    if (localdebug) printf("Index: %s\n", name_p);
+	    if (show_index) {
+	      indices[entries-1] = strdup(name_p);
+	      i = strlen(name_p);
+	      if (i > index_width) index_width = i;
+	    }
+	  }
+	  
 	  if (localdebug) printf("%s => taken\n", string_buf);
 	  sprint_value(string_buf, vars->name, vars->name_length, vars);
-	  dp[i-1] = strdup(string_buf);
+	  dp[col] = strdup(string_buf);
 	  i = strlen(string_buf);
 	  if (i > column[col].width) column[col].width = i;
 	}
+	  if( end_of_table ) {
+		--entries;
+	    /* not part of this subtree */
+	    if (localdebug) {
+	      printf("End of table: %s\n", string_buf);
+	    }
+	    running = 0;
+	    continue;
+	  }
       } else {
 	/* error in response, print it */
 	running = 0;
@@ -422,16 +474,13 @@ void get_table_entries(void)
 	}
       }
     } else if (status == STAT_TIMEOUT){
-      fprintf(stderr, "Timeout: No Response from %s\n", session.peername);
+      fprintf(stderr, "Timeout: No Response from %s\n", ss->peername);
       running = 0;
     } else {    /* status == STAT_ERROR */
-      snmp_perror("snmptable");
+      snmp_sess_perror("snmptable", ss);
       running = 0;
     }
     if (response)
       snmp_free_pdu(response);
   }
-  snmp_close(ss);
-
-  SOCK_CLEANUP;
 }
