@@ -60,7 +60,11 @@ SOFTWARE.
 #include <stdio.h>
 #if HAVE_WINSOCK_H
 #include <winsock.h>
-#else
+#endif
+#if HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+#if HAVE_NETDB_H
 #include <netdb.h>
 #endif
 #if HAVE_ARPA_INET_H
@@ -83,7 +87,7 @@ oid objid_snmptrap[]   = {1, 3, 6, 1, 6, 3, 1, 1, 4, 1, 0};
 
 void usage(void)
 {
-    fprintf(stderr,"Usage:\n  snmptrap ");
+    fprintf(stderr,"Usage: snmptrap ");
     snmp_parse_args_usage(stderr);
     fprintf(stderr," [<trap parameters> ...]\n\n");
     snmp_parse_args_descriptions(stderr);
@@ -124,19 +128,17 @@ int main(int argc, char *argv[])
 {
     struct snmp_session session, *ss;
     struct snmp_pdu *pdu, *response;
+    struct sockaddr_in *pduIp;
     oid name[MAX_OID_LEN];
-    int name_length;
+    size_t name_length;
     int	arg;
     int status, inform = 0;
     char *trap = NULL, *specific = NULL, *description = NULL, *agent = NULL;
-#ifdef _DEBUG_MALLOC_INC
-    unsigned long histid1, histid2, orig_size, current_size;
-#endif
 
     /*
-     * usage: snmptrap gateway-name srcParty dstParty trap-type specific-type device-description [ -a agent-addr ]
+     * usage: snmptrap gateway-name trap-type specific-type device-description [ -a agent-addr ]
      */
-    arg = snmp_parse_args(argc, argv, &session);
+    arg = snmp_parse_args(argc, argv, &session, NULL, NULL);
 
     SOCK_STARTUP;
 
@@ -144,19 +146,17 @@ int main(int argc, char *argv[])
     session.callback_magic = NULL;
     if (session.remote_port == SNMP_DEFAULT_REMPORT)
 	session.remote_port = SNMP_TRAP_PORT;
-    snmp_synch_setup(&session);
     ss = snmp_open(&session);
     if (ss == NULL){
-        snmp_perror("snmptrap");
+      /* diagnose snmp_open errors with the input struct snmp_session pointer */
+        snmp_sess_perror("snmptrap", &session);
         SOCK_CLEANUP;
         exit(1);
     }
-#ifdef _DEBUG_MALLOC_INC
-    orig_size = malloc_inuse(&histid1);
-#endif
 
     if (session.version == SNMP_VERSION_1) {
 	pdu = snmp_pdu_create(SNMP_MSG_TRAP);
+	pduIp = (struct sockaddr_in *)&pdu->agent_addr;
 	if (arg == argc) {
 	    fprintf(stderr, "No enterprise oid\n");
 	    usage();
@@ -171,7 +171,7 @@ int main(int argc, char *argv[])
 	else {
 	    name_length = MAX_OID_LEN;
 	    if (!snmp_parse_oid(argv[arg], name, &name_length)) {
-		fprintf (stderr, "Invalid enterprise id: %s\n", argv[arg]);
+		snmp_perror(argv[arg]);
 		usage ();
                 SOCK_CLEANUP;
 		exit (1);
@@ -187,10 +187,11 @@ int main(int argc, char *argv[])
 	    exit (1);
 	}
 	agent = argv [arg];
+	pduIp->sin_family = AF_INET;
 	if (agent != NULL && strlen (agent) != 0)
-	    pdu->agent_addr.sin_addr.s_addr = parse_address(agent);
+	    pduIp->sin_addr.s_addr = parse_address(agent);
 	else
-	    pdu->agent_addr.sin_addr.s_addr = get_myaddr();
+	    pduIp->sin_addr.s_addr = get_myaddr();
 	if (++arg == argc) {
 	    fprintf (stderr, "Missing generic-trap parameter\n");
 	    usage ();
@@ -250,35 +251,43 @@ int main(int argc, char *argv[])
             SOCK_CLEANUP;
 	    exit (1);
 	}
-	snmp_add_var (pdu, objid_snmptrap, sizeof (objid_snmptrap)/sizeof(oid),
-		      'o', argv [arg]);
+	if (snmp_add_var (pdu, objid_snmptrap, sizeof (objid_snmptrap)/sizeof(oid),
+		      'o', argv [arg]) != 0) {
+	    snmp_perror(argv[arg]);
+	    SOCK_CLEANUP;
+	    exit(1);
+	}
     }
     arg++;
 
     while (arg < argc) {
 	arg += 3;
-	if (arg > argc) break;
+	if (arg > argc) {
+	    fprintf(stderr, "%s: Missing type/value for variable\n", argv[arg-3]);
+	    SOCK_CLEANUP;
+	    exit(1);
+	}
 	name_length = MAX_OID_LEN;
 	if (!snmp_parse_oid(argv [arg-3], name, &name_length)) {
-	    fprintf (stderr, "Invalid object identifier: %s\n", argv [arg-3]);
+	    snmp_perror(argv [arg-3]);
             SOCK_CLEANUP;
 	    exit(1);
 	}
-	snmp_add_var (pdu, name, name_length, argv [arg-2][0], argv [arg-1]);
+	if (snmp_add_var (pdu, name, name_length, argv [arg-2][0], argv [arg-1]) != 0) {
+	    snmp_perror(argv[arg-3]);
+	    SOCK_CLEANUP;
+	    exit(1);
+	}
     }
 
     if (inform) status = snmp_synch_response(ss, pdu, &response);
     else status = snmp_send(ss, pdu) == 0;
     if (status) {
-        snmp_perror(inform ? (char*)"snmpinform" : (char*)"snmptrap");
+        snmp_sess_perror(inform ? "snmpinform" : "snmptrap", ss);
+	snmp_free_pdu(pdu);
     }
-    if (inform) snmp_free_pdu(response);
-    else snmp_free_pdu(pdu);
+    else if (inform) snmp_free_pdu(response);
 
-#ifdef _DEBUG_MALLOC_INC
-    current_size = malloc_inuse(&histid2);
-    if (current_size != orig_size) malloc_list(2, histid1, histid2);
-#endif
     snmp_close(ss);
     SOCK_CLEANUP;
     return (0);
