@@ -61,7 +61,8 @@ SOFTWARE.
 #include <stdio.h>
 #if HAVE_WINSOCK_H
 #include <winsock.h>
-#else
+#endif
+#if HAVE_NETDB_H
 #include <netdb.h>
 #endif
 #if HAVE_ARPA_INET_H
@@ -75,16 +76,73 @@ SOFTWARE.
 #include "snmp.h"
 #include "snmp_impl.h"
 #include "system.h"
+#include "default_store.h"
 #include "snmp_parse_args.h"
+#include "getopt.h"
+
+#define DS_WALK_INCLUDE_REQUESTED 1
+#define DS_WALK_PRINT_STATISTICS  2
 
 oid objid_mib[] = {1, 3, 6, 1, 2, 1};
+int numprinted = 0;
 
 void usage(void)
 {
-  fprintf(stderr,"Usage:\n  snmpwalk ");
+  fprintf(stderr,"Usage: snmpwalk ");
   snmp_parse_args_usage(stderr);
   fprintf(stderr," [<objectID>]\n\n");
   snmp_parse_args_descriptions(stderr);
+  fprintf(stderr, "  -C <APPOPTS>  Toggle various application specific behaviour:\n");
+  fprintf(stderr, "\t\t  APPOPTS values:\n");
+  fprintf(stderr,"\t\t      p: Print the number of variables found.\n");
+  fprintf(stderr,"\t\t      i: Include the requested OID in the search range.\n");
+}
+
+void
+snmp_get_and_print(struct snmp_session *ss, oid *theoid, size_t theoid_len) {
+    struct snmp_pdu *pdu, *response;
+    struct variable_list *vars;
+    int status;
+    
+    pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
+    snmp_add_null_var(pdu, theoid, theoid_len);
+
+    status = snmp_synch_response(ss, pdu, &response);
+    if (status == STAT_SUCCESS && response->errstat == SNMP_ERR_NOERROR) {
+        for(vars = response->variables; vars; vars = vars->next_variable) {
+            numprinted++;
+            print_variable(vars->name, vars->name_length, vars);
+        }
+    }
+    if (response)
+        snmp_free_pdu(response);
+}
+
+static void optProc(int argc, char *const *argv, int opt)
+{
+    switch (opt) {
+        case 'C':
+            while (*optarg) {
+                switch (*optarg++) {
+                    case 'i':
+                        ds_toggle_boolean(DS_APPLICATION_ID,
+                                          DS_WALK_INCLUDE_REQUESTED);
+                        break;
+
+                    case 'p':
+                        ds_toggle_boolean(DS_APPLICATION_ID,
+                                          DS_WALK_PRINT_STATISTICS);
+                        break;
+
+                    default:
+                        fprintf(stderr,
+                                "Unknown flag passed to -C: %c\n", *optarg);
+                        usage();
+                        exit(1);
+                }
+            }
+            break;
+    }
 }
 
 int main(int argc, char *argv[])
@@ -92,27 +150,29 @@ int main(int argc, char *argv[])
     struct snmp_session  session, *ss;
     struct snmp_pdu *pdu, *response;
     struct variable_list *vars;
-    int   arg;
-    oid   name[MAX_OID_LEN];
-    int   name_length;
-    oid   root[MAX_OID_LEN];
-    int   rootlen;
-    int   count;
-    int   running;
-    int   status;
-#ifdef _DEBUG_MALLOC_INC
-    unsigned long histid1, histid2, orig_size, current_size;
-#endif
+    int    arg;
+    oid    name[MAX_OID_LEN];
+    size_t name_length;
+    oid    root[MAX_OID_LEN];
+    size_t rootlen;
+    int    count;
+    int    running;
+    int    status;
+
+    ds_register_config(ASN_BOOLEAN, "snmpwalk", "includeRequested",
+                       DS_APPLICATION_ID, DS_WALK_INCLUDE_REQUESTED);
+    ds_register_config(ASN_BOOLEAN, "snmpwalk", "printStatistics",
+                       DS_APPLICATION_ID, DS_WALK_PRINT_STATISTICS);
 
     /* get the common command line arguments */
-    arg = snmp_parse_args(argc, argv, &session);
+    arg = snmp_parse_args(argc, argv, &session, "C:", optProc);
 
     /* get the initial object and subtree */
     if (arg < argc) {
       /* specified on the command line */
       rootlen = MAX_OID_LEN;
       if (snmp_parse_oid(argv[arg], root, &rootlen) == NULL) {
-        fprintf(stderr, "Invalid object identifier: %s\n", argv[arg]);
+        snmp_perror(argv[arg]);
         exit(1);
       }
     } else {
@@ -120,17 +180,14 @@ int main(int argc, char *argv[])
       memmove(root, objid_mib, sizeof(objid_mib));
       rootlen = sizeof(objid_mib) / sizeof(oid);
     }
-#ifdef _DEBUG_MALLOC_INC
-    orig_size = malloc_inuse(&histid1);
-#endif
 
     SOCK_STARTUP;
 
     /* open an SNMP session */
-    snmp_synch_setup(&session);
     ss = snmp_open(&session);
     if (ss == NULL){
-      snmp_perror("snmpwalk");
+      /* diagnose snmp_open errors with the input struct snmp_session pointer */
+      snmp_sess_perror("snmpwalk", &session);
       SOCK_CLEANUP;
       exit(1);
     }
@@ -139,7 +196,12 @@ int main(int argc, char *argv[])
     memmove(name, root, rootlen * sizeof(oid));
     name_length = rootlen;
 
-    running = 1;
+    running = -1;
+
+    if (ds_get_boolean(DS_APPLICATION_ID, DS_WALK_INCLUDE_REQUESTED)) {
+        snmp_get_and_print(ss, root, rootlen);
+    }
+    
     while(running){
       /* create PDU for GETNEXT request and add object name to request */
       pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
@@ -157,6 +219,7 @@ int main(int argc, char *argv[])
               running = 0;
               continue;
             }
+            numprinted++;
             print_variable(vars->name, vars->name_length, vars);
             if ((vars->type != SNMP_ENDOFMIBVIEW) &&
                 (vars->type != SNMP_NOSUCHOBJECT) &&
@@ -193,19 +256,25 @@ int main(int argc, char *argv[])
         fprintf(stderr, "Timeout: No Response from %s\n", session.peername);
         running = 0;
       } else {    /* status == STAT_ERROR */
-        snmp_perror("snmpwalk");
+        snmp_sess_perror("snmpwalk", ss);
         running = 0;
       }
       if (response)
         snmp_free_pdu(response);
     }
+
+    if (numprinted == 0 && status == STAT_SUCCESS) {
+        /* no printed successful results, which may mean we were
+           pointed at an only existing instance.  Attempt a GET, just
+           for get measure. */
+        snmp_get_and_print(ss, root, rootlen);
+    }
     snmp_close(ss);
 
-#ifdef _DEBUG_MALLOC_INC
-    current_size = malloc_inuse(&histid2);
-    if (current_size != orig_size) malloc_list(2, histid1, histid2);
-#endif
+    if (ds_get_boolean(DS_APPLICATION_ID, DS_WALK_PRINT_STATISTICS)) {
+        printf("Variables found: %d\n", numprinted);
+    }
 
     SOCK_CLEANUP;
-    exit (0);
+    return 0;
 }
