@@ -44,6 +44,15 @@
 
 #include <stdio.h>
 #include <sys/types.h>
+#if HAVE_STRING_H
+#include <string.h>
+#else
+#include <strings.h>
+#endif
+#if HAVE_WINSOCK_H
+#include <winsock.h>
+#endif
+
 #include "md5.h"
 
 /* Compile-time declarations of MD5 ``magic constants''.
@@ -94,6 +103,7 @@
 void MDreverse (unsigned int *);
 static void MDblock (MDptr, unsigned int *);
 
+#ifdef SNMP_TESTING_CODE
 /* MDprint(MDp)
 ** Print message digest buffer MDp as 32 hexadecimal digits.
 ** Order is from low-order byte of buffer[0] to high-order byte of buffer[3].
@@ -110,6 +120,7 @@ MDprint(MDptr MDp)
     printf("\n");
     fflush(stdout);
 }
+#endif /* SNMP_TESTING_CODE */
 
 /* MDbegin(MDp)
 ** Initialize message digest buffer MDp. 
@@ -231,6 +242,9 @@ MDblock(MDptr MDp,
     MDp->buffer[1] += B;
     MDp->buffer[2] += C;
     MDp->buffer[3] += D; 
+#ifdef WORDS_BIGENDIAN
+    MDreverse(X);
+#endif
 }
 
 /* MDupdate(MDp,X,count)
@@ -245,8 +259,11 @@ MDblock(MDptr MDp,
 ** count less than 512.  A call with count 0 will be ignored if the
 ** MD has already been terminated (done != 0), so an extra call with count
 ** 0 can be given as a ``courtesy close'' to force termination if desired.
+** Returns : 0 if processing succeeds or was already done;
+**          -1 if processing was already done
+**          -2 if count was too large
 */
-void 
+int 
 MDupdate(MDptr MDp,
 	 unsigned char *X,
 	 unsigned int count)
@@ -257,9 +274,12 @@ MDupdate(MDptr MDp,
     /* return with no error if this is a courtesy close with count
   ** zero and MDp->done is true.
   */
-    if (count == 0 && MDp->done) return;
+    if (count == 0 && MDp->done) return 0;
     /* check to see if MD is already done and report error */
+    if (MDp->done) { return -1; }
+/*
     if (MDp->done) { fprintf(stderr,"\nError: MDupdate MD already done."); return; }
+*/
     /* Add count to MDp->count */
     tmp = count;
     p = MDp->count;
@@ -274,16 +294,22 @@ MDupdate(MDptr MDp,
 	MDblock(MDp,(unsigned int *)X);
     }
     else if (count > 512) /* Check for count too large */
+    return -2;
+/*
     { fprintf(stderr,"\nError: MDupdate called with illegal count value %d.",count);
     return;
     }
+*/
     else /* partial block -- must be last block so finish up */
     { /* Find out how many bytes and residual bits there are */
+        int copycount;
 	byte = count >> 3;
 	bit =  count & 7;
+	copycount = byte; if (bit) copycount++;
 	/* Copy X into XX since we need to modify it */
-	for (i=0;i<=byte;i++)   XX[i] = X[i];
-	for (i=byte+1;i<64;i++) XX[i] = 0;
+	memset(XX,0,sizeof(XX));
+	memcpy(XX,X,copycount);
+
 	/* Add padding '1' bit and low-order zeros in last byte */
 	mask = ((unsigned long)1) << (7 - bit);
 	XX[byte] = (XX[byte] | mask) & ~( mask - 1);
@@ -301,6 +327,120 @@ MDupdate(MDptr MDp,
 	/* Set flag saying we're done with MD computation */
 	MDp->done = 1;
     }
+    return 0;
+}
+
+/* MDchecksum(data, len, MD5): do a checksum on an arbirtrary amount of data */
+int
+MDchecksum(u_char *data, size_t len, u_char *mac, size_t maclen)
+{
+  MDstruct md;
+  MDstruct *MD = &md;
+  int rc = 0;
+  
+  MDbegin(MD);
+  while (len >= 64) {
+    rc = MDupdate(MD, data, 64*8);
+    if (rc) goto check_end;
+    data += 64;
+    len -= 64;
+  }
+  rc = MDupdate(MD, data, len*8);
+  if (rc) goto check_end;
+
+  /* copy the checksum to the outgoing data (all of it that is requested). */
+  MDget(MD, mac, maclen);
+
+check_end:
+  memset(&md,0,sizeof(md));
+  return rc;
+}
+
+
+/* MDsign(data, len, MD5): do a checksum on an arbirtrary amount
+   of data, and prepended with a secret in the standard fashion */
+int
+MDsign(u_char *data, size_t len, u_char *mac, size_t maclen,
+       u_char *secret, size_t secretlen)
+{
+#define HASHKEYLEN 64
+
+  MDstruct MD;
+  u_char   K1[HASHKEYLEN];
+  u_char   K2[HASHKEYLEN];
+  u_char   extendedAuthKey[HASHKEYLEN];
+  u_char   buf[HASHKEYLEN];
+  size_t   i;
+  u_char  *cp;
+  int      rc = 0;
+
+/*
+  memset(K1,0,HASHKEYLEN);
+  memset(K2,0,HASHKEYLEN);
+  memset(buf,0,HASHKEYLEN);
+  memset(extendedAuthKey,0,HASHKEYLEN);
+*/
+
+  if (secretlen != 16 || secret == NULL || mac == NULL || data == NULL ||
+    len <= 0 || maclen <= 0) {
+/*    DEBUGMSGTL(("md5","MD5 signing not properly initialized")); */
+    return -1;
+  }
+  
+  memset(extendedAuthKey, 0, HASHKEYLEN);
+  memcpy(extendedAuthKey, secret, secretlen);
+  for(i = 0; i < HASHKEYLEN; i++) {
+    K1[i] = extendedAuthKey[i] ^ 0x36;
+    K2[i] = extendedAuthKey[i] ^ 0x5c;
+  }
+
+  MDbegin(&MD);
+  rc = MDupdate(&MD, K1, HASHKEYLEN*8);
+  if (rc) goto update_end;
+
+  i = len;
+  cp = data;
+  while (i >= 64) {
+    rc = MDupdate(&MD, cp, 64*8);
+	if (rc) goto update_end;
+    cp += 64;
+    i -= 64;
+  }
+
+  rc = MDupdate(&MD, cp, i*8);
+  if (rc) goto update_end;
+
+  memset(buf,0,HASHKEYLEN);
+  MDget(&MD, buf, HASHKEYLEN);
+
+  MDbegin(&MD);
+  rc = MDupdate(&MD, K2, HASHKEYLEN*8);
+  if (rc) goto update_end;
+  rc = MDupdate(&MD, buf, 16*8);
+  if (rc) goto update_end;
+  
+  /* copy the sign checksum to the outgoing pointer */
+  MDget(&MD, mac, maclen);
+
+update_end:
+  memset(buf, 0, HASHKEYLEN);
+  memset(K1, 0, HASHKEYLEN);
+  memset(K2, 0, HASHKEYLEN);
+  memset(extendedAuthKey, 0, HASHKEYLEN);
+  memset(&MD, 0, sizeof(MD));
+
+  return rc;
+}
+
+void
+MDget(MDstruct *MD, u_char *buf, size_t buflen)
+{
+  int i, j;
+  
+  /* copy the checksum to the outgoing data (all of it that is requested). */
+  for(i=0; i < 4 && i*4 < (int)buflen; i++)
+    for(j=0; j < 4 && i*4+j < (int)buflen; j++)
+      buf[i*4+j] = (MD->buffer[i] >> j*8) & 0xff;
 }
 
 /* 
