@@ -37,8 +37,14 @@ SOFTWARE.
 #endif
 
 #include <stdio.h>
+#include <stdlib.h>
 
 #include <sys/types.h>
+#ifdef HAVE_STRING_H
+#include <string.h>
+#else
+#include <strings.h>
+#endif
 #if HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
@@ -52,6 +58,10 @@ SOFTWARE.
 #define NULL 0
 #endif
 
+#if HAVE_DMALLOC_H
+#include <dmalloc.h>
+#endif
+
 #ifdef vms
 #include <in.h>
 #endif
@@ -60,40 +70,49 @@ SOFTWARE.
 #include "snmp.h"
 #include "snmp_api.h"
 #include "snmp_impl.h"
+#include "snmp_logging.h"
 #include "mib.h"
 
 void
-xdump(u_char *cp,
-      int length,
-      char *prefix)
+xdump(const u_char *cp,
+      size_t length,
+      const char *prefix)
 {
     int col, count;
+    char *buffer;
 
-    count = 0;
-    while(count < length){
-	printf("%s", prefix);
-	for(col = 0; count + col < length && col < 16; col++){
-	    if (col == 0) printf ("%.4d: ", count);
-	    else if (col % 4 == 0) printf(" ");
-	    printf("%02X ", cp[count + col]);
-	}
-	while(col++ < 16){	/* pad end of buffer with zeros */
-	    if (col % 4 == 0)
-		printf(" ");
-	    printf("   ");
-	}
-	printf("  ");
-	for(col = 0; count + col < length && col < 16; col++){
-	    if (isprint(cp[count + col]))
-		printf("%c", cp[count + col]);
-	    else
-		printf(".");
-	}
-	printf("\n");
-	count += col;
+    buffer=(char *)malloc(strlen(prefix)+80);
+    if (!buffer) {
+      snmp_log(LOG_NOTICE, "xdump: malloc failed. packet-dump skipped\n");
+      return;
     }
 
-}
+    count = 0;
+    while(count < (int)length){
+	strcpy(buffer, prefix);
+        sprintf (buffer+strlen(buffer), "%.4d: ", count);
+
+	for(col = 0; ((count + col) < (int)length) && col < 16; col++){
+	    sprintf(buffer+strlen(buffer), "%02X ", cp[count + col]);
+            if (col % 4 == 3) strcat(buffer, " "); 
+	}
+        for(;col < 16;col++){   /* pad end of buffer with zeros */
+            strcat(buffer, "   ");
+            if (col % 4 == 3) strcat(buffer, " ");
+	}
+	strcat(buffer, "  ");
+	for(col = 0; ((count + col) < (int)length) && col < 16; col++){
+            buffer[col+60]=isprint(cp[count+col])?cp[count+col]:'.';
+	}
+        buffer[col+60]='\n';
+        buffer[col+60+1]=0;
+        snmp_log(LOG_DEBUG, "%s", buffer);
+	count += col;
+    }
+    snmp_log(LOG_DEBUG, "\n");
+    free(buffer);
+
+}  /* end xdump() */
 
 /* 
    u_char * snmp_parse_var_op(
@@ -109,23 +128,22 @@ xdump(u_char *cp,
 u_char *
 snmp_parse_var_op(u_char *data,
 		  oid *var_name,
-		  int *var_name_len,
+		  size_t *var_name_len,
 		  u_char *var_val_type,
-		  int *var_val_len,
+		  size_t *var_val_len,
 		  u_char **var_val,
-		  int *listlength)
+		  size_t *listlength)
 {
     u_char	    var_op_type;
-    int		    var_op_len = *listlength;
+    size_t		    var_op_len = *listlength;
     u_char	    *var_op_start = data;
 
-    data = asn_parse_header(data, &var_op_len, &var_op_type);
+    data = asn_parse_sequence(data, &var_op_len, &var_op_type,
+			(ASN_SEQUENCE | ASN_CONSTRUCTOR), "var_op");
     if (data == NULL){
-	ERROR_MSG("No header for variable");
+    	/* msg detail is set */
 	return NULL;
     }
-    if (var_op_type != (u_char)(ASN_SEQUENCE | ASN_CONSTRUCTOR))
-	return NULL;
     data = asn_parse_objid(data, &var_op_len, &var_op_type, var_name, var_name_len);
     if (data == NULL){
 	ERROR_MSG("No OID for variable");
@@ -140,6 +158,7 @@ snmp_parse_var_op(u_char *data,
 	ERROR_MSG("No header for value");
 	return NULL;
     }
+    /* XXX no check for type! */
     *var_val_len = var_op_len;
     data += var_op_len;
     *listlength -= (int)(data - var_op_start);
@@ -161,14 +180,14 @@ snmp_parse_var_op(u_char *data,
 u_char *
 snmp_build_var_op(u_char *data,
 		  oid *var_name,
-		  int *var_name_len,
+		  size_t *var_name_len,
 		  u_char var_val_type,
-		  int var_val_len,
+		  size_t var_val_len,
 		  u_char *var_val,
-		  int *listlength)
+		  size_t *listlength)
 {
-    int		    dummyLen, headerLen;
-    u_char	    *dataPtr;
+    size_t     dummyLen, headerLen;
+    u_char    *dataPtr;
 
     dummyLen = *listlength;
     dataPtr = data;
@@ -176,14 +195,13 @@ snmp_build_var_op(u_char *data,
     data = asn_build_sequence(data, &dummyLen,
 			      (u_char)(ASN_SEQUENCE | ASN_CONSTRUCTOR), 0);
     if (data == NULL){
-	ERROR_MSG("Can't build sequence header");
 	return NULL;
     }
 #endif
+    if (dummyLen < 4)
+	return NULL;
     data += 4;
     dummyLen -=4;
-    if (dummyLen < 0)
-	return NULL;
 
     headerLen = data - dataPtr;
     *listlength -= headerLen;
