@@ -1,7 +1,3 @@
- 
-#ifndef SNMP_CLIENT_H
-#define SNMP_CLIENT_H
-
 /*
  * snmp_client.h
  */
@@ -26,6 +22,14 @@ WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 ******************************************************************/
+ 
+#ifndef SNMP_CLIENT_H
+#define SNMP_CLIENT_H
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 
 struct snmp_pdu;
 struct snmp_session;
@@ -42,19 +46,28 @@ struct synch_state {
     struct snmp_pdu *pdu;
 };
 
-struct variable_list* snmp_add_null_var (struct snmp_pdu *, oid *, int);
+int snmp_set_var_value(struct variable_list *, u_char *, size_t);
+int snmp_set_var_objid (struct variable_list *vp,
+                    const oid *objid, size_t name_length);
+struct variable_list* snmp_add_null_var (struct snmp_pdu *, oid *, size_t);
 struct snmp_pdu	*snmp_pdu_create (int);
 struct snmp_pdu *snmp_fix_pdu (struct snmp_pdu *, int);
 struct snmp_pdu *snmp_clone_pdu (struct snmp_pdu *);
-char *snmp_errstring (int);
-void snmp_synch_reset (struct snmp_session *);
-void snmp_synch_setup (struct snmp_session *);
-int snmp_synch_response (struct snmp_session *, struct snmp_pdu *, struct snmp_pdu **);
+struct snmp_pdu *snmp_split_pdu (struct snmp_pdu *, int skipCount,
+                                 int copyCount);
 
-int ms_party_init (in_addr_t, oid *, int *, oid *, int *, oid *, int *);
+unsigned long snmp_varbind_len(struct snmp_pdu * pdu);
+int snmp_clone_var(struct variable_list *, struct variable_list *);
+const char *snmp_errstring (int);
+int snmp_synch_response (struct snmp_session *, struct snmp_pdu *, struct snmp_pdu **);
+int snmp_synch_response_cb (struct snmp_session *, struct snmp_pdu *, struct snmp_pdu **, snmp_callback);
+int snmp_clone_mem(void **, void *, unsigned);
 
 /* single session API - see snmp_api.h for full details */
 int snmp_sess_synch_response (void *, struct snmp_pdu *, struct snmp_pdu **);
  
-#endif /* SNMP_CLIENT_H */
+#ifdef __cplusplus
+}
+#endif
 
+#endif /* SNMP_CLIENT_H */
