@@ -1,3 +1,6 @@
+/*
+ * system.c
+ */
 /***********************************************************
         Copyright 1992 by Carnegie Mellon University
 
@@ -24,12 +27,15 @@ SOFTWARE.
  */
 #include <config.h>
 #include <stdio.h>
+#include <ctype.h>
+
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+
 #if TIME_WITH_SYS_TIME
 # ifdef WIN32
 #  include <sys/timeb.h>
@@ -44,33 +50,43 @@ SOFTWARE.
 #  include <time.h>
 # endif
 #endif
+
 #include <sys/types.h>
+
 #if HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
+
 #if HAVE_WINSOCK_H
 #include <winsock.h>
-#else
+#endif
+#if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#if HAVE_NET_IF_H
 #include <net/if.h>
 #endif
+
 #if HAVE_SYS_SOCKIO_H
 #include <sys/sockio.h>
 #endif
+
 #if HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
-#ifndef WIN32
+
 #ifdef HAVE_NLIST_H
 #include <nlist.h>
 #endif
-#endif
+
 #if HAVE_SYS_FILE_H
 #include <sys/file.h>
 #endif
+
 #if HAVE_KSTAT_H
 #include <kstat.h>
 #endif
+
 #if HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
@@ -84,31 +100,46 @@ SOFTWARE.
 #include <strings.h>
 #endif
 
+#if HAVE_DMALLOC_H
+#include <dmalloc.h>
+#endif
+
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
+
 #include "asn1.h"
 #include "snmp_api.h"
+#include "tools.h"
 #include "system.h"
+#include "snmp_logging.h"
 
 #define NUM_NETWORKS    32   /* max number of interfaces to check */
 
 #ifndef IFF_LOOPBACK
-#define IFF_LOOPBACK 0
+#	define IFF_LOOPBACK 0
 #endif
+
 #define LOOPBACK    0x7f000001
 
+
+
 /* ********************************************* */
-#ifdef WIN32
-#define WIN32_LEAN_AND_MEAN
-#define WIN32IO_IS_STDIO
-#include <tchar.h>
-#include <windows.h>
-#include <sys/stat.h>
-#define PATHLEN 1024
+#ifdef							WIN32
+#	define WIN32_LEAN_AND_MEAN
+#	define WIN32IO_IS_STDIO
+#	define PATHLEN	1024
+
+#	include <tchar.h>
+#	include <windows.h>
+
+
 /* The idea here is to read all the directory names into a string table
  * (separated by nulls) and when one of the other dir functions is called
  * return the pointer to the current file name.
  */
 DIR *
-opendir(char *filename)
+opendir(const char *filename)
 {
     DIR            *p;
     long            len;
@@ -125,10 +156,10 @@ opendir(char *filename)
     }
 
     /* get the file system characteristics */
-/*  if(GetFullPathName(filename, MAX_PATH, root, &dummy)) {
+/*  if(GetFullPathName(filename, SNMP_MAXPATH, root, &dummy)) {
  *	if(dummy = strchr(root, '\\'))
  *	    *++dummy = '\0';
- *	if(GetVolumeInformation(root, volname, MAX_PATH, &serial,
+ *	if(GetVolumeInformation(root, volname, SNMP_MAXPATH, &serial,
  *				&maxname, &flags, 0, 0)) {
  *	    downcase = !(flags & FS_CASE_IS_PRESERVED);
  *	}
@@ -141,7 +172,7 @@ opendir(char *filename)
     /* Create the search pattern */
     strcpy(scanname, filename);
 
-    if(index("/\\", *(scanname + strlen(scanname) - 1)) == NULL)
+    if(strchr("/\\", *(scanname + strlen(scanname) - 1)) == NULL)
 	strcat(scanname, "/*");
     else
 	strcat(scanname, "*");
@@ -162,7 +193,7 @@ opendir(char *filename)
      * the filenames that we find.
      */
     idx = strlen(FindData.cFileName)+1;
-    p->start = (char*)malloc(idx * sizeof(char));
+    p->start = (char*)malloc(idx);
     /* New(1304, p->start, idx, char);*/
     if(p->start == NULL) {
 		free(p);
@@ -172,7 +203,7 @@ opendir(char *filename)
 /*  if(downcase)
  *	strlwr(p->start);
  */
-    p->nfiles++;
+    p->nfiles = 0;
 
     /* loop finding all the files that match the wildcard
      * (which should be all of them in this directory!).
@@ -184,8 +215,7 @@ opendir(char *filename)
 	/* bump the string table size by enough for the
 	 * new name and it's null terminator
 	 */
-	p->start = (char*)realloc((void*)p->start,
-			idx+len+1 * sizeof(char));
+	p->start = (char*)realloc((void*)p->start, idx+len+1);
 	/* Renew(p->start, idx+len+1, char);*/
 	if(p->start == NULL) {
 		free(p);
@@ -256,7 +286,7 @@ int gettimeofday(struct timeval *tv,
     tv->tv_sec = timebuffer.time;
     return(0);
 }
-#endif
+#endif	/* !HAVE_GETTIMEOFDAY */
 
 in_addr_t get_myaddr(void)
 {
@@ -269,7 +299,7 @@ in_addr_t get_myaddr(void)
 
   in_addr.sin_addr.s_addr = INADDR_ANY;
 
-  result = gethostname(local_host, 130);
+  result = gethostname(local_host, sizeof(local_host));
   if (result == 0)
   {
 	lpstHostent = gethostbyname((LPSTR)local_host);
@@ -332,7 +362,13 @@ void winsock_cleanup (void)
    WSACleanup();
 }
 
-#else
+#else							/* ! WIN32 */
+/*******************************************************************/
+
+/*
+ * XXX	What if we have multiple addresses?
+ * XXX	Could it be computed once then cached?
+ */
 in_addr_t get_myaddr (void)
 {
     int sd;
@@ -358,7 +394,9 @@ in_addr_t get_myaddr (void)
             continue;
         in_addr = (struct sockaddr_in *)&ifrp->ifr_addr;
         if ((ifreq.ifr_flags & IFF_UP)
+#ifdef IFF_RUNNING
           && (ifreq.ifr_flags & IFF_RUNNING)
+#endif /* IFF_RUNNING */
           && !(ifreq.ifr_flags & IFF_LOOPBACK)
           && in_addr->sin_addr.s_addr != LOOPBACK){
 #ifdef SYS_IOCTL_H_HAS_SIOCGIFADDR
@@ -374,25 +412,44 @@ in_addr_t get_myaddr (void)
     return 0;
 }
 
-#ifndef rtems
+
+#if !defined(solaris2) && !defined(linux) && !defined(cygwin)
 /*
- * Returns uptime in centiseconds(!).
+ * Returns boottime in centiseconds(!).
+ *	Caches this for future use.
  */
-long get_uptime (void)
+long get_boottime (void)
 {
-#if defined(bsdlike) && !defined(solaris2) && !defined(linux)
-    struct timeval boottime, now, diff;
-#ifndef CAN_USE_SYSCTL
+    static long boottime_csecs = 0;
+    struct timeval boottime;
+#ifdef	CAN_USE_SYSCTL
+    int	    mib[2];
+    size_t  len;
+#else
     int kmem;
     static struct nlist nl[] = {
 #if !defined(hpux)
 	    { (char*)"_boottime" },
 #else
-	    { "boottime" },
+	    { (char*)"boottime" },
 #endif
 	    { (char*)"" }
 	};
+#endif
 
+
+    if ( boottime_csecs != 0 )
+	return( boottime_csecs );
+
+#ifdef CAN_USE_SYSCTL
+    mib[0] = CTL_KERN;
+    mib[1] = KERN_BOOTTIME;
+
+    len = sizeof(boottime);
+
+    sysctl(mib, 2, &boottime, &len, NULL, NULL);
+    boottime_csecs = (boottime.tv_sec * 100) + (boottime.tv_usec / 10000);
+#else						/* CAN_USE_SYSCTL */
     if ((kmem = open("/dev/kmem", 0)) < 0)
 	return 0;
     nlist(KERNEL_LOC, nl);
@@ -404,47 +461,51 @@ long get_uptime (void)
     lseek(kmem, (long)nl[0].n_value, L_SET);
     read(kmem, &boottime, sizeof(boottime));
     close(kmem);
-#else /* CAN_USE_SYSCTL */
-    int                 mib[2];
-    size_t              len;
+    boottime_csecs = (boottime.tv_sec * 100) + (boottime.tv_usec / 10000);
+#endif						/* CAN_USE_SYSCTL */
 
-    mib[0] = CTL_KERN;
-    mib[1] = KERN_BOOTTIME;
+    return( boottime_csecs );
+}
+#endif
 
-    len = sizeof(boottime);
-
-    sysctl(mib, 2, &boottime, &len, NULL, NULL);
-#endif /* CAN_USE_SYSCTL */
+/*
+ * Returns uptime in centiseconds(!).
+ */
+long get_uptime (void)
+{
+#if !defined(solaris2) && !defined(linux) && !defined(cygwin)
+    struct timeval now;
+    long boottime_csecs, nowtime_csecs;
 
+    boottime_csecs = get_boottime();
+    if (boottime_csecs == 0)
+	return 0;
     gettimeofday(&now,(struct timezone *)0);
+    nowtime_csecs = (now.tv_sec * 100) + (now.tv_usec / 10000);
 
-    now.tv_sec--;
-    now.tv_usec += 1000000L;
-    diff.tv_sec = now.tv_sec - boottime.tv_sec;
-    diff.tv_usec = now.tv_usec - boottime.tv_usec;
-    if (diff.tv_usec > 1000000L){
-	diff.tv_usec -= 1000000L;
-	diff.tv_sec++;
-    }
-    return ((diff.tv_sec * 100) + (diff.tv_usec / 10000));
-#endif /* bsdlike */
+    return (nowtime_csecs - boottime_csecs);
+#endif
 
 #ifdef solaris2
     kstat_ctl_t *ksc = kstat_open();
     kstat_t *ks;
     kid_t kid;
     kstat_named_t *named;
-    u_long lbolt;
+    u_long lbolt = 0;
 
-    if (ksc == NULL) return 0;
-    ks = kstat_lookup (ksc, "unix", -1, "system_misc");
-    if (ks == NULL) return 0;
-    kid = kstat_read (ksc, ks, NULL);
-    if (kid == -1) return 0;
-    named = kstat_data_lookup(ks, "lbolt");
-    if (named == NULL) return 0;
-    lbolt = named->value.ul;
-    kstat_close(ksc);
+    if (ksc) {
+	ks = kstat_lookup (ksc, "unix", -1, "system_misc");
+	if (ks) {
+	    kid = kstat_read (ksc, ks, NULL);
+	    if (kid != -1) {
+		named = kstat_data_lookup(ks, "lbolt");
+		if (named) {
+		    lbolt = named->value.ul;
+		}
+	    }
+	}
+	kstat_close(ksc);
+    }
     return lbolt;
 #endif /* solaris2 */
 
@@ -458,15 +519,58 @@ long get_uptime (void)
    }
    return uptim;
 #endif /* linux */
-
    return (0); /* not implemented */
 }
-#endif rtems
-#endif
+
+#endif							/* ! WIN32 */
+/*******************************************************************/
+
+#ifndef HAVE_STRNCASECMP
+
+/* test for NULL pointers before and NULL characters after
+ * comparing possibly non-NULL strings.
+ * WARNING: This function does NOT check for array overflow.
+ */
+int strncasecmp(const char *s1, const char *s2, size_t nch)
+{
+    size_t ii;
+    int res = -1;
+
+    if (!s1) {
+        if (!s2)  return 0;
+        return (-1);
+    }
+    if (!s2)
+        return (1);
+
+    for (ii=0; (ii < nch) && (*s1 != '\0')
+         && (*s2 != '\0'); ii++, *s1++, *s2++)
+    {
+        res = (int) (tolower(*s1) - tolower(*s2));
+        if (res != 0) break;
+    }
+
+    if (! *s1) {
+        if (! *s2)  return 0;
+        return (-1);
+    }
+    if (! *s2)
+        return (1);
+
+    return (res);
+}
+
+int strcasecmp(const char *s1, const char *s2)
+{
+    return strncasecmp(s1, s2, 1000000);
+}
+
+#endif /* HAVE_STRNCASECMP */
+
 
 #ifndef HAVE_STRDUP
 char *
-strdup(char *src)
+strdup(const char *src)
 {
     int len;
     char *dst;
@@ -480,8 +584,8 @@ strdup(char *src)
 #endif	/* HAVE_STRDUP */
 
 #ifndef HAVE_SETENV
-int setenv(char *name,
-	   char *value,
+int setenv(const char *name,
+	   const char *value,
 	   int overwrite)
 {
     char *cp;
@@ -497,3 +601,99 @@ int setenv(char *name,
     return ret;
 }
 #endif /* HAVE_SETENV */
+
+int
+calculate_time_diff(struct timeval *now, struct timeval *then)
+{
+  struct timeval tmp, diff;
+  memcpy(&tmp, now, sizeof(struct timeval));
+  tmp.tv_sec--;
+  tmp.tv_usec += 1000000L;
+  diff.tv_sec = tmp.tv_sec - then->tv_sec;
+  diff.tv_usec = tmp.tv_usec - then->tv_usec;
+  if (diff.tv_usec > 1000000L){
+    diff.tv_usec -= 1000000L;
+    diff.tv_sec++;
+  }
+  return ((diff.tv_sec * 100) + (diff.tv_usec / 10000));
+}
+
+#ifndef HAVE_STRCASESTR
+/*
+ * only glibc2 has this.
+ */
+char *strcasestr(const char *haystack, const char *needle)
+{
+    const char *cp1=haystack, *cp2=needle;
+    const char *cx;
+    int tstch1, tstch2;
+
+    /* printf("looking for '%s' in '%s'\n", needle, haystack); */
+    if (cp1 && cp2 && *cp1 && *cp2)
+        for (cp1=haystack, cp2=needle; *cp1; ) {
+            cx = cp1; cp2 = needle;
+            do {
+                /* printf("T'%c' ", *cp1); */
+                if (! *cp2) { /* found the needle */
+                    /* printf("\nfound '%s' in '%s'\n", needle, cx); */
+                    return (char *)cx;
+                }
+                if (! *cp1)
+                    break;
+
+                tstch1 = toupper(*cp1);
+                tstch2 = toupper(*cp2);
+                if (tstch1 != tstch2)
+                    break;
+                /* printf("M'%c' ", *cp1); */
+                cp1++; cp2++;
+            }
+            while (1);
+            if (*cp1)
+                cp1++;
+        }
+    /* printf("\n"); */
+    if (cp1 && *cp1)
+        return (char *)cp1;
+
+    return NULL;
+}
+#endif
+
+int
+mkdirhier(const char *pathname, mode_t mode, int skiplast) {
+    struct stat     sbuf;
+    char *ourcopy = strdup(pathname);
+    char *entry;
+    char buf[SNMP_MAXPATH];
+
+    entry = strtok( ourcopy, "/" );
+
+    buf[0] = '\0';
+    /* check to see if filename is a directory */
+    while ( entry ) {
+        strcat(buf,"/");
+        strcat(buf, entry);
+        entry = strtok( NULL, "/");
+        if (entry == NULL && skiplast)
+            break;
+        if (stat(buf, &sbuf) < 0) {
+            /* DNE, make it */
+            snmp_log(LOG_INFO, "Creating directory: %s\n", buf);
+#ifdef WIN32
+	    CreateDirectory(buf, NULL);
+#else
+            mkdir(buf, mode);
+#endif
+        } else {
+            /* exists, is it a file? */
+            if ((sbuf.st_mode & S_IFDIR) == 0) {
+                /* ack! can't make a directory on top of a file */
+                free(ourcopy);
+                return SNMPERR_GENERR;
+            }
+        }
+    }
+    free(ourcopy);
+    return SNMPERR_SUCCESS;
+}
