@@ -1,5 +1,8 @@
 /* config.h:  a general config file */
 
+/* UC-Davis' IANA-assigned enterprise number is 2021 */
+#define ENTERPRISE_NUMBER 2021
+
 /* don't change these values! */
 #define SNMPV1      0xAAAA       /* readable by anyone */
 #define SNMPV2ANY   0xA000       /* V2 Any type (includes NoAuth) */
@@ -25,11 +28,15 @@
    and are not using the UC-Davis SNMP library. */
 #define UCD_SNMP_LIBRARY 1
 
-/* comment the next line if you do not want SNMPv2 party-based auth. */
-#define USE_V2PARTY_PROTOCOL 1
-
+/* define if you want to compile support for both authentication and
+   privacy support. */
+#define SCAPI_AUTHPRIV 1 
+ 
+/* define if you are using the internal MD5 code */
+#define USE_INTERNAL_MD5 1 
+ 
 /* add in recent CMU library extensions (not complete) */
-#undef CMU_COMPATIBLE
+#define CMU_COMPATIBLE 1
 
 /* should "--" comments in mibs be a comment till the end of the line
    or also until another "--", the latter being the technically
@@ -68,7 +75,11 @@
 #undef pid_t
 
 /* Define as the return type of signal handlers (int or void).  */
-#undef RETSIGTYPE
+#define RETSIGTYPE void
+
+/* Define if you have raise() instead of alarm() */
+
+#define HAVE_RAISE 1
 
 /* If using the C implementation of alloca, define if you know the
    direction of stack growth for your system; otherwise it will be
@@ -97,6 +108,7 @@
 
 #define SNMPLIBPATH "\\USR\\LIB"
 #define SNMPSHAREPATH "\\USR"
+#define SNMPCONFPATH "\\USR"
 
 /* SNMPPATH contains (more) important files */
 
@@ -107,14 +119,20 @@
 
 #undef LOGFILE
 
+/* PERSISTENT_DIRECTORY: If defined, the library is capabile of saving
+   persisant information to this directory in the form of configuration
+   lines: PERSISTENT_DIRECTORY/NAME.persistent.conf */
+#define PERSISTENT_DIRECTORY "\\USR\\SNMP"
+
+#define MAX_PERSISTENT_BACKUPS 10
 /* default system contact */
-#undef SYS_CONTACT
+#define SYS_CONTACT "unknown"
 
 /* system location */
-#undef SYS_LOC
+#define SYS_LOC "unknown"
 
 /* location of UNIX kernel */
-#define KERNEL_LOC "/vmunix"
+#define KERNEL_LOC "unknown"
 
 /* location of mount table list */
 #define ETC_MNTTAB "/etc/mnttab"
@@ -136,18 +154,27 @@
 /* Where is the uname command */
 #define UNAMEPROG "/bin/uname"
 
+/* testing code sections. */
+/* #undef SNMP_TESTING_CODE */ 
+
 /* If you don't have root access don't exit upon kmem errors */
 #undef NO_ROOT_ACCESS
 
 /* Define if you have the bcopy function.  */
 #undef HAVE_BCOPY
 
+/* Define if you have the gethostbyname function.  */
+#define HAVE_GETHOSTBYNAME 1
+
 /* Define if you have the gethostname function.  */
-#undef HAVE_GETHOSTNAME
+#define HAVE_GETHOSTNAME 1
 
 /* Define if you have the getloadavg function.  */
 #undef HAVE_GETLOADAVG
 
+/* Define if you have the getservbyname function.  */
+#define HAVE_GETSERVBYNAME 1
+
 /* Define if you have the setenv function.  */
 #undef HAVE_SETENV
 
@@ -208,15 +235,15 @@
 /* Define if you have the strchr function.  */
 #define HAVE_STRCHR 1
 
-/* Define if you do not have strcasecmp */
-#define strcasecmp _stricmp
-
 /* use win32 strdup */
 #define strdup _strdup
 
 /* Define if you have the strtol function.  */
 #define HAVE_STRTOL 1
 
+/* Define if you have the tcgetattr function.  */
+/* #define HAVE_TCGETATTR 1 */
+
 /* Define if you have the uname function.  */
 #undef HAVE_UNAME
 
@@ -310,6 +337,9 @@
 /* Define if you have the <sgtty.h> header file.  */
 #undef HAVE_SGTTY_H
 
+/* Define if you have the <stdarg.h> header file.  */
+#define HAVE_STDARG_H 1
+
 /* Define if you have the <stdlib.h> header file.  */
 #define HAVE_STDLIB_H 1
 
@@ -337,6 +367,9 @@
 /* Define if you have the <sys/fs.h> header file.  */
 #undef HAVE_SYS_FS_H
 
+/* Define if you have the <sys/stat.h> header file.  */
+#define HAVE_SYS_STAT_H 1
+
 /* Define if you have the <sys/hashing.h> header file.  */
 #undef HAVE_SYS_HASHING_H
 
@@ -433,6 +466,9 @@
 /* Define if you have the <xti.h> header file.  */
 #undef HAVE_XTI_H
 
+/* Define if you have the crypto library (-lcrypto).  */
+#undef HAVE_LIBCRYPTO
+
 /* Define if you have the elf library (-lelf).  */
 #undef HAVE_LIBELF
 
@@ -490,7 +526,10 @@
 
 /* type check for in_addr_t */
 #define in_addr_t u_long
-
+ 
+/* define if your compiler (processor) defines __FUNCTION__ for you */
+#undef HAVE_CPP_UNDERBAR_FUNCTION_DEFINED
+ 
 /* mib pointer to the top of the extensible tree.  This has been
  assigned to UCDavis by the iana group.  Optionally, point this to the
  location in the tree your company/organization has been allocated. */
@@ -596,7 +635,7 @@
 #ifdef netbsd1
 #define OSTYPE NETBSD1ID
 #endif
-#if defined(freebsd2) || defined(freebsd3)
+#ifdef freebsd2
 #define OSTYPE FREEBSDID
 #endif
 #if defined(irix6) || defined(irix5)
@@ -739,9 +778,9 @@
 
 /* Not-to-be-compiled macros for use by configure only */
 #define config_require(x)
-#define config_load_mib(x,y,z)
-  
-/*#include <mib_module_config.h> */
+#define config_arch_require(x,y)
+#define config_parse_dot_conf(w,x,y,z)
+#define config_add_mib(x)
 
 #ifndef DONT_INC_STRUCTS
 /*#include "agent/mibgroup/struct.h" */
@@ -754,10 +793,21 @@
 #endif
 
 #ifdef WIN32
+
+#define IN_UCD_SNMP_SOURCE 1
+
+int strcasecmp(const char *s1, const char *s2);
+
 #define ENV_SEPARATOR ";"
 #define ENV_SEPARATOR_CHAR ';'
+
 #else
+
 #define ENV_SEPARATOR ":"
 #define ENV_SEPARATOR_CHAR ':'
+
 #endif
 
+typedef int mode_t;
+
+#define AGENT_DIRECTORY_MODE 0700
